      use const_and_precisions, only : wp_
      use graydata_flags, only : ipass,igrad
      implicit none
c local variables
      real(wp_) :: p0mw1
c common/external functions/variables
      integer :: ierr,index_rt
      real(wp_) :: sox,p0mw,powrfl,taumn,taumx,pabstot,currtot,
     . pabstott,currtott 
c
      common/ierr/ierr
      common/mode/sox
      common/p0/p0mw
      common/powrfl/powrfl
      common/index_rt/index_rt
      common/taumnx/taumn,taumx,pabstot,currtot
c      
c     read data plus initialization
c
      index_rt=1
      call prfile
      call paraminit
      call read_data
      call vectinit
      if(igrad.eq.0) call ic_rt
      if(igrad.gt.0) call ic_gb
      if(ierr.gt.0) then
        print*,'    IERR = ', ierr
        stop
      end if
c
c     beam/ray propagation
      call gray_integration
c
c     postprocessing
      call after_gray_integration
c 
      pabstott=pabstot
      currtott=currtot
c
      if (ipass.gt.1) then
c     second pass into plasma
        p0mw1=p0mw
        igrad=0
c
        index_rt=2
        p0mw=p0mw1*powrfl
        call prfile
        call vectinit2
        call paraminit
        call ic_rt2
        call gray_integration
        call after_gray_integration
        pabstott=pabstott+pabstot
        currtott=currtott+currtot
c      
        index_rt=3
        sox=-sox
        p0mw=p0mw1*(1.0_wp_-powrfl)
        call prfile
        call vectinit2
        call paraminit
        call ic_rt2
        call gray_integration
        call after_gray_integration
        pabstott=pabstott+pabstot
        currtott=currtott+currtot
      end if
      print*,' '
      write(6,*) 'Pabs (MW), Icd (kA) = ', pabstott,currtott*1.0e3_wp_
c
      end
c
c
c
      subroutine gray_integration
      use const_and_precisions, only : wp_
      use graydata_flags, only : dst
      implicit none
c local variables
      integer :: i
      real(wp_) :: st0
c common/external functions/variables
      integer :: istep,nstep,istop,index_rt
      real(wp_) :: st,strfl11
c
      common/ss/st
      common/istep/istep
      common/nstep/nstep
      common/istop/istop
      common/strfl11/strfl11
      common/index_rt/index_rt
c
c     ray integration: begin
      st0=0.0_wp_
      if(index_rt.gt.1) st0=strfl11
      do i=1,nstep
        istep=i
        st=i*dst+st0
c
c       advance one step 
        call rkint4
c
c       calculations after one step:
        call after_onestep(istep,istop)
        if(istop.eq.1) exit
c
      end do
c
c     ray integration: end
c
      return
      end
c
c
c
      subroutine after_gray_integration
      use const_and_precisions, only : wp_,zero
      use graydata_flags, only : ibeam,iwarm,iequil,iprof,
     . filenmeqq,filenmprf,filenmbm
      use graydata_anequil, only : dens0,te0
      implicit none
c local variables
      integer :: iproj,nfilp
      real(wp_) :: currtka,pabs,currt
c common/external functions/variables
      integer :: nrayr,nrayth,index_rt
      real(wp_) :: st,taumn,taumx,pabstot,currtot
c
      common/ss/st
      common/nray/nrayr,nrayth
      common/index_rt/index_rt
      common/taumnx/taumn,taumx,pabstot,currtot
c
c     print all ray positions in local reference system
c
      if(nrayr.gt.1) then
        iproj=1
        nfilp=9
        call projxyzt(iproj,nfilp)
      end if
c
c     print final results on screen
c
      write(*,*)
      write(*,'(a,f9.4)') 'final step (s, ct, Sr)  = ',st
      if(iwarm.gt.0) then
        write(*,'(a,2e12.5)') 'taumn, taumx  = ', taumn,taumx
      else
        write(*,'(a,2f9.4)') 'taumn, taumx  = ', zero,zero
      end if      
c
      write(*,'(a,f9.4)') 'Pabs_tot (MW) = ',pabstot
      currtka =currtot*1.0e3_wp_
      write(*,'(a,f9.4)') 'I_tot (kA)    = ',currtka
c
      if (index_rt.eq.1) then
      if(iequil.eq.2) write(*,*) 'EQUILIBRIUM CASE : ',filenmeqq
      if(iequil.eq.1) write(*,*) 'ANALTYCAL EQUILIBRIUM'
      if(iprof.eq.1) write(*,*)  'PROFILE file : ',filenmprf
      if(iprof.eq.0) write(*,*)  'ANALTYCAL PROFILES ne0,Te0',dens0,te0
      if(ibeam.ge.1) write(*,*)  'LAUNCHER CASE : ',filenmbm

      end if
c
c       compute power and current density profiles for all rays
c
      pabs=pabstot
      currt=currtot
      call pec(pabs,currt)
c
      return
      end
c
c
c
      subroutine after_onestep(i,istop)
      use const_and_precisions, only : wp_,pi
      use reflections, only : inside
      use graydata_flags, only : iwarm,istpr0,istpl0,dst,ipass,igrad
      use graydata_par, only : psipol0,chipol0
      use interp_eqprof, only : zbmin,zbmax,rlim,zlim,nlim
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36,nmx=8000
      real(wp_), parameter :: taucr=12.0_wp_
c arguments
      integer :: i, istop
c local variables
      integer iproj,nfilp,irfl,j,k,kkk,iopmin,iowmax,iowmin,ivac,j1,k1,
     . kkkk
      real(wp_) :: qqout,uuout,vvout,qqin2,uuin2,vvin2,rrm,zzm,anvjkr,
     . aknmin,akdotn,anwclr
      real(wp_), dimension(6) :: y,dery
      real(wp_), dimension(3) :: xvrfl,anvrfl,xvvac,anw
      real(wp_), dimension(3,jmx,kmx) :: xvjk,anvjk
      complex(wp_) :: extr,eytr,exin2,eyin2
      logical :: ins_pl
c common/external functions/variables
      integer :: nrayr,nrayth,istpr,istpl,jclosest,ierr,index_rt
      integer, dimension(jmx,kmx) :: iiv,iop,iow,ihcd,istore
      real(wp_) :: psinv,psinv11,taumn,taumx,pabstot,currtot,psipol,
     . chipol,powrfl,strfl11
      real(wp_), dimension(3) :: anwcl,xwcl,xv,anv
      real(wp_), dimension(jmx,kmx) :: tau1v
      real(wp_), dimension(jmx,kmx,6) :: yyrfl
      real(wp_), dimension(6,jmx,kmx) :: ywrk,ypwrk
      real(wp_), dimension(jmx,kmx,nmx) :: ppabs,ccci,tauv,alphav,psjki
      complex(wp_), dimension(jmx,kmx,0:4) :: ext,eyt
      logical :: inside_plasma
c
      external inside_plasma
c
      common/pcjki/ppabs,ccci
      common/atjki/tauv,alphav
      common/tau1v/tau1v
      common/nray/nrayr,nrayth
      common/istgr/istpr,istpl
      common/iiv/iiv
      common/iov/iop,iow,ihcd,istore
      common/refln/anwcl,xwcl,jclosest
      common/psjki/psjki
      common/psival/psinv
      common/psinv11/psinv11
      common/ierr/ierr
      common/taumnx/taumn,taumx,pabstot,currtot
      common/xv/xv
      common/anv/anv
      common/polcof/psipol,chipol
      common/evt/ext,eyt
      common/yyrfl/yyrfl
      common/powrfl/powrfl
      common/strfl11/strfl11
      common/index_rt/index_rt
      common/wrk/ywrk,ypwrk
c
      pabstot=0.0_wp_
      currtot=0.0_wp_
      taumn=1.0e+30_wp_
      taumx=-1.0e+30_wp_
      psinv11=1.0_wp_
      iopmin=100
      iowmin=100
      iowmax=0
      
      if(i.eq.1) then
        psipol=psipol0
        chipol=chipol0
      end   if     
c
      do j=1,nrayr
        kkk=nrayth
        if(j.eq.1) kkk=1
        do k=1,kkk
          call gwork(j,k)
c 
          if(ierr.gt.0) then
            write(*,*) '    IERR = ', ierr
            if(ierr.eq.97) then
c              igrad=0
c              ierr=0 
            else  
              istop=1
c              exit
            end if
          end if

          psjki(j,k,i)=psinv
          rrm=sqrt(xv(1)**2+xv(2)**2)/100.0_wp_
          zzm=xv(3)/100.0_wp_

          if (iwarm.gt.0.and.i.gt.1.and.ihcd(j,k).gt.0) then
            if(psinv.ge.0.and.psinv.le.1.0_wp_.and.
     .         zzm.ge.zbmin.and.zzm.le.zbmax) then
              call pabs_curr(i,j,k)
              iiv(j,k)=i
            else
              if(iiv(j,k).gt.1) tauv(j,k,i)=tauv(j,k,i-1)
            end if
            if(tauv(j,k,i).le.taumn) taumn=tauv(j,k,i)
            if(tauv(j,k,i).ge.taumx) taumx=tauv(j,k,i)
            pabstot=pabstot+ppabs(j,k,iiv(j,k))
            currtot=currtot+ccci(j,k,iiv(j,k))
          end if 
          call print_output(i,j,k)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
          ins_pl=inside_plasma(rrm,zzm)
          if (mod(iop(j,k),2).eq.0 .and. ins_pl) then
            iop(j,k)=iop(j,k)+1
            call pol_limit(ext(j,k,iop(j,k)),eyt(j,k,iop(j,k)))

            if (ipass.gt.1 .and. index_rt.eq.1 .and. 
     .          iowmax.gt.1 .and. istore(j,k).eq.0) then
              istore(j,k)=istore(j,k)+1
              yyrfl(j,k,1:3)=xv
              yyrfl(j,k,4:6)=anv
              ihcd(j,k)=0
            end if
          else if (mod(iop(j,k),2).eq.1.and.
     .             .not.ins_pl) then
            iop(j,k)=iop(j,k)+1
            call pol_limit(ext(j,k,iop(j,k)),eyt(j,k,iop(j,k)))
          end if
          
          if (ipass.gt.1) then
            if (iow(j,k).eq.0 .and. inside(rlim,zlim,nlim,rrm,zzm)) then
              iow(j,k)=1
            else if (iow(j,k).eq.1 .and. 
     .               .not.inside(rlim,zlim,nlim,rrm,zzm)) then
              iow(j,k)=2
              if (ins_pl) then
                iop(j,k)=iop(j,k)+1
                call pol_limit(ext(j,k,iop(j,k)),eyt(j,k,iop(j,k)))
              end if
              call wall_refl(xv-dst*anv,anv,ext(j,k,iop(j,k)),
     .                eyt(j,k,iop(j,k)),xvrfl,anvrfl,extr,eytr,anw,irfl)
              istore(j,k)=istore(j,k)+1
              yyrfl(j,k,1:3)=xvrfl
              yyrfl(j,k,4:6)=anvrfl
              tau1v(j,k)=tauv(j,k,iiv(j,k))
              ext(j,k,iop(j,k))=extr
              eyt(j,k,iop(j,k))=eytr
              if (j.lt.jclosest) then
                jclosest=j
                anwcl=anw
                xwcl=xvrfl
              end if
              xv=xvrfl
              anv=anvrfl
              rrm=1.0e-2_wp_*sqrt(xv(1)**2+xv(2)**2)
              zzm=1.0e-2_wp_*xv(3)
              ywrk(1:3,j,k)=xv
              ywrk(4:6,j,k)=anv
              igrad=0
              call gwork(j,k)
              if (ins_pl) then
                iop(j,k)=iop(j,k)+1
                call pol_limit(ext(j,k,iop(j,k)),eyt(j,k,iop(j,k)))
                if (index_rt.eq.1) ihcd(j,k)=0
              end if
            end if
          end if

          if(index_rt.eq.1 .and. j.eq.1) psinv11=psinv
          if(iop(j,k).lt.iopmin) iopmin=iop(j,k)
          if(iow(j,k).lt.iowmin) iowmin=iow(j,k)
          if(iow(j,k).gt.iowmax) iowmax=iow(j,k)

          xvjk(:,j,k)=xv
          anvjk(:,j,k)=anv

        end do
      end do

      if(jclosest.le.nrayr) then
        aknmin=1.0_wp_
        do j=1,nrayr
          kkk=nrayth
          if(j.eq.1) kkk=1
          do k=1,kkk
            anwclr=(anwcl(1)*xwcl(1)+anwcl(2)*xwcl(2))
     .             /sqrt(xwcl(1)**2+xwcl(2)**2)
            anvjkr=(anvjk(1,j,k)*xvjk(1,j,k)+anvjk(2,j,k)*xvjk(2,j,k))
     .             /sqrt(xvjk(1,j,k)**2+xvjk(2,j,k)**2)
            akdotn=anwclr*anvjkr+anwcl(3)*anvjk(3,j,k)
            if(akdotn.lt.aknmin) aknmin=akdotn
          end do
        end do
      else
        aknmin=-1.0_wp_
      end if

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c     print ray positions for j=nrayr in local reference system

      istpr=istpr+1
      if (istpr.eq.istpr0) then
c        print*,'istep = ',i
        if(nrayr.gt.1) then
          iproj=0
          nfilp=8
          call projxyzt(iproj,nfilp)
        end if
        istpr=0
      end if
c
      if (istpl.eq.istpl0) istpl=0
      istpl=istpl+1


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c     single pass is stopped when all the rays have crossed the plasma
c         or complete absorption has occurred
c     same for successive passes of multi-pass simulations (here exit
c         from vessel is detected too
c     first pass in multi-pass simulation is stopped when at least one
c         ray has reflected and all rays are directed away from
c         reflection point, or when no reflection has occurred and
c         central ray re-enters the plasma
      
      if((ipass.eq.1 .and. ((iopmin.gt.1) .or.
     . (taumn.lt.1.0e+30_wp_.and.taumn.gt.taucr)))
     . .or.(index_rt.gt.1 .and. (iopmin.gt.1 .or. iowmin.gt.1 .or.
     . (taumn.lt.1.0e+30_wp_.and.taumn.gt.taucr)))) then
        istop=1
      else if(ipass.gt.1 .and. index_rt.eq.1 .and.
     .   ((iowmin.gt.1 .and. aknmin.gt.0) .or.
     .    (iowmax.le.1 .and. iop(1,1).gt.2))) then
c     flag second pass mode coupling as unset
        powrfl=-1.0_wp_
        qqout=0.0_wp_
        uuout=0.0_wp_
        vvout=0.0_wp_
        do j=1,nrayr
          kkk=nrayth
          if(j.eq.1) kkk=1
          do k=1,kkk
c     store missing initial conditions for the second pass
            if (istore(j,k).eq.0) then
              istore(j,k)=istore(j,k)+1
              yyrfl(j,k,1:3)=xvjk(:,j,k)
              yyrfl(j,k,4:6)=anvjk(:,j,k)
              tau1v(j,k)=tauv(j,k,iiv(j,k))
            end if
c     determine mode coupling at the plasma boundary
            if (powrfl.lt.0.0_wp_) then
              call vacuum_rt(xvjk(:,j,k),anvjk(:,j,k),xvvac,ivac)
c     look for first ray hitting the plasma, starting from the central
c     and evaluate polarization
              if (ivac.eq.1) then
                y(1:3)=xvjk(:,j,k)
                y(4:6)=anvjk(:,j,k)
                call fwork(y,dery)
                call pol_limit(exin2,eyin2)
                call stokes(exin2,eyin2,qqin2,uuin2,vvin2)
       powloop: do j1=1,nrayr
                  kkkk=nrayth
                  if(j1.eq.1) kkkk=1
                  do k1=1,kkkk
c     look for first ray which completed the first pass in the plasma
                    if (iop(j1,k1).gt.1) then
c     if found, use its polarization state to compute mode coupling
                      call stokes(ext(j1,k1,2),eyt(j1,k1,2),
     .                            qqout,uuout,vvout)
                      exit powloop
                    end if
                  end do
                end do powloop
c     if no ray completed a first pass in the plasma, use central ray
c     initial polarization (possibly reflected)
                if (qqout.le.0.0_wp_) then
                  call stokes(ext(1,1,0),eyt(1,1,0),qqout,uuout,vvout)
                end if
                powrfl=0.5_wp_*(1.0_wp_+vvout*vvin2+
     .                              uuout*uuin2+qqout*qqin2)
              end if
            end if
          end do
        end do
        strfl11=i*dst
        write(6,*) '  '
        write(6,*) 'Reflected power fraction =',powrfl
        write(66,*) psipol0,chipol0,powrfl
        istop=1
      end if

      return
      end
c
c
c
      subroutine print_output(i,j,k)
      use const_and_precisions, only : wp_,pi
      use graydata_flags, only : istpl0
      use graydata_par, only : psdbnd
      implicit none
c local constants
      integer, parameter :: ndim=6,jmx=31,kmx=36,nmx=8000
      real(wp_), parameter :: taucr=12.0_wp_
c arguments
      integer i,j,k
c local variables
      real(wp_) :: x,y,z,rr,rrm,zzm,stm,xxm,yym,anx,any,anz,anr,anphi,
     . phi,phideg,psi,rhot,bbr,bbz,bpol,dens11,dids11,dpdv11,ajcd11,
     . alpha11,taujk,tau11,pt11
c common/external functions/variables
      integer :: nharm,nhf,iohkawa,index_rt,nrayr,nrayth,istpr,istpl
      real(wp_) :: p0mw,st,brr,bphi,bzz,ajphi,btot,xg,yg,dens,ddens,
     . tekev,alpha,effjcd,akim,tau0,anpl,anpr,ddr,an2s,an2,fdia,bdotgr,
     . ddi,ddr11,anprr,anpri,frhotor
      real(wp_), dimension(jmx) :: q
      real(wp_), dimension(jmx,kmx) :: tau1v
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk,ypwrk
      real(wp_), dimension(jmx,kmx,nmx) :: psjki,tauv,alphav,pdjki,
     . ppabs,currj,didst,ccci
      complex(wp_) :: ex,ey,ez
c
      common/psjki/psjki
      common/atjki/tauv,alphav
      common/dpjjki/pdjki,currj
      common/pcjki/ppabs,ccci
      common/dcjki/didst
      common/nharm/nharm,nhf
      common/iokh/iohkawa
      common/p0/p0mw
      common/tau1v/tau1v
      common/qw/q
      common/index_rt/index_rt
      common/ss/st
      common/nray/nrayr,nrayth
      common/istgr/istpr,istpl
      common/parpl/brr,bphi,bzz,ajphi
      common/btot/btot
      common/xgxg/xg
      common/ygyg/yg
      common/dens/dens,ddens
      common/tete/tekev
      common/absor/alpha,effjcd,akim,tau0
      common/epolar/ex,ey,ez
      common/nplr/anpl,anpr
      common/ddd/ddr,an2s,an2,fdia,bdotgr,ddi,ddr11
      common/nprw/anprr,anpri
      common/wrk/ywrk,ypwrk
c
      x=ywrk(1,j,k)
      y=ywrk(2,j,k)
      z=ywrk(3,j,k)
      rr=sqrt(x*x+y*y)
c
      anx=ywrk(4,j,k)
      any=ywrk(5,j,k)
      anz=ywrk(6,j,k)
      anr=(anx*x+any*y)/rr
      anphi=(any*x-anx*y)/rr
c      cnphi=(any*x-anx*y)
c
      rrm=rr*1.0e-2_wp_
      zzm=z*1.0e-2_wp_
      stm=st*1.0e-2_wp_
      xxm=x*1.0e-2_wp_
      yym=y*1.0e-2_wp_
c
      if(index_rt.gt.1) then
        taujk=tauv(j,k,i)+tau1v(j,k)
      else
        taujk=tauv(j,k,i)
      end if

c     central ray only begin
      if(j.eq.1) then
        phi=acos(x/sqrt(y*y+x*x))
        if(y.lt.0.0_wp_) phi=-phi
        phideg=phi*180.0_wp_/pi
        psi=psjki(j,k,i)
        rhot=1.0_wp_
        bbr=0.0_wp_
        bbz=0.0_wp_
        bpol=0.0_wp_
        rhot=1.0_wp_
        dens11=0.0_wp_
        if(psi.ge.0.0_wp_) then
            if (psi.le.1.0_wp_) rhot=frhotor(psi)
            bbr=brr
            bbz=bzz
            bpol=sqrt(bbr**2+bbz**2)
        else
          tekev=0.0_wp_
          akim=0.0_wp_
        end if
        ddr11=ddr
c        cutoff=xg-(1-yg)*(1-anpl**2)        

c     print dIds in A/m/W, Jphi and Jt in MA/m2, ki in m^-1

        if(psi.le.psdbnd.and.psi.ge.0.0_wp_) dens11=dens
        dids11=didst(j,k,i)*1.0e2_wp_/(p0mw*q(j))
        dpdv11=pdjki(j,k,i)/(p0mw*q(j))
        ajcd11=currj(j,k,i)/(p0mw*q(j))
        alpha11=alphav(j,k,i)
        tau11=taujk
        pt11=exp(-taujk)

        write(4,99) stm,rrm,zzm,phideg,
     .  psi,rhot,dens11,tekev,
     .  bbr,bphi,bbz,ajphi*1.0e-6_wp_,sqrt(an2),anpl,akim*100,
     .  alpha11,tau11,pt11,ajcd11,dids11,
     .  dble(nhf),dble(iohkawa),dble(index_rt)

c        call polarcold(exf,eyif,ezf,elf,etf)
      end if
c     central ray only end

      if(k.eq.1.and.j.eq.nrayr) write(17,99) st,ddr11,ddr,ddi

c     print dIds in MA/m, Jcd in MA/m^2, ppabs and pdjki in MW/m^3
      if(istpl.eq.istpl0)  then
        if(j.eq.nrayr) then
c        if(j.eq.nrayr.and.tauv(j,k,i).le.taucr) then
          write(33,111) i,j,k,stm,xxm,yym,rrm,zzm,
     .          psjki(j,k,i),taujk,anpl,alphav(j,k,i),dble(index_rt)
        end if
c      if(k.eq.nrayth) write(33,*) ' '
      end if
c
      return
 99   format(30(1x,e16.8e3))
111   format(3i5,16(1x,e16.8e3))
      end
c
c
c
      subroutine prfile
      implicit none
c common/external functions/variables
      integer :: index_rt
c
      common/index_rt/index_rt
c
      if(index_rt.eq.1) then
        write(4,*)' #sst R z phi psi rhot ne Te BR Bphi Bz Jphi '//
     .   'N Npl ki alpha tau Pt Jcd dIds nh iohkw index_rt'
        write(8,*) ' #istep j k xt yt zt rt psin'
        write(9,*) ' #istep j k xt yt zt rt psin'
        write(17,*) ' #sst Dr_11 Dr_Nr1 Di_Nr1'
        write(33,*) ' #i j k sst x y R z psi tauv Npl alpha index_rt'
        write(12,*) ' #i sst psi w1 w2'
        write(7,*)'#Icd Pa Jphip dPdVp '//
     .   'rhotj rhotjava rhotp rhotpav drhotjava drhotpav ratjamx '//
     .   'ratjbmx stmx psipol chipol index_rt Jphimx dPdVmx drhotj '//
     .   'drhotp'
        write(48,*) '#psi rhot Jphi Jcdb dPdV Icdins Pins P% index_rt'
        write(66,*) "# psipol0 chipol0 powrfl"
c
      else
c
c      if(index_rt.eq.3) then
        write(4,*) ' '
        write(8,*) ' '
        write(9,*) ' '
        write(17,*) ' '
        write(12,*) ' '
        write(48,*) ' '
c      end if
      end if
c
      return
      end
c
c
c
      subroutine read_data
      use const_and_precisions, only : wp_,qe=>ecgs_,me=>mecgs_,
     . vc=>ccgs_,cvdr=>degree,pi
      use green_func_p, only:Setup_SpitzFunc
      use graydata_flags
      use graydata_par
      use graydata_anequil
      use interp_eqprof, only : rmxm,rlim,zlim,nlim,zbmin,zbmax,
     . btrcen,rcen,alloc_lim
      implicit none
c local constants
      integer, parameter :: nmx=8000
c local variables
      integer :: nfil,iox,ierr,leqq,lprf
      real(wp_) :: dummy,bresg,r00,anr0c,anphi0c,fghz,zeff,
     . w0csi,w0eta,d0csi,d0eta,phi0,zrcsi,zreta
      character(len=8) :: wdat
      character(len=10) :: wtim
c common/external functions/variables
      integer :: nstep,nrayr,nrayth
      real(wp_) :: xgcn,ak0,akinv,fhz,wcsi,weta,rcicsi,rcieta,phiw,
     . phir,anx0c,any0c,anz0c,x00,y00,z00,bres,p0mw,sox,alpha0,beta0
c
      common/xgcn/xgcn
      common/nstep/nstep
      common/nray/nrayr,nrayth
      common/parwv/ak0,akinv,fhz
      common/parbeam/wcsi,weta,rcicsi,rcieta,phiw,phir
      common/anic/anx0c,any0c,anz0c
      common/mirr/x00,y00,z00
      common/parbres/bres
      common/p0/p0mw
      common/mode/sox
      common/angles/alpha0,beta0
c
      open(2,file='gray.data',status= 'unknown')
c
c     alpha0, beta0 (cartesian) launching angles
c     fghz      wave frequency (GHz)
c     p0mw      injected power (MW)
c
      read(2,*) alpha0,beta0
      read(2,*) fghz
      read(2,*) p0mw
c
c     nrayr     number of rays in radial direction
c     nrayth      number of rays in angular direction
c     rwmax      normalized maximum radius of beam power
c     rwmax=1 -> last ray at radius = waist
c
      read(2,*) nrayr,nrayth,rwmax
      if(nrayr.eq.1) nrayth=1
c
c     x00,y00,z00 coordinates of launching point
c
      read(2,*) x00,y00,z00
c
c     beams parameters in local reference system
c     w0 -> waist, d0 -> waist distance from launching point
c     phiw  angle of beam ellipse
c
      read(2,*) w0csi,w0eta,d0csi,d0eta,phiw
c
c     ibeam=0  :read data for beam as above
c     ibeam=1  :read data from file simple astigmatic beam
c     ibeam=2  :read data from file general astigmatic beam
      read(2,*) ibeam
      read(2,*) filenmbm
c
c     iequil=0  :vacuum
c     iequil=1  :analytical equilibrium
c     iequil=2  :read eqdsk
c     ixp=0,-1,+1  : no X point , bottom/up X point
c
      read(2,*) iequil,ixp
c
c     iprof=0  :analytical density and temp. profiles
c     iprof>0  :numerical density and temp. profiles
c
      read(2,*) iprof
c
c     iwarm=0  :no absorption and cd
c     iwarm=1  :weakly relativistic absorption
c     iwarm=2  :relativistic absorption, n<1 asymptotic expansion
c     iwarm=3  :relativistic absorption, numerical integration
c     ilarm    :order of larmor expansion
c     imx      :max n of iterations in dispersion, imx<0 uses 1st
c               iteration in case of failure after |imx| iterations

      read(2,*) iwarm,ilarm,imx
c      if(iwarm.gt.2) iwarm=2
c
c     ieccd 0/1 NO/YES  ECCD calculation ieccd>0 different CD models
c
      read(2,*) ieccd
      if (ieccd.ge.10)     call Setup_SpitzFunc
c
c     ipec=0/1  :pec profiles grid in psi/rhop
c     nnd       :number of grid steps for pec profiles +1
c
      read(2,*) ipec,nnd
c
c     dst       integration step
c     nstep      maximum number of integration steps
c     istpr0     projection step = dsdt*istprj
c     istpl0      plot step = dsdt*istpl
c     igrad=0    optical ray-tracing, initial conditions as for beam
c     igrad=1    quasi-optical ray-tracing
c     igrad=-1   ray-tracing, init. condit.
c                from center of mirror and with angular spread
c     ipass=1/2 1 or 2 passes into plasma
c     iox=1/2    OM/XM
c     idst=0/1/2  0 integration in s, 1 integr. in ct, 2 integr. in Sr
c     psipol0,chipol0 polarization angles
c     ipol=0 compute mode polarization at antenna, ipol=1 use polariz angles
c
      read(2,*) dst,nstep,istpr0,istpl0,idst
      read(2,*) igrad,ipass,rwallm
      read(2,*) iox, psipol0,chipol0,ipol
c
c     ipsinorm  0 standard EQDSK format, 1 format Portone summer 2004
c     sspl      spline parameter for psi interpolation
c
      read(2,*) filenmeqq
      read(2,*) ipsinorm,sspl
c
c     factb     factor for magnetic field (only for numerical equil)
c     scaling adopted: beta=const, qpsi=const, nustar=const
c     iscal    ne Te scaling 0: nustar=const,  1: n_greenw=const; 2 no rescaling
c     factT factn   factor for Te&ne scaling 
c
      read(2,*) factb, iscal,factt,factn
      if (factb.eq.0.0_wp_) factb=1.0_wp_
c
c     psbnd     value of psi ( > 1 ) of density boundary
c
      read(2,*) filenmprf
      read(2,*) psdbnd
      if(iprof.eq.0) psdbnd=1.0_wp_
c
c     signum of toroidal B and I
c     icocos index for equilibrium from COCOS - O. Sauter Feb 2012
      read(2,*) sgnbphi,sgniphi,icocos
c
c     read parameters for analytical density and temperature
c     profiles if iprof=0
c
      if(iprof.eq.0) then
        read(2,*) dens0,aln1,aln2
        read(2,*) te0,dte0,alt1,alt2
      else
        read(2,*) dummy,dummy,dummy
        read(2,*) dummy,dummy,dummy,dummy
      end if
      read(2,*) zeff
c
c     read parameters for analytical simple cylindical equilibrium
c     if iequil=0,1

      if(iequil.lt.2) then
        read(2,*) rr0,zr0,rpa
        read(2,*) b0
        read(2,*) q0,qa,alq
        rr0m=rr0/1.0e2_wp_
        zr0m=zr0/1.0e2_wp_
        rpam=rpa/1.0e2_wp_
        rcen=rr0m
        btrcen=b0
        zbmin=(zr0-rpa)/100.0_wp_
        zbmax=(zr0+rpa)/100.0_wp_
        b0=b0*factb
        call flux_average_an
c        call print_prof_an
      else
        read(2,*) dummy,dummy,dummy
        read(2,*) dummy
        read(2,*) dummy,dummy,dummy
      end if
c
      close(unit=2)
c
      if(nrayr.eq.1) igrad=0
      if (nrayr.lt.5) then
        igrad=0
        print*,'    nrayr < 5 ! => OPTICAL CASE ONLY'
        print*,' '
      end if
c
      fhz=fghz*1.0e9_wp_
      ak0=2.0e9_wp_*pi*fghz/vc
      akinv=1.0_wp_/ak0
c
      bresg=2.0_wp_*pi*fhz*me*vc/qe
      bres=bresg*1.0e-4_wp_
c
c     xg=xgcn*dens19
c
      xgcn=1.0e-5_wp_*qe**2/(pi*me*fghz**2)
c
      sox=-1.0_wp_
      if(iox.eq.2) sox=1.0_wp_
c
c     read data for beam from file if ibeam>0
c
      if(ibeam.gt.0) then
        call read_beams
      else
        zrcsi=0.5_wp_*ak0*w0csi**2
        zreta=0.5_wp_*ak0*w0eta**2
        if(igrad.gt.0) then
          wcsi=w0csi*sqrt(1.0_wp_+(d0csi/zrcsi)**2)
          weta=w0eta*sqrt(1.0_wp_+(d0eta/zreta)**2)
          rcicsi=-d0csi/(d0csi**2+zrcsi**2)
          rcieta=-d0eta/(d0eta**2+zreta**2)
          phir=phiw
        else
          d0eta=d0csi
          wcsi=w0csi*abs(d0csi/zrcsi)
          weta=w0eta*abs(d0eta/zreta)
          rcicsi=w0csi/zrcsi
          rcieta=w0eta/zreta
          if(d0csi.gt.0) then
            rcicsi=-rcicsi
            rcieta=-rcieta
          end if
          phir=phiw
        end if
      end if
c
      print'(a,2f8.3)','alpha0, beta0 = ',alpha0,beta0
c
      r00=sqrt(x00**2+y00**2)
      phi0=acos(x00/r00)
      if(y00.lt.0) phi0=-phi0
      print'(a,4f8.3)','x00, y00, R00, z00 = ',x00,y00,r00,z00
      print*,' '
c
c      anx0c=(anr0c*x00-anphi0c*y00)/r00
c      any0c=(anr0c*y00+anphi0c*x00)/r00
c
c      anr0c=(anx0c*x00+any0c*y00)/r00
c      anphi0c=(any0c*x00-anx0c*y00)/r00
c
c     angles alpha0, beta0 in a local reference system as proposed by Gribov et al
c
c      anr0c=-cos(cvdr*beta0)*cos(cvdr*alpha0)
c      anphi0c=sin(cvdr*beta0)
c      anz0c=-cos(cvdr*beta0)*sin(cvdr*alpha0)

      anr0c=-cos(cvdr*beta0)*cos(cvdr*alpha0)
      anphi0c=sin(cvdr*beta0)
      anz0c=-cos(cvdr*beta0)*sin(cvdr*alpha0)
 
      anx0c=(anr0c*x00-anphi0c*y00)/r00
      any0c=(anr0c*y00+anphi0c*x00)/r00
c
c     read data for Te , ne , Zeff from file if iprof>0
c

      if (iprof.eq.1) then
        nprof=98
        lprf=len_trim(filenmprf)
        open(file=filenmprf(1:lprf)//'.prf',
     .                    status= 'unknown',unit=nprof)
        call profdata
        close(nprof)
      end if
c
c     read equilibrium data  from file if iequil=2
c
      if (iequil.eq.2) then
        neqdsk=99
        leqq=len_trim(filenmeqq)
         open(file=filenmeqq(1:leqq)//'.eqdsk',
     .        status= 'unknown', unit=neqdsk)
        call equidata
        close(neqdsk)
        
c     print density, temperature, safecty factor, toroidal current dens
c     versus psi, rhop, rhot
        call print_prof
      end if

      if (iequil.eq.1)   call bres_anal

      if (iequil.ne.2.or.ipass.lt.0) then
c     set simple limiter as two cylindrical walls at rwallm and r00
        nlim=5
        call alloc_lim(ierr)
        if (ierr.ne.0) stop
        rlim(1)=rwallm
        rlim(2)=max(r00*1.0e-2_wp_,rmxm)
        rlim(3)=rlim(2)
        rlim(4)=rlim(1)
        rlim(5)=rlim(1)
        zlim(1)=(z00-dst*nmx)*1.0e-2_wp_
        zlim(2)=zlim(1)
        zlim(3)=(z00+dst*nmx)*1.0e-2_wp_
        zlim(4)=zlim(3)
        zlim(5)=zlim(1)
        ipass=abs(ipass)
      end if

      nfil=78
      
      open(nfil,file='headers.txt',status= 'unknown')
      call date_and_time(wdat,wtim)
      write(nfil,916) wdat(1:4),wdat(5:6),wdat(7:8),
     .                wtim(1:2),wtim(3:4),wtim(5:6)
      write(nfil,904) REVISION
      if (iequil.eq.2) then
        write(nfil,905) trim(filenmeqq)
      else
        if (iequil.eq.1) write(nfil,912) rr0,zr0,rpa,B0,q0,qa,alq
        if (iequil.eq.0) write(nfil,'("# VACUUM CASE")') 
      end if    
      if (iprof.eq.1) then
        write(nfil,907) trim(filenmprf)
      else
        write(nfil,913) dens0,aln1,aln2,te0,dte0,alt1,alt2,zeff
      end if    
      write(nfil,911) fghz,p0mw,iox
      write(nfil,902) x00,y00,z00      
      write(nfil,908) alpha0,beta0   
      if(ibeam.ge.1) write(nfil,909) trim(filenmbm)
      if(ibeam.eq.0) write(nfil,903) w0csi,w0eta,d0csi,d0eta,phiw
      write(nfil,900) nrayr, nrayth, rwmax
      write(nfil,901) igrad,iwarm,ilarm,ieccd,idst
      write(nfil,915) dst,nstep
      write(nfil,914) sgnbphi,sgniphi,icocos
      write(nfil,906) factb,factt,factn,iscal
      write(nfil,910) sspl,psdbnd,nnd,ipec,ipsinorm
      write(nfil,*) ' '
      close(nfil) 
      
      return
      
900   format('# Nray_r Nray_th rwmax : ',2i5,1x,es12.5)      
901   format('# igrad iwarm ilarm ieccd idst ipol: ',7i5)      
902   format('# X0 Y0 Z0 launching point (cm) : ',3(1x,es12.5))
903   format('# w0xi w0eta d0xi d0eta (cm) phiw (deg) : ',5(1x,es12.5))
904   format('# GRAY revision : ',a)
905   format('# EQUILIBRIUM file : ',a)      
906   format('# fact_B  fact_T fact_n iscal : ',(3(1x,es12.5),i5))
907   format('# PROFILES file    : ',a)      
908   format('# alpha0 beta0 launch angles (deg) CYL : ',2(1x,es12.5))
909   format('# LAUNCHER file : ',a24)      
910   format('# sspl psdbnd nd ipec ipsinorm : ',2(1x,es12.5),3i5)      
911   format('# fghz P0 IOX : ',2(1x,es12.5),i5) 
912   format('# AN. EQUILIBRIUM R0 z0 a B0 q0 qa alq : ',7(1x,es12.5))
913   format('# AN. PROFILES ne0 aln1 aln2 Te0 Tea alt1 alt2 Zeff : '
     . ,8(1x,es12.5))
914   format('# sgnB_phi sgnI_phi icocos : ',2(1x,es12.5),i5)      
915   format('# dst nstep : ',1x,es12.5,i5)
916   format('# Date : ',a4,2('/',a2),1x,a2,2(':',a2))

      end
c
c
c
      subroutine surf_anal
      use const_and_precisions, only : wp_,pi
      use graydata_anequil, only : b0,rr0m,zr0m,rpam
      use magsurf_data, only : npsi,npoints,psicon,rcon,zcon,
     . alloc_surf_anal
      implicit none
c local variables
      integer :: i,k,ierr
      real(wp_) :: rni,rres,zzres,zmx,zmn,dal,drn,drrm,dzzm,rrm,zzm,zrm
c common/external functions/variables
      real(wp_) :: bres
c
      common/parbres/bres
c
      npsi=10
      npoints=101
      call alloc_surf_anal(ierr)
      if (ierr.ne.0) stop
c
c     print circular magnetic surfaces iequil=1
c
      write(71,*) '#i psin R z'
      dal=2.0e-2_wp_*pi
      drn=0.1_wp_
      do i=1,npsi
        rni=i*drn
        psicon(i)=rni
        do k=1,npoints
          drrm=rpam*rni*cos((k-1)*dal)
          dzzm=rpam*rni*sin((k-1)*dal)
          rrm=rr0m+drrm
          zzm=zr0m+dzzm
          rcon(i,k)=rrm
          zcon(i,k)=zrm
          write(71,111) i,rni,rrm,zzm
        end do
        write(71,*) ' '
        write(71,*) ' '
      end do
c
c     print resonant B iequil=1
c
      write(70,*)'#i Btot  R  z'
      rres=b0*rr0m/bres
      zmx=zr0m+rpam
      zmn=zr0m-rpam
      do i=1,21
          zzres=zmn+(i-1)*(zmx-zmn)/2.0e1_wp_
          write(70,111) i,bres,rres,zzres
      end do

      return
111   format(i4,20(1x,e16.8e3))
      end
c
c
c
      subroutine bres_anal
      use const_and_precisions, only : wp_,pi
      use graydata_anequil, only : b0,rr0m,zr0m,rpam
      implicit none
c local variables
      integer :: i
      real(wp_) :: rres,zmx,zmn,zzres
c common/external functions/variables
      real(wp_) :: bres
c
      common/parbres/bres
c
c     print resonant B iequil=1
      write(70,*)'#i Btot  R  z'
      rres=b0*rr0m/bres
      zmx=zr0m+rpam
      zmn=zr0m-rpam
      do i=1,21
          zzres=zmn+(i-1)*(zmx-zmn)/2.0e1_wp_
          write(70,111) i,bres,rres,zzres
      end do

      return
111   format(i4,20(1x,e16.8e3))
      end
c
c
c
      subroutine read_beams
      use const_and_precisions, only : wp_
      use graydata_flags, only : filenmbm, ibeam
      use utils, only : locate
      use simplespline, only : difcs,spli
      implicit none
c local constants
      integer, parameter :: nstrmx=50
c local variables
      integer :: i,k,ii,nfbeam,lbm,nisteer,steer,iopt,ier
      real(wp_) :: alphast,betast,x00mm,y00mm,z00mm,waist1,waist2,
     . dvir1,dvir2,delta,phi1,phi2,zr1,zr2,w1,w2,rci1,rci2,dal,betst
      real(wp_), dimension(nstrmx) :: alphastv,betastv,x00v,y00v,
     . z00v,waist1v,waist2v,rci1v,rci2v,phi1v,phi2v
      real(wp_), dimension(nstrmx,4) :: cbeta,cx0,cy0,cz0,cwaist1,
     . cwaist2,crci1,crci2,cphi1,cphi2
c common/external functions/variables
      real(wp_) :: wcsi,weta,rcicsi,rcieta,phiw,phir,x00,y00,z00,
     . alpha0,beta0,ak0,akinv,fhz
c
      common/parbeam/wcsi,weta,rcicsi,rcieta,phiw,phir
      common/mirr/x00,y00,z00
      common/angles/alpha0,beta0
      common/parwv/ak0,akinv,fhz
c
c     for given alpha0 -> beta0 + beam parameters
c
c     ibeam=1 simple astigmatic beam
c     ibeam=2 general astigmatic beam
c
c     initial beam data are measured in mm -> transformed to cm
c
      nfbeam=97
      lbm=len_trim(filenmbm)
      open(file=filenmbm(1:lbm)//'.txt',status= 'unknown',unit=nfbeam)
c
      read(nfbeam,*) nisteer
      do i=1,nisteer
        if(ibeam.eq.1) then
          read(nfbeam,*) steer,alphast,betast,x00mm,y00mm,z00mm,
     .                   waist1,dvir1,waist2,dvir2,delta
          phi1=delta
          phi2=delta
          zr1=0.5e-1_wp_*ak0*waist1**2
          zr2=0.5e-1_wp_*ak0*waist2**2
          w1=waist1*sqrt(1.0_wp_+(dvir1/zr1)**2)
          w2=waist2*sqrt(1.0_wp_+(dvir2/zr2)**2)
          rci1=-dvir1/(dvir1**2+zr1**2)
          rci2=-dvir2/(dvir2**2+zr2**2)
        else
          read(nfbeam,*) steer,alphast,betast,x00mm,y00mm,z00mm,
     .                   w1,w2,rci1,rci2,phi1,phi2
        end if
        x00v(i)=0.1_wp_*x00mm
        y00v(i)=0.1_wp_*y00mm
        z00v(i)=0.1_wp_*z00mm
        alphastv(i)=alphast
        betastv(i)=betast
        waist1v(i)=0.1_wp_*w1
        rci1v(i)=1.0e1_wp_*rci1
        waist2v(i)=0.1_wp_*w2
        rci2v(i)=1.0e1_wp_*rci2
        phi1v(i)=phi1
        phi2v(i)=phi2
      end do
c
      iopt=0
      call difcs(alphastv,betastv,nisteer,iopt,cbeta,ier)
      call difcs(alphastv,waist1v,nisteer,iopt,cwaist1,ier)
      call difcs(alphastv,rci1v,nisteer,iopt,crci1,ier)
      call difcs(alphastv,waist2v,nisteer,iopt,cwaist2,ier)
      call difcs(alphastv,rci2v,nisteer,iopt,crci2,ier)
      call difcs(alphastv,phi1v,nisteer,iopt,cphi1,ier)
      call difcs(alphastv,phi2v,nisteer,iopt,cphi2,ier)
      call difcs(alphastv,x00v,nisteer,iopt,cx0,ier)
      call difcs(alphastv,y00v,nisteer,iopt,cy0,ier)
      call difcs(alphastv,z00v,nisteer,iopt,cz0,ier)
c
      if(alpha0.gt.alphastv(1).and.alpha0.lt.alphastv(nisteer)) then
        call locate(alphastv,nisteer,alpha0,k)
        dal=alpha0-alphastv(k)
        betst=spli(cbeta,nisteer,k,dal)
        x00=spli(cx0,nisteer,k,dal)
        y00=spli(cy0,nisteer,k,dal)
        z00=spli(cz0,nisteer,k,dal)
        wcsi=spli(cwaist1,nisteer,k,dal)
        weta=spli(cwaist2,nisteer,k,dal)
        rcicsi=spli(crci1,nisteer,k,dal)
        rcieta=spli(crci2,nisteer,k,dal)
        phiw=spli(cphi1,nisteer,k,dal)
        phir=spli(cphi2,nisteer,k,dal)
      else
        print*,'    alpha0 outside table range !!!'
        if(alpha0.ge.alphastv(nisteer)) ii=nisteer
        if(alpha0.le.alphastv(1)) ii=1
        betst=betastv(ii)
        x00=x00v(ii)
        y00=y00v(ii)
        z00=z00v(ii)
        wcsi=waist1v(ii)
        weta=waist2v(ii)
        rcicsi=rci1v(ii)
        rcieta=rci2v(ii)
        phiw=phi1v(ii)
        phir=phi2v(ii)
      end if
      beta0=betst
c
      close(nfbeam)
c
      return
      end
c
c
c
      subroutine equidata
      use const_and_precisions, only : wp_,pi
      use utils, only : vmaxmini
      use dierckx, only : curfit,splev,regrid,bispev,coeff_parder
      use graydata_flags, only : ipsinorm,sspl,ixp,icocos,neqdsk
      use graydata_par, only : sgnbphi,sgniphi,factb
      use interp_eqprof, only : nsrt,nszt,nsft,rlim,zlim,nlim,nr,nz,
     . psia,psiant,psinop,btrcen,rcen,btaxis,rmaxis,zmaxis,rmnm,
     . rmxm,zmnm,zmxm,dr,dz,zbmin,zbmax,fpolas,phitedge,rup,zup,
     . rlw,zlw,tr,tz,tfp,cc=>cceq,cfp,cc01=>cceq01,cc10=>cceq10,
     . cc20=>cceq20,cc02=>cceq02,cc11=>cceq11,psinr,qpsi,rv,zv,psin,
     . psi,rbbbs,zbbbs,nbbbs,alloc_equilvec,alloc_bnd!,rrtor
      implicit none
c local constants
      integer, parameter :: kspl=3
c local variables
      integer :: idum,nrest,nzest,lwrk,liwrk,lw10,lw01,lw20,lw02,lw11,
     . lwrkf,lwrkbsp,liwrkbsp,i,j,ij,iopt,ier,nsr,nsz,nur,nuz,info,
     . iqmn,iqmx,icocosmod,ierr
      integer, dimension(:), allocatable :: iwrk,iwrkf,iwrkbsp
      real(wp_) :: xdum,drnr1,dznz1,rleft,zmid,psiaxis,psiedge,current,
     . dpsinr,psia0,fp,xb,xe,ssfp,psinxp,rmop,zmop,psinoptmp,r10,z10,
     . rxp,zxp,psinxptmp,r1,z1,psin1,rbmin,rbmax,q2,q15,qmin,qmax,
     . rhot15,psi15,rhot2,psi2
      real(wp_), dimension(:), allocatable :: fpoli,fvpsi,ffvpsi,fpol,
     . wrkf,wf,wrk,wrkbsp,pres,ffprim,pprim
      character(len=48) :: stringa
c common/external functions/variables
      real(wp_) :: rhotsx
      real(wp_) :: frhotor
c
      common/rhotsx/rhotsx
c
c     read from file eqdsk
c     (see http://fusion.gat.com/efit/g_eqdsk.html)
c
c     spline interpolation of psi and derivatives
c
      if(icocos.gt.0) then 
        read (neqdsk,'(a48,3i4)') stringa,idum,nr,nz
      else
        read (neqdsk,*) nr,nz
      end if
c
      call alloc_equilvec(ierr)
      if (ierr.ne.0) stop

      nrest=nr+4
      nzest=nz+4
      lwrk=4*(nr+nz)+11*(nrest+nzest)+nrest*nz+nzest+54
      liwrk=nz+nr+nrest+nzest+3
      lw10=nr*3+nz*4+nr*nz
      lw01=nr*4+nz*3+nr*nz
      lw20=nr*2+nz*4+nr*nz
      lw02=nr*4+nz*2+nr*nz
      lw11=nr*3+nz*3+nr*nz
      lwrkf=nr*4+nrest*16
      lwrkbsp=4*(nr+nz)
      liwrkbsp=nr+nz
c
      if(allocated(fvpsi)) deallocate(fvpsi)
      if(allocated(pres)) deallocate(pres)
      if(allocated(ffprim)) deallocate(ffprim)
      if(allocated(pprim)) deallocate(pprim)
      if(allocated(ffvpsi)) deallocate(ffvpsi)
      if(allocated(fpol)) deallocate(fpol)
      if(allocated(fpoli)) deallocate(fpoli)
      if(allocated(wrkf)) deallocate(wrkf)
      if(allocated(iwrkf)) deallocate(iwrkf)
      if(allocated(wf)) deallocate(wf)
      if(allocated(wrk)) deallocate(wrk)
      if(allocated(iwrk)) deallocate(iwrk)
      if(allocated(wrkbsp)) deallocate(wrkbsp)
      if(allocated(iwrkbsp)) deallocate(iwrkbsp)
c
      allocate(fvpsi(nr*nz),pres(nr),ffprim(nr),pprim(nr),
     . ffvpsi(nr*nz),fpol(nr),fpoli(nr),wrkf(lwrkf),iwrkf(nrest),
     . wf(nr),wrk(lwrk),iwrk(liwrk),wrkbsp(lwrkbsp),iwrkbsp(liwrkbsp))
c
      if(ipsinorm.eq.0) then
        read (neqdsk,2020) drnr1,dznz1,rcen,rleft,zmid
        read (neqdsk,2020) rmaxis,zmaxis,psiaxis,psiedge,btrcen
        read (neqdsk,2020) current,xdum,xdum,xdum,xdum
        read (neqdsk,2020) xdum,xdum,xdum,xdum,xdum
        read (neqdsk,2020) (fpol(i),i=1,nr)
        read (neqdsk,2020) (pres(i),i=1,nr)
        read (neqdsk,2020) (ffprim(i),i=1,nr)
        read (neqdsk,2020) (pprim(i),i=1,nr)
        read (neqdsk,2020) ((psi(i,j),i=1,nr),j=1,nz)
        read (neqdsk,2020) (qpsi(i),i=1,nr)
      else
        read (neqdsk,*) drnr1,dznz1,rcen,rleft,zmid
        read (neqdsk,*) rmaxis,zmaxis,psiaxis,psiedge,btrcen
        read (neqdsk,*) current,xdum,xdum,xdum,xdum
        read (neqdsk,*) xdum,xdum,xdum,xdum,xdum
        read (neqdsk,*) (fpol(i),i=1,nr)
        read (neqdsk,*) (pres(i),i=1,nr)
        read (neqdsk,*) (ffprim(i),i=1,nr)
        read (neqdsk,*) (pprim(i),i=1,nr)
        read (neqdsk,*) ((psin(i,j),i=1,nr),j=1,nz)
        read (neqdsk,*) (qpsi(i),i=1,nr)
      end if
2020  format (5e16.9)
c
c  compensate for different reference systems
c
      icocosmod=mod(icocos,10)
c
      if (mod(icocos,2).eq.0.and.icocosmod.gt.0) then
c  icocos mod 2 = 0: toroidal angle phi CW (opposite to gray convention)
        btrcen=-btrcen
        current=-current
        do i=1,nr
          fpol(i)=-fpol(i)
        end do
      end if
c
      if (icocosmod.eq.1 .or. icocosmod.eq.4 .or. 
     &    icocosmod.eq.5 .or. icocosmod.eq.8) then
c  icocos mod 10 = 1,4,5,8: psi increasing with CCW Ip
c  icocos mod 10 = 2,3,6,7: psi decreasing with CCW Ip
        psiedge=-psiedge
        psiaxis=-psiaxis
        if (ipsinorm.eq.0) then
          do j=1,nz
            do i=1,nr
              psi(i,j)=-psi(i,j)
            end do
          end do
        end if
      end if
c
c  add check for Ip/psi and B0/Fpol sign consistency?
c
      current=sign(current,psiaxis-psiedge)
      btrcen=sign(btrcen,fpol(nr))
c
c     length in m !!!
c
      dr=drnr1/dble(nr-1)
      dz=dznz1/dble(nz-1)
      rv(1)=rleft
      zv(1)=zmid-dznz1/2.0_wp_
      dpsinr=1.0_wp_/dble(nr-1)
c
      do i=1,nr
        psinr(i)=(i-1)*dpsinr
        rv(i)=rv(1)+(i-1)*dr
      end do
c
      do j=1,nz
        zv(j)=zv(1)+(j-1)*dz
      end do
c
      rmnm=rv(1)
      rmxm=rv(nr)
      zmnm=zv(1)
      zmxm=zv(nz)
c
c     psi function
c
      psia0=psiedge-psiaxis
c  icocos=0: adapt psi to force Ip sign, otherwise maintain psi
      if (icocosmod.ne.0) sgniphi=sign(1.0_wp_,-psia0)
      current=sign(current,sgniphi)
c
      psia=-sgniphi*abs(psia0)*factb
c  icocos>10: input psi is in Wb
c  icocos<10: input psi is in Wb/rad (gray convention)
      if (icocos.ge.10) psia=psia/(2.0_wp_*pi)
c
c      do j=1,nz
c        do i=1,nr
c          write(80,2021) rv(i),zv(j),psi(i,j)
c        enddo
c        write(80,*) ' '
c      enddo
c
      do j=1,nz
        do i=1,nr
          if(ipsinorm.eq.0) then
            psin(i,j)=(psi(i,j)-psiaxis)/psia0
            psi(i,j)=psin(i,j)*psia
          else
            psi(i,j)=psin(i,j)*psia
          end if
          ij=nz*(i-1)+j
          fvpsi(ij)=psin(i,j)
        enddo
      enddo
c
c     spline fitting/interpolation of psin(i,j) and derivatives
c
      iopt=0
      call regrid(iopt,nr,rv,nz,zv,fvpsi,rmnm,rmxm,zmnm,zmxm,
     .              kspl,kspl,sspl,nrest,nzest,nsr,tr,nsz,tz,cc,fp,
     .              wrk,lwrk,iwrk,liwrk,ier)
c     if ier=-1 data are fitted using sspl=0    
      if(ier.eq.-1) then
      sspl=0.0_wp_
      call regrid(iopt,nr,rv,nz,zv,fvpsi,rmnm,rmxm,zmnm,zmxm,
     .              kspl,kspl,sspl,nrest,nzest,nsr,tr,nsz,tz,cc,fp,
     .              wrk,lwrk,iwrk,liwrk,ier)
      end if
      nsrt=nsr
      nszt=nsz
      if (sspl.gt.0.0_wp_) then
        call bispev(tr,nsr,tz,nsz,cc,kspl,kspl,rv,nr,zv,nz,ffvpsi,
     .   wrkbsp,lwrkbsp,iwrkbsp,liwrkbsp,ier)
c
        do j=1,nz
          do i=1,nr
            ij=nz*(i-1)+j
            psin(i,j)=ffvpsi(ij)
            psi(i,j)=psin(i,j)*psia
c            write(79,2021) rv(i),zv(j),psin(i,j)
          enddo
c            write(79,*) ' '
        enddo
      end if
c2021  format(5(1x,e16.9))
c
      nur=1
      nuz=0
      call coeff_parder(tr,nsr,tz,nsz,cc,kspl,kspl,nur,nuz,
     . cc10,lw10,ier)
c
      nur=0
      nuz=1
      call coeff_parder(tr,nsr,tz,nsz,cc,kspl,kspl,nur,nuz,
     . cc01,lw01,ier)
c
      nur=2
      nuz=0
      call coeff_parder(tr,nsr,tz,nsz,cc,kspl,kspl,nur,nuz,
     . cc20,lw20,ier)
c
      nur=0
      nuz=2
      call coeff_parder(tr,nsr,tz,nsz,cc,kspl,kspl,nur,nuz,
     . cc02,lw02,ier)
c
      nur=1
      nuz=1
      call coeff_parder(tr,nsr,tz,nsz,cc,kspl,kspl,nur,nuz,
     . cc11,lw11,ier)
c
c     scaling of f_poloidal
c
c  icocos=0: adapt fpol to force Ip sign, otherwise maintain fpol
      if (icocosmod.ne.0) sgnbphi=sign(1.0_wp_,fpol(nr))
      btrcen=sign(btrcen,sgnbphi)
c
      do i=1,nr
        fpol(i)=sgnbphi*abs(fpol(i))*factb
        wf(i)=1.0_wp_
      end do
      wf(nr)=1.0e2_wp_
c
c     spline interpolation of fpol(i)
c
      iopt=0
      xb=0.0_wp_
      xe=1.0_wp_
      ssfp=0.01_wp_
      call curfit(iopt,nr,psinr,fpol,wf,xb,xe,kspl,ssfp,nrest,nsft,
     . tfp,cfp,fp,wrkf,lwrkf,iwrkf,ier)
c
      call splev(tfp,nsft,cfp,3,psinr,fpoli,nr,ier)
      fpolas=fpoli(nr)
c
c     read plasma boundaries from eqdsk file
c
      read (neqdsk,*) nbbbs,nlim
c
      call alloc_bnd(ierr)
      if (ierr.ne.0) stop
c
      if(nbbbs.gt.0) then
        if(ipsinorm.eq.1)
     .   read(neqdsk,*) (rbbbs(i),zbbbs(i),i=1,nbbbs)
        if(ipsinorm.eq.0)
     .   read(neqdsk,2020) (rbbbs(i),zbbbs(i),i=1,nbbbs)
c          do i=1,nbbbs
c            write(51,*) rbbbs(i),zbbbs(i)
c          end do
      end if
      if(nlim.gt.0) then
        if(ipsinorm.eq.1)
     .   read(neqdsk,*) (rlim(i),zlim(i),i=1,nlim)
        if(ipsinorm.eq.0) 
     .   read(neqdsk,2020) (rlim(i),zlim(i),i=1,nlim)
      end if
c
c     compute max and min z of last closed surface
c
      rbmin=rmaxis
      rbmax=rmaxis
      if (nbbbs.gt.1) then
        zbmin=1.0e+30_wp_
        zbmax=-1.0e+30_wp_
        do i=1,nbbbs
          if(zbbbs(i).le.zbmin) then
            zbmin=zbbbs(i)
            rbmin=rbbbs(i)
          end if
          if(zbbbs(i).ge.zbmax) then
            zbmax=zbbbs(i)
            rbmax=rbbbs(i)
          end if
        end do
      else
        zbmin=-1.0e+30_wp_
        zbmax=1.0e+30_wp_
      end if
      if(zbmin.le.zmnm) zbmin=zbmin+dz
      if(rbmin.le.rmnm) rbmin=rbmin+dr
      if(zbmax.ge.zmxm) zbmax=zbmax-dz
      if(rbmax.ge.rmxm) rbmax=rbmax-dr
c
c     start with uncorrected normalized psi
c
      psinop=0.0_wp_
      psinxp=1.0_wp_
      psiant=1.0_wp_
c
c     search for O-point
c
      call points_ox(rmaxis,zmaxis,rmop,zmop,psinoptmp,info)
      rmaxis=rmop
      zmaxis=zmop
      print'(a,2f8.4,es12.5)','O-point',rmop,zmop,psinoptmp
c
c     search for X-point   if ixp not = 0
c
      if(ixp.ne.0) then
        if(ixp.lt.0) then
          r10=rbmin
          z10=zbmin
          call points_ox(r10,z10,rxp,zxp,psinxptmp,info)
          if(psinxp.ne.-1.0_wp_) then
            print'(a,2f8.4,es12.5)','X-point',rxp,zxp,psinxptmp
            rbmin=rxp
            zbmin=zxp
            psinop=psinoptmp
            psinxp=psinxptmp
            psiant=psinxp-psinop
            psin1=1.0_wp_
            r10=rmaxis
            z10=(zbmax+zmaxis)/2.0_wp_
            call points_tgo(r10,z10,r1,z1,psin1,info)
            rbmax=r1
            zbmax=z1
          else
            ixp=0
c            print'(a)','no X-point'
           end if
        else
          r10=rmop
          z10=zbmax
          call points_ox(r10,z10,rxp,zxp,psinxptmp,info)
          if(psinxp.ne.-1.0_wp_) then
            print'(a,2f8.4,e16.8)','X-point',rxp,zxp,psinxptmp
            zbmax=zxp
            psinop=psinoptmp
            psinxp=psinxptmp
            psiant=psinxp-psinop
            psin1=1.0_wp_
            z10=(zbmin+zmaxis)/2.0_wp_
            call points_tgo(r10,z10,r1,z1,psin1,info)
            zbmin=z1
          else
            ixp=0
c           print'(a)','no X-point'
           end if
        end if
      end if
c
      if (ixp.eq.0) then
        psin1=1.0_wp_
        psinop=psinoptmp
        psiant=psin1-psinop
        r10=rmaxis
        z10=(zbmax+zmaxis)/2.0_wp_
        call points_tgo(r10,z10,r1,z1,psin1,info)
        zbmax=z1
        rbmax=r1
        z10=(zbmin+zmaxis)/2.0_wp_
        call points_tgo(r10,z10,r1,z1,psin1,info)
        zbmin=z1
        rbmin=r1
        print'(a,4f8.4)','no X-point  ',rbmin,zbmin,rbmax,zbmax
      end if
      print*,' '

c     compute B_toroidal on axis

      btaxis=fpol(1)/rmaxis
      btrcen=abs(btrcen)*factb
      print'(a,f8.4)','factb =   ',factb
      print'(a,f8.4)','BT_centr= ',btrcen
      print'(a,f8.4)','BT_axis = ',btaxis

c     compute normalized rho_tor from eqdsk q profile
      call rhotor(nr)
      phitedge=abs(psia)*rhotsx*2*pi
c      rrtor=sqrt(phitedge/abs(btrcen)/pi)
      call rhopol
c      print*,rhotsx,phitedge,rrtor,abs(psia)

c     compute flux surface averaged quantities

      rup=rmaxis
      rlw=rmaxis
      zup=zmaxis+(zbmax-zmaxis)/10.0_wp_
      zlw=zmaxis-(zmaxis-zbmin)/10.0_wp_
      call flux_average
c      ipr=1
c      call contours_psi(1.0_wp_,rcon,zcon,ipr)
c      do ii=1,2*np+1
c        write(52,*) rcon(ii), zcon(ii)
c      end do
c
c     locate  psi surface for q=1.5 and q=2

      rup=rmaxis
      rlw=rmaxis
      zup=(zbmax+zmaxis)/2.0_wp_
      zlw=(zmaxis+zbmin)/2.0_wp_
      q2=2.0_wp_
      q15=1.5_wp_
      call vmaxmini(qpsi,nr,qmin,qmax,iqmn,iqmx)
      if (q15.gt.qmin.and.q15.lt.qmax) then
        call surfq(q15,psi15)
        rhot15=frhotor(psi15)
        print'(3(a,f8.5))','psi_1.5 = ',psi15,'  rhop_1.5 = '
     .  ,sqrt(psi15),'  rhot_1.5 = ',rhot15
      end if
      if (q2.gt.qmin.and.q2.lt.qmax) then
        call surfq(q2,psi2)
        rhot2=frhotor(psi2)
        print'(3(a,f8.5))','psi_2   = ',psi2,'  rhop_2   = '
     .   ,sqrt(psi2),'  rhot_2   = ',rhot2
      end if
c
c     locate  btot=bres
c
      call bfield_res
c
      deallocate(iwrk,iwrkf,iwrkbsp,fpoli,fvpsi,ffvpsi,fpol,
     . wrkf,wf,wrk,wrkbsp,pres,ffprim,pprim)
      return
      end
c
c
c
      subroutine points_ox(rz,zz,rf,zf,psinvf,info)
      use const_and_precisions, only : wp_,comp_eps
      use minpack, only : hybrj1
      implicit none
c local constants
      integer, parameter :: n=2,ldfjac=n,lwa=(n*(n+13))/2
c arguments
      integer :: info
      real(wp_) :: rz,zz,rf,zf,psinvf
c local variables
      real(wp_) :: tol
      real(wp_), dimension(n) :: xvec,fvec
      real(wp_), dimension(lwa) :: wa
      real(wp_), dimension(ldfjac,n) :: fjac
c common/external functions/variables
      real(wp_) :: psinv
c
      external fcnox
c
      common/psival/psinv
c
      xvec(1)=rz
      xvec(2)=zz
      tol = sqrt(comp_eps)
      call hybrj1(fcnox,n,xvec,fvec,fjac,ldfjac,tol,info,wa,lwa)
      if(info.gt.1)  then
        print'(a,i2,a,2f8.4)','    info subr points_ox =',info,
     .   '  O/X coord.',xvec
      end if
      rf=xvec(1)
      zf=xvec(2)
      call equinum_psi(rf,zf)
      psinvf=psinv
      return
c
      end
c
c
c
      subroutine fcnox(n,x,fvec,fjac,ldfjac,iflag)
      use const_and_precisions, only : wp_
      use interp_eqprof, only: psia
      implicit none
c arguments
      integer :: n,iflag,ldfjac
      real(wp_), dimension(n) :: x,fvec
      real(wp_), dimension(ldfjac,n) :: fjac
c common/external functions/variables
      real(wp_) :: dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz
c
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
c
      select case(iflag)
      case(1)
          call equinum_derpsi(x(1),x(2),iflag)
          fvec(1) = dpsidr/psia
          fvec(2) = dpsidz/psia    
      case(2)
          call equinum_derpsi(x(1),x(2),iflag)
          fjac(1,1) = ddpsidrr/psia
          fjac(1,2) = ddpsidrz/psia
          fjac(2,1) = ddpsidrz/psia
          fjac(2,2) = ddpsidzz/psia
      case default
      print*,'iflag undefined'
      end select

      return
      end
c
c
c
      subroutine points_tgo(rz,zz,rf,zf,psin,info)
      use const_and_precisions, only : wp_,comp_eps
      use minpack, only : hybrj1
      implicit none
c local constants
      integer, parameter :: n=2,ldfjac=n,lwa=(n*(n+13))/2
c arguments
      integer :: info
      real(wp_) :: rz,zz,rf,zf,psin
c local variables
      real(wp_) :: tol
      real(wp_), dimension(n) :: xvec,fvec
      real(wp_), dimension(lwa) :: wa
      real(wp_), dimension(ldfjac,n) :: fjac
c common/external functions/variables
      real(wp_) :: h
c
      external fcntgo
c
      common/cnpsi/h
c
      h=psin
      xvec(1)=rz
      xvec(2)=zz
      tol = sqrt(comp_eps)
      call hybrj1(fcntgo,n,xvec,fvec,fjac,ldfjac,tol,info,wa,lwa)
      if(info.gt.1)  then
      end if
      rf=xvec(1)
      zf=xvec(2)
      return
      end
c
c
c
      subroutine fcntgo(n,x,fvec,fjac,ldfjac,iflag)
      use const_and_precisions, only : wp_
      use interp_eqprof, only: psia
      implicit none
c arguments
      integer :: n,ldfjac,iflag
      real(wp_), dimension(n) :: x,fvec
      real(wp_), dimension(ldfjac,n) :: fjac
c internal variables
      integer :: ii
c common/external functions/variables
      real(wp_) :: dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz,
     . psinv,h
c
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
      common/psival/psinv
      common/cnpsi/h
c
      select case(iflag)
        case(1)
          call equinum_psi(x(1),x(2))
          call equinum_derpsi(x(1),x(2),iflag)
          fvec(1) = psinv-h
          fvec(2) = dpsidr/psia
        case(2)
          ii=iflag+1
          call equinum_derpsi(x(1),x(2),ii)
          fjac(1,1) = dpsidr/psia
          fjac(1,2) = dpsidz/psia
          fjac(2,1) = ddpsidrr/psia
          fjac(2,2) = ddpsidrz/psia
        case default
          print*,'iflag undefined'
      end select
c
      return
      end
c
c
c
      subroutine print_prof
      use const_and_precisions, only : wp_
      use interp_eqprof, only : psinr,qpsi,nr
      use utils, only : intlin
      implicit none
c local constants
      real(wp_), parameter :: eps=1.e-4_wp_
c local variables
      integer :: i,ips,nst
      real(wp_) :: psin,rhop,rhot,ajphi,te,qq
c common/external functions/variables
      real(wp_) :: dens,ddens
      real(wp_) :: frhotor,temp
c
      common/dens/dens,ddens
c
      write(55,*) ' #psi rhot ne Te q Jphi'
      psin=0.0_wp_
      rhop=0.0_wp_
      rhot=0.0_wp_
      call density(psin)
      call tor_curr_psi(eps,ajphi)
      te=temp(psin)
      qq=qpsi(1)
c
      write(55,111) psin,rhot,dens,te,qq,ajphi*1.0e-6_wp_
c
      nst=nr
      do i=2,nst
        psin=dble(i-1)/dble(nst-1)
        rhop=sqrt(psin)
c
        call density(psin)
        te=temp(psin)
c
        ips=int((nr-1)*psin+1)
        if(i.lt.nst) then
          call intlin(psinr(ips),qpsi(ips),psinr(ips+1),qpsi(ips+1),
     .            psin,qq)
        else
          qq=qpsi(nr)
        end if
        rhot=frhotor(psin)
        call tor_curr_psi(psin,ajphi)
        write(55,111) psin,rhot,dens,te,qq,ajphi*1.0e-6_wp_
      end do
c
      return
 111  format(12(1x,e12.5))
      end
c
c
c
      subroutine print_prof_an
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: nst=51
c local variables
      integer :: i
      real(wp_) :: psin,rhop,rhot,te
c common/external functions/variables
      real(wp_) :: dens,ddens
      real(wp_) :: temp,frhotor
c
      common/dens/dens,ddens
c
      write(55,*) ' #psi rhot ne Te'
      do i=1,nst
        psin=dble(i-1)/dble(nst-1)
        rhop=sqrt(psin)
        rhot=frhotor(psin)
        call density(psin)
        te=temp(psin)
        write(55,111) psin,rhot,dens,te
      end do
c
      return
 111  format(12(1x,e12.5))
      end
c
c
c
      subroutine surfq(qval,psival)
      use const_and_precisions, only : wp_
      use magsurf_data, only : npoints
      use interp_eqprof, only : nr,psinr,qpsi
      use utils, only : locate, intlin
      implicit none
c arguments
      real(wp_) qval,psival
c local variables
      integer ncnt,i1,ipr
      real(wp_), dimension(npoints) :: rcn,zcn
c
      ncnt=(npoints-1)/2
c
c     locate  psi surface for q=qval
c
      call locate(qpsi,nr,qval,i1)
      call intlin(qpsi(i1),psinr(i1),qpsi(i1+1),psinr(i1+1),
     .  qval,psival)
      ipr=1
      call contours_psi(psival,rcn,zcn,ipr)
c
      return
      end
c
c
c
      subroutine bfield_res
      use const_and_precisions, only : wp_
      use interp_eqprof, only : rv,zv,nr,nz,btotal
      implicit none
c local constants
      integer, parameter :: icmx=2002
c local variables
      integer j,k,n,nconts,inc,nctot
      integer, dimension(10) :: ncpts
      real(wp_) btmx,btmn,zzk,rrj,bbphi,bbr,bbz,bbb
      real(wp_), dimension(icmx) :: rrcb,zzcb
c common/external functions/variables
      real(wp_) :: bres
c
      common/parbres/bres
c
c     Btotal on psi grid
c
      btmx=-1.0e30_wp_
      btmn=1.0e30_wp_
      do j=1,nr
        rrj=rv(j)
        do k=1,nz
          zzk=zv(k)
          call bfield(rrj,zzk,bbphi,bbr,bbz)
          btotal(j,k)=sqrt(bbr**2+bbz**2+bbphi**2)
          if(btotal(j,k).ge.btmx) btmx=btotal(j,k)
          if(btotal(j,k).le.btmn) btmn=btotal(j,k)
c          write(90,113) j,rrj,zzk,btotal(j,k)
        enddo
c          write(90,*) ' '
       enddo
c
c     compute Btot=Bres and Btot=Bres/2
c
      write(70,*)'#i Btot  R  z'
      do n=1,5
        bbb=bres/dble(n)
        if (bbb.ge.btmn.and.bbb.le.btmx) then
          call cniteq(bbb,nconts,ncpts,nctot,rrcb,zzcb,1)
          do inc=1,nctot
            write(70,113) inc,bbb,rrcb(inc),zzcb(inc)
          end do
        end if
        write(70,*) ' '
      end do
c
      return
 113  format(i6,12(1x,e12.5))
      end
c
c
      subroutine profdata
      use const_and_precisions, only : wp_
      use graydata_flags, only : iprof,iscal,nprof
      use graydata_par, only : psdbnd,factb,factt,factn
      use interp_eqprof, only : nsfd,npp,psnpp,aa,bb,cc,
     . psrad,derad,terad,zfc,tfn,cfn,ct,cz,alloc_profvec
      use simplespline, only : difcsn
      use dierckx, only : curfit,splev,splder
      implicit none
c local variables
      integer ierr,i,iopt,ier,kspl,npest,lwrkf,nu,nn,nn1,nn2
      integer, dimension(:), allocatable :: iwrkf
      real(wp_) aat,aan,ffact,psrad0,terad0,derad0,zfc0,psradi,
     . teradi,deradi,zfci,xb,xe,sspl,dnpp,ddnpp,d2dnpp,dpsb,fp
      real(wp_), dimension(:), allocatable :: wf,wrkf,wrkfd,densi,
     . ddensi,d2densi
c
c     read plasma profiles from file if iprof>0
c
      if (iscal.eq.0) then
        aat=2.0_wp_/3.0_wp_
        aan=4.0_wp_/3.0_wp_
      else
        aat=1.0_wp_
        aan=1.0_wp_
      end if
      ffact=factb
      if(iscal.eq.2) ffact=1.0_wp_
c
      if (iprof.gt.0) then
        read(nprof,*) npp
c
        call alloc_profvec(ierr)
        if (ierr.ne.0) stop
c
        npest=npp+4
        lwrkf=npp*4+npest*16
        if(allocated(wrkf)) deallocate(wrkf)
        if(allocated(iwrkf)) deallocate(iwrkf)
        if(allocated(wf)) deallocate(wf)
        if(allocated(wrkfd)) deallocate(wrkfd)
        if(allocated(densi)) deallocate(densi)
        if(allocated(ddensi)) deallocate(ddensi)
        if(allocated(d2densi)) deallocate(d2densi)
        allocate(wrkf(lwrkf),iwrkf(npest),wf(npp),densi(npest),
     .   wrkfd(npest),ddensi(npest),d2densi(npest))
c
        read(nprof,*) psrad0,terad0,derad0,zfc0
        if(psrad0.ne.0.0_wp_) psrad0=0.0_wp_
        psrad(1)=psrad0
        terad(1)=terad0*ffact**aat*factt
        derad(1)=derad0*ffact**aan*factn
        zfc(1)=zfc0
        wf(1)=1.0_wp_
        do i=2,npp
          read(nprof,*) psradi,teradi,deradi,zfci
          psrad(i)=psradi
          terad(i)=teradi*ffact**aat*factt
          derad(i)=deradi*ffact**aan*factn
          zfc(i)=zfci
          wf(i)=1.0_wp_
        end do
c
c     spline approximation of  temperature and Zeff
c
        iopt=0
        call difcsn(psrad,terad,npp,npp,iopt,ct,ier)
c
        iopt=0
        call difcsn(psrad,zfc,npp,npp,iopt,cz,ier)
c
c     spline approximation of density
c
        iopt=0
        xb=0.0_wp_
        xe=psrad(npp)
        kspl=3
        sspl=.001_wp_
c 
        call curfit(iopt,npp,psrad,derad,wf,xb,xe,kspl,sspl,npest,nsfd,
     .   tfn,cfn,fp,wrkf,lwrkf,iwrkf,ier)
c
        call splev(tfn,nsfd,cfn,3,psrad,densi,npp,ier)
        nu=1
        call splder(tfn,nsfd,cfn,3,nu,psrad,ddensi,npp,wrkfd,ier)
        dnpp=densi(npp)
        ddnpp=ddensi(npp)
c
        nu=2
        call splder(tfn,nsfd,cfn,3,nu,psrad,d2densi,npp,wrkfd,ier)
        d2dnpp=d2densi(npp)

        if(derad(npp).eq.0.0_wp_) then
          psdbnd=psrad(npp)
        else
          psnpp=psrad(npp)
          dpsb=-psnpp+psdbnd
          nn=3
          nn1=nn+1
          nn2=nn+2
          aa=(nn1*nn2*dnpp+2*nn1*ddnpp*dpsb+d2dnpp*dpsb**2)
          aa=aa/(-dpsb)**nn/2.0_wp_
          bb=-(nn*nn2*dnpp+(2*nn+1)*ddnpp*dpsb+d2dnpp*dpsb**2)
          bb=bb/(-dpsb)**nn1
          cc=(nn1*nn*dnpp+2*nn*ddnpp*dpsb+d2dnpp*dpsb**2)
          cc=cc/(-dpsb)**nn2/2.0_wp_
        end if
c
      end if
c
      deallocate(iwrkf,wrkf,wf,densi,ddensi,d2densi,wrkfd)
      
      return
      end
c
c
c
      subroutine rhotor(nnr)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : nr,psinr,qpsi,crhot,cq
      use simplespline, only : difcsn
      implicit none
c arguments
      integer nnr
c local variables
      integer iopt,ier,k
      real(wp_) dx,drhot
      real(wp_), dimension(:), allocatable :: rhotnr
c common/external functions/variables
      real(wp_) rhotsx
c
      common/rhotsx/rhotsx
c
      if(allocated(cq)) deallocate(cq)
      if(allocated(crhot)) deallocate(crhot)
      allocate(rhotnr(nr),cq(nr,4),crhot(nr,4))
c
c     normalized toroidal rho : ~ Integral q(psi) dpsi
c
      iopt=0
      call difcsn(psinr,qpsi,nr,nnr,iopt,cq,ier)
c
      rhotnr(1)=0.0_wp_
      do k=1,nnr-1
        dx=psinr(k+1)-psinr(k)
        drhot=dx*(cq(k,1)+dx*(cq(k,2)/2.0_wp_+dx*(cq(k,3)/3.0_wp_
     .        +dx*cq(k,4)/4.0_wp_)))
        rhotnr(k+1)=rhotnr(k)+drhot
      end do
      rhotsx=rhotnr(nnr)
      do k=1,nr
        rhotnr(k)=sqrt(rhotnr(k)/rhotnr(nnr))
      end do
c
c     spline interpolation of rhotor
c
      iopt=0
      call difcsn(psinr,rhotnr,nr,nnr,iopt,crhot,ier)
c      
      deallocate(rhotnr)
c
      return
      end
c
c
c
      function fq_eq(psi)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : psinr,nr,cq
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) psi,fq_eq
c local variables
      integer irt
      real(wp_) dps
c
      irt=int((nr-1)*psi+1)
      if(irt.eq.0) irt=1
      if(irt.eq.nr) irt=nr-1
      dps=psi-psinr(irt)
      fq_eq=spli(cq,nr,irt,dps)
c
      return
      end
c
c
c
      function frhotor_eq(psi)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : psinr,nr,crhot
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: psi,frhotor_eq
c local variables
      integer :: irt
      real(wp_) :: dps
c
      irt=int((nr-1)*psi+1)
c      if(irt.eq.0) irt=1
c      if(irt.eq.nr) irt=nr-1
      irt=min(max(1,irt),nr-1)
      dps=psi-psinr(irt)
      frhotor_eq=spli(crhot,nr,irt,dps)
c
      return
      end
c
c
c
      function frhotor(psi)
      use const_and_precisions, only : wp_
      use graydata_flags, only : iequil
      implicit none
c arguments
      real(wp_) :: psi,frhotor
c common/external functions/variables
      real(wp_) :: frhotor_eq,frhotor_an
c
      if(iequil.eq.2) frhotor=frhotor_eq(psi)
      if(iequil.eq.1) frhotor=frhotor_an(sqrt(psi))
c
      return
      end
c
c
c
      function frhotor_av(psi)
      use const_and_precisions, only : wp_
      use magsurf_data, only : rpstab, crhotq, npsi
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: psi,frhotor_av
c local variables
      integer :: ip
      real(wp_) :: rpsi,dps
c
      rpsi=sqrt(psi)
      ip=int((npsi-1)*rpsi+1)
c      if(ip.eq.0) ip=1
c      if(ip.eq.npsi) ip=npsi-1
      ip=min(max(1,ip),npsi-1)
      dps=rpsi-rpstab(ip)
      frhotor_av=spli(crhotq,npsi,ip,dps)
c
      return
      end
c
c
c
      subroutine rhopol
      use dierckx, only : curfit,splev
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: nnr=101,nrest=nnr+4,
     . lwrkp=nnr*4+nrest*16
c local variables
      integer :: i,iopt,ier,kspl
      integer, dimension(nrest) :: iwrkp
      real(wp_) dr,psin,xb,xe,ss,rp
      real(wp_), dimension(nnr) :: rhop,rhot,rhopi
      real(wp_), dimension(lwrkp) :: wrkp
      real(wp_), dimension(nrest) :: wp
c common/external functions/variables
      integer :: nsrp
      real(wp_), dimension(nrest) :: trp,crp
      real(wp_) :: frhotor
c
      common/coffrtp/trp
      common/coffrn/nsrp
      common/coffrp/crp
c     
      dr=1.0_wp_/dble(nnr-1)
      do i=1,nnr
        rhop(i)=(i-1)*dr
        psin=rhop(i)*rhop(i)
        rhot(i)=frhotor(psin)
        wp(i)=1.0_wp_
      end do 
        wp(1)=1.0e3_wp_
        wp(nnr)=1.0e3_wp_
      
c     spline interpolation of rhopol versus rhotor
      iopt=0
      xb=0.0_wp_
      xe=1.0_wp_
      ss=0.00001_wp_
      kspl=3
      call curfit(iopt,nnr,rhot,rhop,wp,xb,xe,kspl,ss,nrest,nsrp,
     . trp,crp,rp,wrkp,lwrkp,iwrkp,ier)
c      print*,ier
      call splev(trp,nsrp,crp,3,rhot,rhopi,nnr,ier)
c      do i=1,nnr
c        write(54,*) rhop(i),rhot(i),rhopi(i)
c      end do 
c
      return
      end
c
c
c
      function frhopol(rhot)
      use dierckx, only : splev
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: nnr=101,nrest=nnr+4
c arguments
      real(wp_) :: rhot,frhopol
c local variables
      integer :: ier
      real(wp_), dimension(1) :: rrs,ffspl
c common/external functions/variables
      integer :: nsrp
      real(wp_), dimension(nrest) :: trp,crp
c
      common/coffrtp/trp
      common/coffrn/nsrp
      common/coffrp/crp
c
      rrs(1)=rhot
      call splev(trp,nsrp,crp,3,rrs,ffspl,1,ier)
      frhopol=ffspl(1)       
      return
c
      end
c
c
c
      subroutine cniteq(h,ncon,npts,icount,rcon,zcon,ichoi)
c v2.01 12/07/95 -- written by d v bartlett, jet joint undertaking.
c                   (based on an older code)
c
      use const_and_precisions, only : wp_
      use interp_eqprof, only : dr,dz,nr,nz,rqgrid=>rv,zqgrid=>zv,psin,
     . btotal
      implicit none
c local constants
      integer, parameter :: icmx=2002
c arguments
      integer :: ncon,icount,ichoi
      integer, dimension(10) :: npts
      real(wp_) :: h
      real(wp_), dimension(icmx) :: rcon,zcon
c local variables
      integer i,j,k,l,ico,nrqmax,nreq,nzeq,iclast,mpl,ix,jx,
     . mxr,n1,jm,jfor,lda,ldb,jabs,jnb,kx,ikx,itm,inext,in
      integer, dimension(3,2) :: ja
      integer, dimension(1000) :: lx
      real(wp_) :: drgrd,dzgrd,ah,adn,px,x,y
      real(wp_), dimension(:), allocatable :: a
c
      data px/0.5d0/
c
      if(allocated(a)) deallocate(a)
      allocate(a(nr*nz))
c      
      if(ichoi.eq.0) then
         do j=1,nz
          do i=1,nr
            a(nr*(j-1)+i)=psin(i,j)
          enddo
        enddo
      endif
c
      if(ichoi.eq.1) then
         do j=1,nz
          do i=1,nr
            a(nr*(j-1)+i)=btotal(i,j)
          enddo
        enddo
      endif
c
      do ico=1,icmx
        rcon(ico)=0.0_wp_
        zcon(ico)=0.0_wp_
      enddo
c
      nrqmax=nr
      nreq=nr
      nzeq=nz
      drgrd=dr
      dzgrd=dz
c
      ncon = 0
      do  i=1,10
        npts(i) = 0
      end do
      iclast = 0
      icount = 0
      mpl=0
      ix=0
      mxr = nrqmax * (nzeq - 1)
      n1 = nreq - 1
c
      do 3 jx=2,n1
      do 3 jm=jx,mxr,nrqmax
          j = jm + nrqmax
          ah=a(j)-h
          if(ah) 60,60,61
   60     if(a(jm)-h) 62,62,1
   61     if(a(jm)-h) 1,1,63
    1     ix=ix+1
          lx(ix)=-j
          if(ah) 62,62,63
   62     if(a(j-1)-h) 3,3,4
   63     if(a(j-1)-h) 4,4,3
    4     ix=ix+1
          lx(ix)=j
    3 continue
c
      do 79 jm=nreq,mxr,nrqmax
         j = jm + nrqmax
         ah=a(j)-h
         if(ah) 64,64,65
   64    if(a(j-1)-h) 66,66,76
   65    if(a(j-1)-h) 76,76,67
   76    ix=ix+1
         lx(ix)=j
         if(ah) 66,66,67
   66    if(a(jm)-h) 79,79,78
   67    if(a(jm)-h) 78,78,79
   78    ix=ix+1
         lx(ix)=-j
   79 continue
c
      do 75 jm=1,mxr,nrqmax
         j = jm + nrqmax
         if(a(j)-h) 68,68,69
   68    if(a(jm)-h)75,75,71
   69    if(a(jm)-h)71,71,75
   71    ix=ix+1
         lx(ix) =-j
   75 continue
c
      do 73 j=2,nreq
         if(a(j)-h) 58,58,59
   58    if(a(j-1)-h) 73,73,72
   59    if(a(j-1)-h) 72,72,73
   72    ix=ix+1
         lx(ix)=j
   73 continue
c
      if(ix) 50,50,8
  108 if(mpl.lt.4) go to 8
      ncon = ncon + 1
      npts(ncon) = icount - iclast
      iclast = icount
    8 in=ix
      jx=lx(in)
      jfor=0
      lda=1
      ldb=2
   30 if(jx) 21,22,22
   21 jabs=-jx
      jnb = jabs - nrqmax
      go to 23
   22 jabs=jx
      jnb=jabs-1
   23 adn=a(jabs)-a(jnb)
      if(adn) 24,9,24
   24 px=(a(jabs)-h)/adn
    9 kx = (jabs - 1) / nrqmax
      ikx = jabs - nrqmax * kx - 1
      if(jx) 25,26,26
   25 x = drgrd * ikx
      y = dzgrd * (kx - px)
      go to 27
   26 x = drgrd * (ikx - px)
      y = dzgrd * kx
   27 continue
      icount = icount + 1
      rcon(icount) = x + rqgrid(1)
      zcon(icount) = y + zqgrid(1)
      mpl= icount
      itm = 1
      ja(1,1) = jabs + nrqmax
      j=1
      if(jx) 10,10,11
   10 ja(1,1) = -jabs-1
      j=2
   11 ja(2,1)=-ja(1,1)
      ja(3,1) = -jx + 1 - nrqmax
      ja(3,2) = -jx
      ja(j,2) = jabs - nrqmax
      k= 3-j
      ja(k,2) = 1-jabs
      if (kx) 14,14,39
   39 if(ikx) 14,14,36
   36 if(ikx + 1 - nreq) 35, 37, 37
   37 if(jx) 38,38,35
   35 if(jfor) 28,29,28
   28 do 13 i=1,3
         if(jfor-ja(i,2)) 13,14,13
   13 continue
   38 lda=2
      go to 15
   14 lda=1
   15 ldb=lda
   29 do 18 k=1,3
         do 18 l=lda,ldb
            do 16 i=1,ix
            if(lx(i)-ja(k,l)) 16,17,16
   16    continue
         go to 18
   17    itm=itm+1
         inext= i
         if(jfor) 19,33,19
   33    if(itm .gt. 3) goto 20
   18 continue
   19 lx(in)=0
      if(itm .eq. 1) goto 6
   20 jfor=jx
      jx=lx(inext)
      in = inext
      go to 30
    6 if(lx(ix)) 108,7,108
    7 ix= ix-1
      if(ix) 51,51,6
   51 if(mpl.lt.4) go to  50
      ncon = ncon + 1
      npts(ncon) = icount - iclast
      iclast = icount
   50 continue
c
      deallocate(a)
c
      return
      end
c
c
c
      subroutine contours_psi(h,rcn,zcn,ipr)
      use const_and_precisions, only : wp_,pi
      use graydata_par, only : rwallm
      use magsurf_data, only : npoints
      use interp_eqprof, only : psiant,psinop,nsr=>nsrt,nsz=>nszt,
     . cc=>cceq,tr,tz,rup,zup,rlw,zlw,nr
      use dierckx, only : profil,sproota
      implicit none
c local constants
      integer, parameter :: mest=4,kspl=3
c arguments
      integer :: ipr
      real(wp_) :: h
      real(wp_), dimension(npoints) :: rcn,zcn
c local variables
      integer :: np,info,ic,ier,ii,iopt,m
      real(wp_) :: ra,rb,za,zb,th,zc,val
      real(wp_), dimension(mest) :: zeroc
      real(wp_), dimension(:), allocatable :: czc
c
      np=(npoints-1)/2
      if(allocated(czc)) deallocate(czc)
      allocate(czc(nr+4))
c
      ra=rup
      rb=rlw
      za=zup
      zb=zlw
      call points_tgo(ra,za,rup,zup,h,info)
      call points_tgo(rb,zb,rlw,zlw,h,info)
c
      th=pi/dble(np)
      rcn(1)=rlw
      zcn(1)=zlw
      rcn(npoints)=rlw
      zcn(npoints)=zlw
      rcn(np+1)=rup
      zcn(np+1)=zup
      do ic=2,np
        zc=zlw+(zup-zlw)*(1.0_wp_-cos(th*(ic-1)))/2.0_wp_
        iopt=1
        call profil(iopt,tr,nsr,tz,nsz,cc,kspl,kspl,zc,nsr,czc,ier)
        if(ier.gt.0) print*,'    profil =',ier
        val=h*psiant+psinop
        call sproota(val,tr,nsr,czc,zeroc,mest,m,ier)      
        if (zeroc(1).gt.rwallm) then
        rcn(ic)=zeroc(1)
        zcn(ic)=zc
        rcn(npoints+1-ic)=zeroc(2)
        zcn(npoints+1-ic)=zc
        else
        rcn(ic)=zeroc(2)
        zcn(ic)=zc
        rcn(npoints+1-ic)=zeroc(3)
        zcn(npoints+1-ic)=zc        
        end if
      end do
      if (ipr.gt.0) then
      do ii=1,npoints
        write(71,111) ii,h,rcn(ii),zcn(ii)
      end do
        write(71,*)
        write(71,*)
      end if
c
      deallocate(czc)
c
      return
111   format(i6,12(1x,e12.5))
      end
c
c
c
      subroutine flux_average
      use const_and_precisions, only : wp_,zero,one,pi,ccj=>mu0inv
      use magsurf_data
      use interp_eqprof, only : btrcen,btaxis,rmaxis,zmaxis,phitedge
      use simplespline, only : difcs
      use dierckx, only : regrid,coeff_parder
      implicit none
c local constants
      integer, parameter :: nnintp=101,ncnt=100,nlam=41,ksp=3,
     . njest=nnintp+ksp+1,nlest=nlam+ksp+1,
     . lwrk=4*(nnintp+nlam)+11*(njest+nlest)+njest*nnintp+nlest+54,
     . kwrk=nnintp+nlam+njest+nlest+3,lw01=nnintp*4+nlam*3+nnintp*nlam
c local variables
      integer ier,ierr,l,jp,ipr,jpr,inc,inc1,iopt,njp,nlm,ninpr
      integer, dimension(kwrk) :: iwrk
      real(wp_) :: lam,dlam,anorm,b2av,dvdpsi,dadpsi,ratio_cdator,
     . ratio_cdbtor,ratio_pltor,qq,fc,rhot2q,height,height2,r2iav,currp,
     . area,volume,ajphiav,bbav,bmmx,bmmn,btot0,bpoloid0,rpsim0,dla,dlb,
     . dlp,drc,ph,area2,rzp,rz,rpsim,zpsim,btot,bpoloid,dlph,ajphi0,
     . shlam,srl,rl2,rl0,rl,dhlam,dhlam0,ccfh,s,frhotor_eq,fq_eq,ajphi,
     . bphi,brr,bzz,riav,fp
      real(wp_), dimension(nnintp) :: dadrhotv,dvdrhotv,vratjpl
      real(wp_), dimension(2*ncnt) :: dlpv
      real(wp_), dimension(2*ncnt+1) :: bv,bpv
      real(wp_), dimension(nlam) :: alam,weights
      real(wp_), dimension(nnintp,nlam) :: fhlam
      real(wp_), dimension(nnintp*nlam) :: ffhlam,dffhlam
      real(wp_), dimension(lwrk) :: wrk
      real(wp_), dimension(:), allocatable :: rctemp,zctemp
c common/external functions/variables
      real(wp_) :: fpolv,ffpv
c
      common/fpol/fpolv,ffpv
c
      npsi=nnintp
      ninpr=(npsi-1)/10
      npoints = 2*ncnt+1
c
      call alloc_surfvec(ierr)
      if(allocated(tjp)) deallocate(tjp)
      if(allocated(tlm)) deallocate(tlm)
      if(allocated(ch)) deallocate(ch)
      if(allocated(ch01)) deallocate(ch01)
      allocate(tjp(njest),tlm(nlest),ch((njest-4)*(nlest-4)),
     . ch01(lw01),rctemp(npoints),zctemp(npoints),stat=ierr)
      if (ierr.ne.0) return

c     computation of flux surface averaged quantities

      write(71,*)' #i psin R  z'

      dlam=1.0_wp_/dble(nlam-1)
      do l=1,nlam-1
        alam(l)=dble(l-1)*dlam
        fhlam(1,l)=sqrt(1.0_wp_-alam(l))
        ffhlam(l)=fhlam(1,l)
        dffhlam(l)=-0.5_wp_/sqrt(1.0_wp_-alam(l))
        weights(l)=1.0_wp_
      end do 
      weights(1)=0.5_wp_
      weights(nlam)=0.5_wp_
      alam(nlam)=1.0_wp_
      fhlam(1,nlam)=0.0_wp_
      ffhlam(nlam)=0.0_wp_
      dffhlam(nlam)=-99999.0_wp_

      jp=1
      anorm=2.0_wp_*pi*rmaxis/abs(btaxis)
      b2av=btaxis**2
      dvdpsi=2.0_wp_*pi*anorm
      dadpsi=2.0_wp_*pi/abs(btaxis)
      ratio_cdator=abs(btaxis/btrcen)
      ratio_cdbtor=1.0_wp_
      ratio_pltor=1.0_wp_
      qq=1.0_wp_
      fc=1.0_wp_
      
      psicon(1)=0.0_wp_
      rcon(1,:)=0.0_wp_
      zcon(1,:)=0.0_wp_
      pstab(1)=0.0_wp_
      rhot_eq(1)=0.0_wp_
      rpstab(1)=0.0_wp_
      rhotqv(1)=0.0_wp_
      vcurrp(1)=0.0_wp_
      vajphiav(1)=0.0_wp_
      bmxpsi(1)=abs(btaxis)
      bmnpsi(1)=abs(btaxis)
      bav(1)=abs(btaxis)
      rbav(1)=1.0_wp_
      rri(1)=rmaxis
      varea(1)=0.0_wp_
      vvol(1)=0.0_wp_
      vratjpl(1)=ratio_pltor
      vratja(1)=ratio_cdator
      vratjb(1)=ratio_cdbtor
      ffc(1)=fc
      rhot2q=0.0_wp_
      qqv(1)=1.0_wp_
      
      dadrhotv(1)=0.0_wp_
      dvdrhotv(1)=0.0_wp_

c      rup=rmaxis
c      rlw=rmaxis
c      zup=(zbmax+zmaxis)/2.0_wp_
c      zlw=(zmaxis+zbmin)/2.0_wp_

      do jp=2,npsi
        height=dble(jp-1)/dble(npsi-1)
        psicon(jp)=height
        if(jp.eq.npsi) height=0.9999_wp_
        ipr=0
        jpr=mod(jp,ninpr)
        if(jpr.eq.1) ipr=1
        height2=height*height
        call contours_psi(height2,rctemp,zctemp,ipr)
        rcon(jp,:) = rctemp
        zcon(jp,:) = zctemp
c
        r2iav=0.0_wp_
        anorm=0.0_wp_
        dadpsi=0.0_wp_
        currp=0.0_wp_
        b2av=0.0_wp_
        area=0.0_wp_
        volume=0.0_wp_
        ajphiav=0.0_wp_
        bbav=0.0_wp_
        bmmx=-1.0e+30_wp_
        bmmn=1.0e+30_wp_

        call tor_curr(rctemp(1),zctemp(1),ajphi0)
        call bpol(rctemp(1),zctemp(1),brr,bzz)
        call equinum_fpol(0)
        bphi=fpolv/rctemp(1)
        btot0=sqrt(bphi**2+brr**2+bzz**2)
        bpoloid0=sqrt(brr**2+bzz**2)
        bv(1)=btot0
        bpv(1)=bpoloid0
        rpsim0=rctemp(1)

        do inc=1,npoints-1
          inc1=inc+1
          dla=sqrt((rctemp(inc)-rmaxis)**2+(zctemp(inc)-zmaxis)**2)
          dlb=sqrt((rctemp(inc1)-rmaxis)**2+(zctemp(inc1)-zmaxis)**2)
          dlp=sqrt((rctemp(inc1)-rctemp(inc))**2+
     .             (zctemp(inc1)-zctemp(inc))**2)
          drc=(rctemp(inc1)-rctemp(inc))

c         compute length, area and volume defined by psi=height^2

          ph=0.5_wp_*(dla+dlb+dlp)
          area2=ph*(ph-dla)*(ph-dlb)*(ph-dlp)
          area=area+sqrt(area2)
          rzp=rctemp(inc1)*zctemp(inc1)
          rz=rctemp(inc)*zctemp(inc)
          volume=pi*(rzp+rz)*drc+volume

c         compute line integral on the contour psi=height^2

          rpsim=rctemp(inc1)
          zpsim=zctemp(inc1)
          call bpol(rpsim,zpsim,brr,bzz)
          call tor_curr(rpsim,zpsim,ajphi)
          call equinum_fpol(0)
          bphi=fpolv/rpsim
          btot=sqrt(bphi**2+brr**2+bzz**2)
          bpoloid=sqrt(brr**2+bzz**2)
          dlpv(inc)=dlp
          bv(inc1)=btot
          bpv(inc1)=bpoloid
          
          dlph=0.5_wp_*dlp
          anorm=anorm+dlph*(1.0_wp_/bpoloid+1.0_wp_/bpoloid0)
          dadpsi=dadpsi+dlph*
     .          (1.0_wp_/(bpoloid*rpsim)+1.0_wp_/(bpoloid0*rpsim0))
          currp=currp+dlph*(bpoloid+bpoloid0)
          b2av=b2av+dlph*(btot0**2/bpoloid0+btot**2/bpoloid)
          bbav=bbav+dlph*(btot/bpoloid+btot0/bpoloid0)
          r2iav=r2iav+dlph*(1.0_wp_/(bpoloid*rpsim**2)
     .          +1.0_wp_/(bpoloid0*rpsim0**2))
          ajphiav=ajphiav+dlph*
     .            (ajphi0/(bpoloid0*rpsim0)+ajphi/(bpoloid*rpsim))
          
          ajphi0=ajphi
          rpsim0=rpsim
          bpoloid0=bpoloid
          btot0=btot        

c       computation maximum/minimum B values on given flux surface

          if(btot.le.bmmn) bmmn=btot
          if(btot.ge.bmmx) bmmx=btot
        end do       

c       bav=<B> [T] ,  b2av=<B^2> [T^2] , rbav=<B>/b_min
c       anorm = int d l_p/B_p = dV/dpsi/(2pi)
c       r2iav=<1/R^2> [m^-2] ,
c       riav=<1/R> [m^-1] = dA/dpsi/(dV/dpsi/(2pi)),
c       rri = <B>/(|R B_tor|<1/R^2>) , used to compute I_tor  [m^-1]
c       currp = plasma current within psi=const

        bbav=bbav/anorm
        r2iav=r2iav/anorm
        dvdpsi=2.0_wp_*pi*anorm
        riav=dadpsi/anorm
        b2av=b2av/anorm
        vcurrp(jp)=ccj*currp
        vajphiav(jp)=ajphiav/dadpsi

c     area == varea, volume == vvol
c     flux surface minor radius == (area/pi)^1/2
c     ratio_cdator = Jcd_astra/J_phi      Jcd_astra = <Jcd.B>/B0
c     ratio_cdbtor = Jcd_jintrac/J_phi    Jcd_jintrac = <Jcd.B>/<B>
c     ratio_pltor  = Jcd_||/J_phi         Jcd_|| = <Jcd.b>

        pstab(jp)=height2
        rpstab(jp)=height
        vvol(jp)=abs(volume)
        varea(jp)=area
        bav(jp)=bbav
        rbav(jp)=bbav/bmmn
        bmxpsi(jp)=bmmx
        bmnpsi(jp)=bmmn
        rri(jp)=bav(jp)/abs(fpolv*r2iav)
        ratio_cdator=abs(b2av*riav/(fpolv*r2iav*btrcen))
        ratio_cdbtor=abs(b2av*riav/(fpolv*r2iav*bbav))
        ratio_pltor=abs(bbav*riav/(fpolv*r2iav))
        vratjpl(jp)=ratio_pltor
        vratja(jp)=ratio_cdator
        vratjb(jp)=ratio_cdbtor
        qq=abs(dvdpsi*fpolv*r2iav/(4*pi*pi))
        qqv(jp)=qq
        
        dadrhotv(jp)=phitedge*frhotor_eq(height2)/fq_eq(height2)
     .   *dadpsi/pi
        dvdrhotv(jp)=phitedge*frhotor_eq(height2)/fq_eq(height2)
     .   *dvdpsi/pi

c       computation of rhot from calculated q profile 

        rhot2q=rhot2q+(qqv(jp)*rpstab(jp)+qqv(jp-1)*rpstab(jp-1))
     .   /dble(npsi-1)
        rhotqv(jp)=sqrt(rhot2q)
        
c  computation of fraction of circulating/trapped fraction fc, ft 
c  and of function H(lambda,rhop)
c  ffhlam = Bmn/Bmx/fc integral_lambda^1 dlam/<sqrt(1-lam*B(rhop)/Bmx)>

        fc=0.0_wp_
        shlam=0.0_wp_
        do l=nlam,1,-1
          lam=alam(l)
          srl=0.0_wp_
          rl2=1.0_wp_-lam*bv(1)/bmmx
          rl0=0.0_wp_
          if(rl2.gt.0) rl0=sqrt(rl2)
          do inc=1,npoints-1
            rl2=1.0_wp_-lam*bv(inc+1)/bmmx
            rl=0.0_wp_
            if(rl2.gt.0) rl=sqrt(rl2)
            srl=srl+0.5_wp_*dlpv(inc)*(rl/bpv(inc+1)+rl0/bpv(inc))
            rl0=rl
          end do
          srl=srl/anorm
          dhlam=0.5_wp_/srl
          fc=fc+lam/srl*weights(l)
          if(l.eq.nlam) then
            fhlam(jp,l)=0.0_wp_
            ffhlam(nlam*(jp-1)+l)=0.0_wp_
            dffhlam(nlam*(jp-1)+l)=-dhlam
            dhlam0=dhlam
          else
            shlam=shlam+0.5_wp_*(dhlam+dhlam0)*dlam
            fhlam(jp,l)=shlam
            dffhlam(nlam*(jp-1)+l)=-dhlam
            dhlam0=dhlam
          end if
        end do
        fc=0.75_wp_*b2av/bmmx**2*fc*dlam
        ffc(jp)=fc
        
        ccfh=bmmn/bmmx/fc
        do l=1,nlam
          ffhlam(nlam*(jp-1)+l)=ccfh*fhlam(jp,l)
          dffhlam(nlam*(jp-1)+l)=ccfh*dffhlam(nlam*(jp-1)+l)
        end do
      end do

      write(56,*)' #psi rhot_eq rhot_av |<B>| |Bmx| |Bmn|'//
     .'  Area Vol |I_pl| <J_phi> qq fc ratioJa ratioJb'

      qqv(1)=qqv(2)
      vajphiav(1)=vajphiav(2)
      do jp=1,npsi
        rhotqv(jp)=rhotqv(jp)/rhotqv(npsi)
        if(jp.eq.npsi) then
          rhotqv(jp)=1.0_wp_
          rpstab(jp)=1.0_wp_
          pstab(jp)=1.0_wp_
        end if
        rhot_eq(jp)=frhotor_eq(pstab(jp))
        write(56,99) pstab(jp),rhot_eq(jp),rhotqv(jp),
     .    bav(jp),bmxpsi(jp),bmnpsi(jp),varea(jp),vvol(jp), 
     .    vcurrp(jp),vajphiav(jp),qqv(jp),ffc(jp),
     .    vratja(jp),vratjb(jp)
      end do 

c      rarea=sqrt(varea(npsi)/pi)

c     spline coefficients of vvol,rbav,rri,bmxpsi,bmnpsi
c     used for computations of dP/dV and J_cd
c     spline coefficients of rhot

        iopt=0
        call difcs(rpstab,vvol,npsi,iopt,cvol,ier)
        iopt=0
        call difcs(rpstab,rbav,npsi,iopt,crbav,ier)
        iopt=0
        call difcs(rpstab,rri,npsi,iopt,crri,ier)
        iopt=0
        call difcs(rpstab,bmxpsi,npsi,iopt,cbmx,ier)
        iopt=0
        call difcs(rpstab,bmnpsi,npsi,iopt,cbmn,ier)
        iopt=0
        call difcs(rpstab,vratja,npsi,iopt,cratja,ier)
        iopt=0
        call difcs(rpstab,vratjb,npsi,iopt,cratjb,ier)
        iopt=0
        call difcs(rpstab,vratjpl,npsi,iopt,cratjpl,ier)
        iopt=0
        call difcs(rpstab,varea,npsi,iopt,carea,ier)
        iopt=0
        call difcs(rpstab,ffc,npsi,iopt,cfc,ier)
        iopt=0
        call difcs(rpstab,rhotqv,npsi,iopt,crhotq,ier)
        iopt=0
        call difcs(rpstab,dadrhotv,npsi,iopt,cdadrhot,ier)
        iopt=0
        call difcs(rpstab,dvdrhotv,npsi,iopt,cdvdrhot,ier)

c   spline interpolation of H(lambda,rhop) and dH/dlambda

        iopt=0
        s=0.0_wp_
        call regrid(iopt,npsi,rpstab,nlam,alam,ffhlam,
     .   zero,one,zero,one,ksp,ksp,s,
     .   njest,nlest,njp,tjp,nlm,tlm,ch,fp,wrk,lwrk,iwrk,kwrk,ier)       
        njpt=njp
        nlmt=nlm
        
        call coeff_parder(tjp,njp,tlm,nlm,ch,ksp,ksp,0,1,
     .                    ch01,lw01,ier)
c 
      return
 99   format(20(1x,e12.5))
      end
c
c
c
      function fdadrhot(rpsi)
      use const_and_precisions, only : wp_
      use magsurf_data, only : rpstab,cdadrhot,npsi
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: rpsi,fdadrhot
c local variables
      integer :: ip
      real(wp_) :: dps
c
      ip=int((npsi-1)*rpsi+1)
c      if(ip.eq.0) ip=1
c      if(ip.eq.npsi) ip=npsi-1
      ip=min(max(1,ip),npsi-1)
      dps=rpsi-rpstab(ip)
      fdadrhot=spli(cdadrhot,npsi,ip,dps)
      return
      end
c
c
c
      function fdvdrhot(rpsi)
      use const_and_precisions, only : wp_
      use magsurf_data, only : rpstab,cdvdrhot,npsi
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: rpsi,fdvdrhot
c local variables
      integer :: ip
      real(wp_) :: dps
c
      ip=int((npsi-1)*rpsi+1)
      ip=min(max(1,ip),npsi-1)
      dps=rpsi-rpstab(ip)
      fdvdrhot=spli(cdvdrhot,npsi,ip,dps)
c
      return
      end
c
c
c      
      subroutine flux_average_an
      use const_and_precisions, only : wp_,zero,one,pi,ccj=>mu0inv
      use graydata_anequil, only : q0,qa,alq,b0,rr0m,zr0m,rpam
      use magsurf_data
      use interp_eqprof, only : btrcen
      use simplespline, only : difcs
      use dierckx, only : regrid,coeff_parder
      implicit none
c local constants
      integer, parameter :: nnintp=101,ncnt=100,nlam=41,ksp=3,
     . njest=nnintp+ksp+1,nlest=nlam+ksp+1,
     . lwrk=4*(nnintp+nlam)+11*(njest+nlest)+njest*nnintp+nlest+54,
     . kwrk=nnintp+nlam+njest+nlest+3,lw01=nnintp*4+nlam*3+nnintp*nlam
c local variables
      integer ier,ierr,l,jp,ipr,jpr,inc,inc1,iopt,njp,nlm,ninpr
      integer, dimension(kwrk) :: iwrk
      real(wp_) :: lam,dlam,anorm,b2av,dvdpsi,dadpsi,ratio_cdator,
     . ratio_cdbtor,ratio_pltor,qq,fc,rhot2q,height2,r2iav,currp,area,
     . volume,ajphiav,bbav,bmmx,bmmn,btot0,bpoloid0,rpsim0,dla,dlb,dlp,
     . drc,ph,area2,rzp,rz,rpsim,zpsim,btot,bpoloid,dlph,ajphi0,shlam,
     . srl,rl2,rl0,rl,dhlam,dhlam0,ccfh,s,ajphi,bphi,brr,bzz,riav,fp,
     . btaxis,dadr,dbvcdc13,dbvcdc31,dvdr,height,phitedge,qqan,rmaxis,
     . zmaxis,rn
      real(wp_), dimension(nnintp) :: dadrhotv,dvdrhotv,vratjpl
      real(wp_), dimension(2*ncnt) :: dlpv
      real(wp_), dimension(2*ncnt+1) :: bv,bpv
      real(wp_), dimension(nlam) :: alam,weights
      real(wp_), dimension(nnintp,nlam) :: fhlam
      real(wp_), dimension(nnintp*nlam) :: ffhlam,dffhlam
      real(wp_), dimension(lwrk) :: wrk
      real(wp_), dimension(:), allocatable :: rctemp,zctemp
c common/external functions/variables
      real(wp_) :: dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz,fpolv,ffpv
      real(wp_) :: frhotor_an
c
      common/fpol/fpolv,ffpv
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
c
      npsi=nnintp
      ninpr=(npsi-1)/10
      npoints = 2*ncnt+1
c
      call alloc_surfvec(ierr)
      if(allocated(tjp)) deallocate(tjp)
      if(allocated(tlm)) deallocate(tlm)
      if(allocated(ch)) deallocate(ch)
      allocate(tjp(njest),tlm(nlest),ch((njest-4)*(nlest-4)),
     . rctemp(npoints),zctemp(npoints),stat=ierr)
      if (ierr.ne.0) return
c
c     computation of flux surface averaged quantities

      rmaxis=rr0m
      zmaxis=zr0m
      btaxis=b0

      call rhopol_an
      phitedge=pi*rpam*rpam*btaxis
       
      write(71,*)' #i psin R  z'

      dlam=1.0_wp_/dble(nlam-1)
      do l=1,nlam-1
        alam(l)=dble(l-1)*dlam
        fhlam(1,l)=sqrt(1.0_wp_-alam(l))
        ffhlam(l)=fhlam(1,l)
        dffhlam(l)=-0.5_wp_/sqrt(1.0_wp_-alam(l))
        weights(l)=1.0_wp_
      end do 
      weights(1)=0.5_wp_
      weights(nlam)=0.5_wp_
      alam(nlam)=1.0_wp_
      fhlam(1,nlam)=0.0_wp_
      ffhlam(nlam)=0.0_wp_
      dffhlam(nlam)=-99999.0_wp_

      jp=1
      anorm=2.0_wp_*pi*rmaxis/abs(btaxis)
      b2av=btaxis**2
      dvdpsi=2.0_wp_*pi*anorm
      dadpsi=2.0_wp_*pi/abs(btaxis)
      ratio_cdator=abs(btaxis/btrcen)
      ratio_cdbtor=1.0_wp_
      ratio_pltor=1.0_wp_
      qq=1.0_wp_
      fc=1.0_wp_
      
      psicon(1)=0.0_wp_
      rcon(1,:)=0.0_wp_
      zcon(1,:)=0.0_wp_
      pstab(1)=0.0_wp_
      rhot_eq(1)=0.0_wp_
      rpstab(1)=0.0_wp_
      rhotqv(1)=0.0_wp_
      vcurrp(1)=0.0_wp_
      vajphiav(1)=0.0_wp_
      bmxpsi(1)=abs(btaxis)
      bmnpsi(1)=abs(btaxis)
      bav(1)=abs(btaxis)
      rbav(1)=1.0_wp_
      rri(1)=rmaxis
      varea(1)=0.0_wp_
      vvol(1)=0.0_wp_
      vratjpl(1)=ratio_pltor
      vratja(1)=ratio_cdator
      vratjb(1)=ratio_cdbtor
      ffc(1)=fc
      rhot2q=0.0_wp_
      dadrhotv(1)=0.0_wp_
      dvdrhotv(1)=0.0_wp_
      qqv(1)=1.0_wp_
      
c      rup=rmaxis
c      rlw=rmaxis
c      zup=(zbmax+zmaxis)/2.0_wp_
c      zlw=(zmaxis+zbmin)/2.0_wp_
      
      do jp=2,npsi
        height=dble(jp-1)/dble(npsi-1)
        psicon(jp)=height
        if(jp.eq.npsi) height=0.9999_wp_
        height2=height*height
        ipr=0
        jpr=mod(jp,ninpr)
        if(jpr.eq.1) ipr=1
        call contours_psi_an(height2,rctemp,zctemp,ipr)
        rcon(jp,:) = rctemp
        zcon(jp,:) = zctemp
c
        r2iav=0.0_wp_
        anorm=0.0_wp_
        dadpsi=0.0_wp_
        currp=0.0_wp_
        b2av=0.0_wp_
        area=0.0_wp_
        volume=0.0_wp_
        ajphiav=0.0_wp_
        bbav=0.0_wp_
        bmmx=-1.0e+30_wp_
        bmmn=1.0e+30_wp_

        call equian(rctemp(1),zctemp(1))
        dbvcdc13=-ddpsidzz/rctemp(1)
        dbvcdc31= ddpsidrr/rctemp(1)-bzz/rctemp(1)
        ajphi=ccj*(dbvcdc13-dbvcdc31)
        brr=-dpsidz/rctemp(1)
        bzz= dpsidr/rctemp(1)
        bphi=fpolv/rctemp(1)
        btot0=sqrt(bphi**2+brr**2+bzz**2)
        bpoloid0=sqrt(brr**2+bzz**2)
        bv(1)=btot0
        bpv(1)=bpoloid0
        rpsim0=rctemp(1)

        do inc=1,npoints-1
          inc1=inc+1
          dla=sqrt((rctemp(inc)-rmaxis)**2+(zctemp(inc)-zmaxis)**2)
          dlb=sqrt((rctemp(inc1)-rmaxis)**2+(zctemp(inc1)-zmaxis)**2)
          dlp=sqrt((rctemp(inc1)-rctemp(inc))**2+
     .             (zctemp(inc1)-zctemp(inc))**2)
          drc=(rctemp(inc1)-rctemp(inc))

c         compute length, area and volume defined by psi=height^2

          ph=0.5_wp_*(dla+dlb+dlp)
          area2=ph*(ph-dla)*(ph-dlb)*(ph-dlp)
          area=area+sqrt(area2)
          rzp=rctemp(inc1)*zctemp(inc1)
          rz=rctemp(inc)*zctemp(inc)
          volume=pi*(rzp+rz)*drc+volume

c         compute line integral on the contour psi=height^2

          rpsim=rctemp(inc1)
          zpsim=zctemp(inc1)
          call equian(rpsim,zpsim)
          brr=-dpsidz/rpsim
          bzz= dpsidr/rpsim
          dbvcdc13=-ddpsidzz/rpsim
          dbvcdc31= ddpsidrr/rpsim-bzz/rpsim
          ajphi=ccj*(dbvcdc13-dbvcdc31)
          bphi=fpolv/rpsim
          btot=sqrt(bphi**2+brr**2+bzz**2)
          bpoloid=sqrt(brr**2+bzz**2)
          dlpv(inc)=dlp
          bv(inc1)=btot
          bpv(inc1)=bpoloid
          
          dlph=0.5_wp_*dlp
          anorm=anorm+dlph*(1.0_wp_/bpoloid+1.0_wp_/bpoloid0)
          dadpsi=dadpsi+dlph*
     .          (1.0_wp_/(bpoloid*rpsim)+1.0_wp_/(bpoloid0*rpsim0))
          currp=currp+dlph*(bpoloid+bpoloid0)
          b2av=b2av+dlph*(btot0**2/bpoloid0+btot**2/bpoloid)
          bbav=bbav+dlph*(btot/bpoloid+btot0/bpoloid0)
          r2iav=r2iav+dlph*(1.0_wp_/(bpoloid*rpsim**2)
     .          +1.0_wp_/(bpoloid0*rpsim0**2))
          ajphiav=ajphiav+dlph*
     .            (ajphi0/(bpoloid0*rpsim0)+ajphi/(bpoloid*rpsim))
          
          ajphi0=ajphi
          rpsim0=rpsim
          bpoloid0=bpoloid
          btot0=btot        
c       computation maximum/minimum B values on given flux surface

          if(btot.le.bmmn) bmmn=btot
          if(btot.ge.bmmx) bmmx=btot
        end do       

c       bav=<B> [T] ,  b2av=<B^2> [T^2] , rbav=<B>/b_min
c       anorm = int d l_p/B_p = dV/dpsi/(2pi)
c       r2iav=<1/R^2> [m^-2] ,
c       riav=<1/R> [m^-1] = dA/dpsi/(dV/dpsi/(2pi)),
c       rri = <B>/(|R B_tor|<1/R^2>) , used to compute I_tor  [m^-1]
c       currp = plasma current within psi=const

        bbav=bbav/anorm
        r2iav=r2iav/anorm
        dvdpsi=2.0_wp_*pi*anorm
        riav=dadpsi/anorm
        b2av=b2av/anorm
        vcurrp(jp)=ccj*currp
        vajphiav(jp)=ajphiav/dadpsi

c     area == varea, volume == vvol
c     flux surface minor radius == (area/pi)^1/2
c     ratio_cdator = Jcd_astra/J_phi      Jcd_astra = <Jcd.B>/B0
c     ratio_cdbtor = Jcd_jintrac/J_phi    Jcd_jintrac = <Jcd.B>/<B>
c     ratio_pltor  = Jcd_||/J_phi         Jcd_|| = <Jcd.b>

        pstab(jp)=height2
        rpstab(jp)=height
        vvol(jp)=abs(volume)
        varea(jp)=area
        bav(jp)=bbav
        rbav(jp)=bbav/bmmn
        bmxpsi(jp)=bmmx
        bmnpsi(jp)=bmmn
        rri(jp)=bav(jp)/abs(fpolv*r2iav)
        ratio_cdator=abs(b2av*riav/(fpolv*r2iav*btrcen))
        ratio_cdbtor=abs(b2av*riav/(fpolv*r2iav*bbav))
        ratio_pltor=abs(bbav*riav/(fpolv*r2iav))
        vratjpl(jp)=ratio_pltor
        vratja(jp)=ratio_cdator
        vratjb(jp)=ratio_cdbtor
        qq=abs(dvdpsi*fpolv*r2iav/(4.0_wp_*pi*pi))
        qqv(jp)=qq
        
        rn=frhotor_an(sqrt(pstab(jp)))
        qqan=q0+(qa-q0)*rn**alq

        dadr=2.0_wp_*pi*rn*rpam*rpam
        dvdr=4.0_wp_*pi*pi*rn*rmaxis*rpam*rpam

c       dadrhotv(jp)=rpam*rpam*btaxis*rn/qqan*dadpsi
c       dvdrhotv(jp)=rpam*rpam*btaxis*rn/qqan*dvdpsi
        dadrhotv(jp)=phitedge*rn*dadpsi/pi/qqan
        dvdrhotv(jp)=phitedge*rn*dvdpsi/pi/qqan

c       computation of rhot from calculated q profile 
        rhot2q=rhot2q+(qqv(jp)*rpstab(jp)+qqv(jp-1)*rpstab(jp-1))
     .         /dble(npsi-1)
c        print*,jp,rhot2q,qqv(jp),rpstab(jp),qqv(jp-1),rpstab(jp-1)
        rhotqv(jp)=sqrt(rhot2q)
c                 rhotqv(jp)=rn
c        
      write(57,99) rpstab(jp),rn,rhotqv(jp),qqv(jp),qqan,r2iav,
     . dadr,dadrhotv(jp),dadpsi,dvdpsi,fpolv
        
c  computation of fraction of circulating/trapped fraction fc, ft 
c  and of function H(lambda,rhop)
c  ffhlam = Bmn/Bmx/fc integral_lambda^1 dlam/<sqrt(1-lam*B(rhop)/Bmx)>

        fc=0.0_wp_
        shlam=0.0_wp_
        do l=nlam,1,-1
          lam=alam(l)
          srl=0.0_wp_
          rl2=1.0_wp_-lam*bv(1)/bmmx
          rl0=0.0_wp_
          if(rl2.gt.0) rl0=sqrt(rl2)
          do inc=1,npoints-1
            rl2=1.0_wp_-lam*bv(inc+1)/bmmx
            rl=0.0_wp_
            if(rl2.gt.0) rl=sqrt(rl2)
            srl=srl+0.5_wp_*dlpv(inc)*(rl/bpv(inc+1)+rl0/bpv(inc))
            rl0=rl
          end do
          srl=srl/anorm
          dhlam=0.5_wp_/srl
          fc=fc+lam/srl*weights(l)
          if(l.eq.nlam) then
            fhlam(jp,l)=0.0_wp_
            ffhlam(nlam*(jp-1)+l)=0.0_wp_
            dffhlam(nlam*(jp-1)+l)=-dhlam
            dhlam0=dhlam
          else
            shlam=shlam+0.5_wp_*(dhlam+dhlam0)*dlam
            fhlam(jp,l)=shlam
            dffhlam(nlam*(jp-1)+l)=-dhlam
            dhlam0=dhlam
          end if
        end do
        fc=0.75_wp_*b2av/bmmx**2*fc*dlam
        ffc(jp)=fc
        
        ccfh=bmmn/bmmx/fc
        do l=1,nlam
          ffhlam(nlam*(jp-1)+l)=ccfh*fhlam(jp,l)
          dffhlam(nlam*(jp-1)+l)=ccfh*dffhlam(nlam*(jp-1)+l)
        end do
      end do

      write(56,*)' #psi rhot_eq rhot_av |<B>| |Bmx| |Bmn|'//
     .'  Area Vol |I_pl| <J_phi> qq fc ratioJa ratioJb dadr dvdr'

      qqv(1)=qqv(2)
      vajphiav(1)=vajphiav(2)
      do jp=1,npsi
        rhotqv(jp)=rhotqv(jp)/rhotqv(npsi)
        if(jp.eq.npsi) then
              rhotqv(jp)=1.0_wp_
              rpstab(jp)=1.0_wp_
              pstab(jp)=1.0_wp_
        end if
        rhot_eq(jp)=frhotor_an(sqrt(pstab(jp)))
        write(56,99) pstab(jp),rhot_eq(jp),rhotqv(jp),
     .    bav(jp),bmxpsi(jp),bmnpsi(jp),varea(jp),vvol(jp), 
     .    vcurrp(jp),vajphiav(jp),qqv(jp),ffc(jp),
     .    vratja(jp),vratjb(jp),dadrhotv(jp),dvdrhotv(jp)
      end do 

c      rarea=sqrt(varea(npsi)/pi)

c     spline coefficients of vvol,rbav,rri,bmxpsi,bmnpsi
c     used for computations of dP/dV and J_cd
c     spline coefficients of rhot

        iopt=0
        call difcs(rpstab,vvol,npsi,iopt,cvol,ier)
        iopt=0
        call difcs(rpstab,rbav,npsi,iopt,crbav,ier)
        iopt=0
        call difcs(rpstab,rri,npsi,iopt,crri,ier)
        iopt=0
        call difcs(rpstab,bmxpsi,npsi,iopt,cbmx,ier)
        iopt=0
        call difcs(rpstab,bmnpsi,npsi,iopt,cbmn,ier)
        iopt=0
        call difcs(rpstab,vratja,npsi,iopt,cratja,ier)
        iopt=0
        call difcs(rpstab,vratjb,npsi,iopt,cratjb,ier)
        iopt=0
        call difcs(rpstab,vratjpl,npsi,iopt,cratjpl,ier)
        iopt=0
        call difcs(rpstab,varea,npsi,iopt,carea,ier)
        iopt=0
        call difcs(rpstab,ffc,npsi,iopt,cfc,ier)
        iopt=0
        call difcs(rpstab,rhotqv,npsi,iopt,crhotq,ier)
        iopt=0
        call difcs(rpstab,dadrhotv,npsi,iopt,cdadrhot,ier)
        iopt=0
        call difcs(rpstab,dvdrhotv,npsi,iopt,cdvdrhot,ier)

c   spline interpolation of H(lambda,rhop)

        iopt=0
        s=0.0_wp_
        call regrid(iopt,npsi,rpstab,nlam,alam,ffhlam,
     .   zero,one,zero,one,ksp,ksp,s,
     .   njest,nlest,njp,tjp,nlm,tlm,ch,fp,wrk,lwrk,iwrk,kwrk,ier)       
        njpt=njp
        nlmt=nlm
        
      return
 99   format(20(1x,e12.5))
      end
c
c
c
      subroutine rhopol_an
      use dierckx, only : curfit,splev
      use const_and_precisions, only : wp_
      use graydata_par, only : sgniphi
      use graydata_anequil, only : q0,qa,alq,b0,rr0m,rpam
      use interp_eqprof, only : psia
      implicit none
c local constants
      integer, parameter :: nnr=101,nrest=nnr+4,lwrk=nnr*4+nrest*16
c local variables
      integer :: i,ier,iopt,kspl
      integer, dimension(nrest) :: iwrk
      real(wp_) :: dr,xb,xe,ss,rp,fq0,fq1,qq,res,rn
      real(wp_), dimension(nnr) :: rhop,rhot,rhopi,rhoti,psin
      real(wp_), dimension(lwrk) :: wrk
      real(wp_), dimension(nrest) :: wp
c common/external functions/variables
      integer :: nsrp,nsrot
      real(wp_), dimension(nrest) :: trp,crp,trot,crot
c
      common/coffrtp/trp
      common/coffrn/nsrp
      common/coffrp/crp
      common/coffrptt/trot
      common/coffrnt/nsrot
      common/coffrpt/crot
c
      dr=1.0_wp_/dble(nnr-1)
      rhot(1)=0.0_wp_
      psin(1)=0.0_wp_
      res=0.0_wp_
      fq0=0.0_wp_
      do i=2,nnr
        rhot(i)=(i-1)*dr
        rn=rhot(i)
        qq=q0+(qa-q0)*rn**alq
        fq1=rn/qq
        res=res+0.5_wp_*(fq1+fq0)*dr
        fq0=fq1
        psin(i)=res
      end do 

      wp=1.0_wp_      
      psin=psin/res
      rhop=sqrt(psin)
      psia=-sgniphi*abs(res*rpam*rpam*b0)
      print*,psia,log(8.0_wp_*rr0m/rpam)-2.0_wp_
      wp(1)=1.0e3_wp_
      wp(nnr)=1.0e3_wp_
      
c     spline interpolation of rhopol versus rhotor
      iopt=0
      xb=0.0_wp_
      xe=1.0_wp_
      ss=0.00001_wp_
      kspl=3
      call curfit(iopt,nnr,rhot,rhop,wp,xb,xe,kspl,ss,nrest,nsrp,
     . trp,crp,rp,wrk,lwrk,iwrk,ier)
      call splev(trp,nsrp,crp,3,rhot,rhopi,nnr,ier)

c     spline interpolation of rhotor versus rhopol
      iopt=0
      xb=0.0_wp_
      xe=1.0_wp_
      ss=0.00001_wp_
      kspl=3
      call curfit(iopt,nnr,rhop,rhot,wp,xb,xe,kspl,ss,nrest,nsrot,
     . trot,crot,rp,wrk,lwrk,iwrk,ier)
      call splev(trot,nsrot,crot,3,rhop,rhoti,nnr,ier)

      do i=1,nnr
        write(54,*) rhop(i),rhot(i),rhopi(i),rhoti(i)
      end do 
           
      return
      end
c
c
c
      function frhotor_an(rhop)
      use dierckx, only : splev
      use const_and_precisions, only : wp_
      implicit none
c local contants
      integer, parameter :: nnr=101,nrest=nnr+4
c arguments
      real(wp_) :: rhop,frhotor_an
c local variables
      integer :: nsrot,ier
      real(wp_), dimension(1) :: rrs(1),ffspl(1)
      real(wp_), dimension(nrest) :: trot,crot
c
      common/coffrptt/trot
      common/coffrnt/nsrot
      common/coffrpt/crot
c
      rrs(1)=rhop
      call splev(trot,nsrot,crot,3,rrs,ffspl,1,ier)
      frhotor_an=ffspl(1)       
c
      return
      end
c
c
c
      subroutine contours_psi_an(h,rcn,zcn,ipr)
      use const_and_precisions, only : wp_,pi
      use graydata_anequil, only : rr0m,zr0m,rpam
      use magsurf_data, only : npoints
      implicit none
c arguments
      integer :: ipr
      real(wp_) :: h
      real(wp_), dimension(npoints) :: rcn,zcn
c local variables
      integer :: np,ic
      real(wp_) :: rn,th
c common/external functions/variables
      real(wp_) :: frhotor_an
c
      np=(npoints-1)/2
      th=pi/dble(np)
      rn=frhotor_an(sqrt(h))
      do ic=1,npoints
        zcn(ic)=zr0m+rpam*rn*sin(th*(ic-1))
        rcn(ic)=rr0m+rpam*rn*cos(th*(ic-1))
c
        if (ipr.gt.0) write(71,111) ic,h,rcn(ic),zcn(ic)
      end do
      write(71,*)
c
      return
111   format(i6,12(1x,e12.5))
      end
c
c
c
      subroutine vectinit
      use const_and_precisions, only : wp_
      use dispersion, only: expinit
      use graydata_flags, only : iwarm
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36,nmx=8000
c local variables
      integer :: i,j,k
c common/external functions/variables
      integer nrayr,nrayth,nstep
      integer, dimension(jmx,kmx) :: iiv,iop,iow,ihcd,istore
      real(wp_) :: jclosest
      real(wp_), dimension(3) :: anwcl,xwcl
      real(wp_), dimension(jmx,kmx) :: tau1v
      real(wp_), dimension(jmx,kmx,nmx) :: psjki,tauv,alphav,pdjki,
     . ppabs,currj,didst,ccci
c
      common/iiv/iiv
      common/iov/iop,iow,ihcd,istore
      common/psjki/psjki
      common/atjki/tauv,alphav
      common/dpjjki/pdjki,currj
      common/pcjki/ppabs,ccci
      common/dcjki/didst
      common/nray/nrayr,nrayth
      common/nstep/nstep
      common/tau1v/tau1v
      common/refln/anwcl,xwcl,jclosest
c
      if(nstep.gt.nmx) nstep=nmx
      if(nrayr.gt.jmx) nrayr=jmx
      if(nrayth.gt.kmx) nrayth=kmx
      jclosest=nrayr+1
      anwcl(1:3)=0.0_wp_
      xwcl(1:3)=0.0_wp_
c
      do i=1,nstep
        do k=1,nrayth
          do j=1,nrayr
            psjki(j,k,i)=0.0_wp_
            tauv(j,k,i)=0.0_wp_
            alphav(j,k,i)=0.0_wp_
            pdjki(j,k,i)=0.0_wp_
            ppabs(j,k,i)=0.0_wp_
            didst(j,k,i)=0.0_wp_
            ccci(j,k,i)=0.0_wp_
            currj(j,k,i)=0.0_wp_
            iiv(j,k)=1
            iop(j,k)=0
            iow(j,k)=0
            ihcd(j,k)=1
            istore(j,k)=0
            tau1v(j,k)=0.0_wp_
          end do
        end do
      end do
c
      if(iwarm.gt.1) call expinit
c
      return
      end
c
c
c
      subroutine vectinit2
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36,nmx=8000
c local variables
      integer :: i,j,k
c common/external functions/variables
      integer nrayr,nrayth,nstep
      integer, dimension(jmx,kmx) :: iiv,iop,iow,ihcd,istore
      real(wp_), dimension(jmx,kmx,nmx) :: psjki,tauv,alphav,pdjki,
     . ppabs,currj,didst,ccci
c
      common/iiv/iiv
      common/iov/iop,iow,ihcd,istore
      common/psjki/psjki
      common/atjki/tauv,alphav
      common/dpjjki/pdjki,currj
      common/pcjki/ppabs,ccci
      common/dcjki/didst
      common/nray/nrayr,nrayth
      common/nstep/nstep
c      
      do i=1,nstep
        do k=1,nrayth
          do j=1,nrayr
            psjki(j,k,i)=0.0_wp_
            tauv(j,k,i)=0.0_wp_
            alphav(j,k,i)=0.0_wp_
            pdjki(j,k,i)=0.0_wp_
            ppabs(j,k,i)=0.0_wp_
            didst(j,k,i)=0.0_wp_
            ccci(j,k,i)=0.0_wp_
            currj(j,k,i)=0.0_wp_
            iiv(j,k)=1
            iop(j,k)=0
            iow(j,k)=0
            ihcd(j,k)=1
          end do
        end do
      end do

      return
      end
c
c
c
      subroutine paraminit
      use const_and_precisions, only : wp_
      implicit none
c common/external functions/variables
      integer istep,istpr,istpl,ierr,istop
c
      common/istep/istep
      common/istgr/istpr,istpl
      common/ierr/ierr
      common/istop/istop
c
      istpr=0
      istpl=1
      ierr=0
      istep=0
      istop=0
c
      return
      end
c
c
c
      subroutine updatepos
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36
c local variables
      integer j,k
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_), dimension(3,jmx,kmx) :: xc,xco,du1,du1o
      real(wp_), dimension(6,jmx,kmx) :: ywrk,ypwrk
c
      common/nray/nrayr,nrayth
      common/grco/xco,du1o
      common/grc/xc,du1
      common/wrk/ywrk,ypwrk
c
      do j=1,nrayr
        do k=1,nrayth
          if(j.eq.1.and.k.gt.1) then
            xco(1,j,k)=xco(1,j,1)
            xco(2,j,k)=xco(2,j,1)
            xco(3,j,k)=xco(3,j,1)
            xc(1,j,k)=xc(1,j,1)
            xc(2,j,k)=xc(2,j,1)
            xc(3,j,k)=xc(3,j,1)
          else
            xco(1,j,k)=xc(1,j,k)
            xco(2,j,k)=xc(2,j,k)
            xco(3,j,k)=xc(3,j,k)
            xc(1,j,k)=ywrk(1,j,k)
            xc(2,j,k)=ywrk(2,j,k)
            xc(3,j,k)=ywrk(3,j,k)
          end if
          du1o(1,j,k)=du1(1,j,k)
          du1o(2,j,k)=du1(2,j,k)
          du1o(3,j,k)=du1(3,j,k)
        end do
      end do
c
      return
      end
c
c
c
      subroutine gradi
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36
c local variables
      integer iv,j,jm,jp,k,km,kp
      real(wp_) ux,uxx,uxy,uxz,uy,uyy,uyz,uz,uzz,
     . dfu,dfuu,gx,gxx,gxy,gxz,gy,gyy,gyz,gz,gzz
      real(wp_), dimension(3) :: dxv1,dxv2,dxv3,dgu,
     . dgg1,dgg2,dgg3,df1,df2,df3
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_), dimension(jmx) :: dffiu,ddffiu
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: xc,xco,du1,du1o,gri,dgrad2v
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
c
      common/nray/nrayr,nrayth
      common/fi/dffiu,ddffiu
      common/grco/xco,du1o
      common/grc/xc,du1
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
c
c     compute  grad u and grad(S_I)
c
      do k=1,nrayth
        do j=1,nrayr
          if(j.eq.1) then
            gri(1,j,k)=0.0_wp_
            gri(2,j,k)=0.0_wp_
            gri(3,j,k)=0.0_wp_
            jp=j+1
            km=k-1
            if(k.eq.1) km=nrayth
            kp=k+1
            if(k.eq.nrayth) kp=1
            do iv=1,3
              dxv1(iv)=xc(iv,jp,k)-xc(iv,j,k)
              dxv2(iv)=xc(iv,jp,kp)-xc(iv,jp,km)
              dxv3(iv)=xc(iv,j,k)-xco(iv,j,k)
            end do
            call solg0(dxv1,dxv2,dxv3,dgu)
          else
            jm=j-1
            km=k-1
            if(k.eq.1) km=nrayth
            kp=k+1
            if(k.eq.nrayth) kp=1
            do iv=1,3
              dxv1(iv)=xc(iv,j,k)-xc(iv,jm,k)
              dxv2(iv)=xc(iv,j,kp)-xc(iv,j,km)
              dxv3(iv)=xc(iv,j,k)-xco(iv,j,k)
            end do
            call solg0(dxv1,dxv2,dxv3,dgu)
          end if
          du1(1,j,k)=dgu(1)
          du1(2,j,k)=dgu(2)
          du1(3,j,k)=dgu(3)
          gri(1,j,k)=dgu(1)*dffiu(j)
          gri(2,j,k)=dgu(2)*dffiu(j)
          gri(3,j,k)=dgu(3)*dffiu(j)
          grad2(j,k)=gri(1,j,k)**2+gri(2,j,k)**2+gri(3,j,k)**2
        end do
      end do
c
c     compute derivatives of grad u and grad(S_I)
c
      do k=1,nrayth
        do j=1,nrayr
          if(j.eq.1) then
            jp=j+1
            km=k-1
            if(k.eq.1) km=nrayth
            kp=k+1
            if(k.eq.nrayth) kp=1
            do iv=1,3
              dxv1(iv)=xc(iv,jp,kp)-xc(iv,jp,km)
              dxv2(iv)=xc(iv,jp,k)-xc(iv,j,k)
              dxv3(iv)=xc(iv,j,k)-xco(iv,j,k)
            end do
            df1(1)=du1(1,jp,kp)-du1(1,jp,km)
            df1(2)=du1(1,jp,k)-du1(1,j,k)
            df1(3)=du1(1,j,k)-du1o(1,j,k)
            df2(1)=du1(2,jp,kp)-du1(2,jp,km)
            df2(2)=du1(2,jp,k)-du1(2,j,k)
            df2(3)=du1(2,j,k)-du1o(2,j,k)
            df3(1)=du1(3,jp,kp)-du1(3,jp,km)
            df3(2)=du1(3,jp,k)-du1(3,j,k)
            df3(3)=du1(3,j,k)-du1o(3,j,k)
            call solg3(dxv1,dxv2,dxv3,df1,df2,df3,dgg1,dgg2,dgg3)
          else
            jm=j-1
            km=k-1
            if(k.eq.1) km=nrayth
            kp=k+1
            if(k.eq.nrayth) kp=1
            do iv=1,3
              dxv1(iv)=xc(iv,j,k)-xc(iv,jm,k)
              dxv2(iv)=xc(iv,j,kp)-xc(iv,j,km)
              dxv3(iv)=xc(iv,j,k)-xco(iv,j,k)
            end do
            df1(1)=du1(1,j,k)-du1(1,jm,k)
            df1(2)=du1(1,j,kp)-du1(1,j,km)
            df1(3)=du1(1,j,k)-du1o(1,j,k)
            df2(1)=du1(2,j,k)-du1(2,jm,k)
            df2(2)=du1(2,j,kp)-du1(2,j,km)
            df2(3)=du1(2,j,k)-du1o(2,j,k)
            df3(1)=du1(3,j,k)-du1(3,jm,k)
            df3(2)=du1(3,j,kp)-du1(3,j,km)
            df3(3)=du1(3,j,k)-du1o(3,j,k)
            call solg3(dxv1,dxv2,dxv3,df1,df2,df3,dgg1,dgg2,dgg3)
          end if
c
c     derivatives of u
c
          ux=du1(1,j,k)
          uy=du1(2,j,k)
          uz=du1(3,j,k)
          uxx=dgg1(1)
          uyy=dgg2(2)
          uzz=dgg3(3)
          uxy=(dgg1(2)+dgg2(1))/2.0_wp_
          uxz=(dgg1(3)+dgg3(1))/2.0_wp_
          uyz=(dgg2(3)+dgg3(2))/2.0_wp_
c
c     derivatives of S_I and Grad(S_I)
c
          dfu=dffiu(j)
          dfuu=ddffiu(j)
          gx=ux*dfu
          gy=uy*dfu
          gz=uz*dfu
          gxx=dfuu*ux*ux+dfu*uxx
          gyy=dfuu*uy*uy+dfu*uyy
          gzz=dfuu*uz*uz+dfu*uzz
          gxy=dfuu*ux*uy+dfu*uxy
          gxz=dfuu*ux*uz+dfu*uxz
          gyz=dfuu*uy*uz+dfu*uyz
c
          ggri(1,1,j,k)=gxx
          ggri(2,2,j,k)=gyy
          ggri(3,3,j,k)=gzz
          ggri(1,2,j,k)=gxy
          ggri(2,1,j,k)=gxy
          ggri(1,3,j,k)=gxz
          ggri(3,1,j,k)=gxz
          ggri(2,3,j,k)=gyz
          ggri(3,2,j,k)=gyz
c
c     derivatives of |Grad(S_I)|^2
c
          dgrad2v(1,j,k)=2.0_wp_*(gx*gxx+gy*gxy+gz*gxz)
          dgrad2v(2,j,k)=2.0_wp_*(gx*gxy+gy*gyy+gz*gyz)
          dgrad2v(3,j,k)=2.0_wp_*(gx*gxz+gy*gyz+gz*gzz)
        end do
      end do
c
      return
      end
c
c
c
      subroutine solg0(dxv1,dxv2,dxv3,dgg)
c     solution of the linear system of 3 eqs : dgg . dxv = dff
c     input vectors : dxv1, dxv2, dxv3, dff
c     output vector : dgg
c     dff=(1,0,0)
c
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_), dimension(3) :: dxv1,dxv2,dxv3,dgg
c local variables
      real(wp_) denom,aa1,aa2,aa3
c
      aa1=(dxv2(2)*dxv3(3)-dxv3(2)*dxv2(3))
      aa2=(dxv1(2)*dxv3(3)-dxv1(3)*dxv3(2))
      aa3=(dxv1(2)*dxv2(3)-dxv1(3)*dxv2(2))
      denom = dxv1(1)*aa1-dxv2(1)*aa2+dxv3(1)*aa3
      dgg(1) = aa1/denom
      dgg(2) = -(dxv2(1)*dxv3(3)-dxv3(1)*dxv2(3))/denom
      dgg(3) = (dxv2(1)*dxv3(2)-dxv3(1)*dxv2(2))/denom
c
      return
      end
c
c
c
      subroutine solg3(dxv1,dxv2,dxv3,df1,df2,df3,dg1,dg2,dg3)
c     three rhs vectors df1, df2, df3
c
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_), dimension(3) :: dxv1,dxv2,dxv3,df1,df2,df3,
     . dg1,dg2,dg3
c local variables
      real(wp_) denom,a11,a21,a31,a12,a22,a32,a13,a23,a33
c
      a11=(dxv2(2)*dxv3(3)-dxv3(2)*dxv2(3))
      a21=(dxv1(2)*dxv3(3)-dxv1(3)*dxv3(2))
      a31=(dxv1(2)*dxv2(3)-dxv1(3)*dxv2(2))
      a12=(dxv2(1)*dxv3(3)-dxv3(1)*dxv2(3))
      a22=(dxv1(1)*dxv3(3)-dxv1(3)*dxv3(1))
      a32=(dxv1(1)*dxv2(3)-dxv1(3)*dxv2(1))
      a13=(dxv2(1)*dxv3(2)-dxv3(1)*dxv2(2))
      a23=(dxv1(1)*dxv3(2)-dxv1(2)*dxv3(1))
      a33=(dxv1(1)*dxv2(2)-dxv1(2)*dxv2(1))
      denom = dxv1(1)*a11-dxv2(1)*a21+dxv3(1)*a31
      dg1(1) = (df1(1)*a11-df1(2)*a21+df1(3)*a31)/denom
      dg1(2) = -(df1(1)*a12-df1(2)*a22+df1(3)*a32)/denom
      dg1(3) = (df1(1)*a13-df1(2)*a23+df1(3)*a33)/denom
      dg2(1) = (df2(1)*a11-df2(2)*a21+df2(3)*a31)/denom
      dg2(2) = -(df2(1)*a12-df2(2)*a22+df2(3)*a32)/denom
      dg2(3) = (df2(1)*a13-df2(2)*a23+df2(3)*a33)/denom
      dg3(1) = (df3(1)*a11-df3(2)*a21+df3(3)*a31)/denom
      dg3(2) = -(df3(1)*a12-df3(2)*a22+df3(3)*a32)/denom
      dg3(3) = (df3(1)*a13-df3(2)*a23+df3(3)*a33)/denom
c
      return
      end
c
c
c
      subroutine rkint4
c     Runge-Kutta integrator
c
      use const_and_precisions, only : wp_
      use graydata_flags, only : dst,igrad
      implicit none
c parameter
      integer, parameter :: ndim=6,jmx=31,kmx=36
c local variables
      integer ieq,iv,j,jv,k,kkk
      real(wp_) h,hh,h6
      real(wp_), dimension(ndim) :: y,yy,fk1,fk2,fk3,fk4
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_) gr2
      real(wp_), dimension(3) :: dgr2,dgr
      real(wp_), dimension(3,3) :: ddgr
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: dgrad2v,gri
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk,ypwrk
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
c
      common/nray/nrayr,nrayth
      common/wrk/ywrk,ypwrk
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
      common/gr/gr2
      common/dgr/dgr2,dgr,ddgr
c
      h=dst
      hh=h*0.5_wp_
      h6=h/6.0_wp_
c
      do j=1,nrayr
        kkk=nrayth
        if(j.eq.1) kkk=1
        do k=1,kkk
          do ieq=1,ndim
            y(ieq)=ywrk(ieq,j,k)
            fk1(ieq)=ypwrk(ieq,j,k)
            yy(ieq)=y(ieq)+fk1(ieq)*hh
          end do
          gr2=grad2(j,k)
          do iv=1,3
            dgr2(iv)=dgrad2v(iv,j,k)
            dgr(iv)=gri(iv,j,k)
            do jv=1,3
              ddgr(iv,jv)=ggri(iv,jv,j,k)
            end do
          end do
          call fwork(yy,fk2)
c
          do ieq=1,ndim
            yy(ieq)=y(ieq)+fk2(ieq)*hh
          end do
          call fwork(yy,fk3)
c
          do ieq=1,ndim
            yy(ieq)=y(ieq)+fk3(ieq)*h
          end do
          call fwork(yy,fk4)
c
          do ieq=1,ndim
            ywrk(ieq,j,k)=y(ieq)
     .        +h6*(fk1(ieq)+2.0_wp_*fk2(ieq)+2.0_wp_*fk3(ieq)+fk4(ieq))
          end do
        end do
      end do
c
      call updatepos
c
      if(igrad.eq.1) call gradi
c
      return
      end
c
c
c
      subroutine gwork(j,k)
      use const_and_precisions, only : wp_
      use graydata_flags, only : igrad
      implicit none
c local constants
      integer, parameter :: ndim=6,jmx=31,kmx=36
c arguments
      integer j,k
c local variables
      integer :: ieq,iv,jv
      real(wp_), dimension(ndim) :: yy,yyp
c common/external functions/variables
      real(wp_) :: gr2
      real(wp_), dimension(3) :: dgr2,dgr
      real(wp_), dimension(3,3) :: ddgr
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: dgrad2v,gri
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk,ypwrk
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
c
      common/wrk/ywrk,ypwrk
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
      common/gr/gr2
      common/dgr/dgr2,dgr,ddgr
c
c     begin --- update vector yy
c
      do ieq=1,ndim
        yy(ieq)=ywrk(ieq,j,k)
      end do
c
      if(igrad.eq.1) then
        gr2=grad2(j,k)
        do iv=1,3
          dgr2(iv)=dgrad2v(iv,j,k)
          dgr(iv)=gri(iv,j,k)
          do jv=1,3
            ddgr(iv,jv)=ggri(iv,jv,j,k)
          end do
        end do
      end if
c
      call fwork(yy,yyp)
c
      do ieq=1,ndim
         ypwrk(ieq,j,k)=yyp(ieq)
      end do
c
c     end --- update vector yy
c
      return
      end
c
c
c
      subroutine fwork(y,dery)
      use const_and_precisions, only : wp_
      use graydata_flags, only : idst,igrad
      implicit none
c local constants
      integer, parameter :: ndim=6
c arguments
      real(wp_), dimension(ndim) :: y,dery
c local variables
      integer iv,jv
      real(wp_) :: xx,yy,zz,yg2,anpl2,anpr2,del,dnl,duh,dan2sdnpl,
     . dan2sdxg,dan2sdyg,ddelnpl2,ddelnpl2x,ddelnpl2y,denom,derdel,
     . derdom,dfdiadnpl,dfdiadxg,dfdiadyg
      real(wp_), dimension(3) :: vgv,derdxv,danpldxv,derdnv,dbgr
c common/external functions/variables
      integer ierr
      real(wp_) sox,gr2,dd,an2s,an2,fdia,bdotgr,ddi,ddr11,anpl,
     . anpr,xg,yg,vgm,derdnm,dersdst
      real(wp_), dimension(3) :: dgr2,dgr,xv,anv,bv,derxg,deryg
      real(wp_), dimension(3,3) :: ddgr,derbv
c
      common/gr/gr2
      common/dgr/dgr2,dgr,ddgr
      common/ddd/dd,an2s,an2,fdia,bdotgr,ddi,ddr11
      common/mode/sox
      common/nplr/anpl,anpr
      common/bb/bv
      common/dbb/derbv
      common/xgxg/xg
      common/ygyg/yg
      common/dxgyg/derxg,deryg
      common/vgv/vgm,derdnm
      common/dersdst/dersdst
      common/ierr/ierr
      common/anv/anv
      common/xv/xv
c
      xx=y(1)
      yy=y(2)
      zz=y(3)
      
      xv(1)=y(1)
      xv(2)=y(2)
      xv(3)=y(3)
c
      anv(1) = y(4)
      anv(2) = y(5)
      anv(3) = y(6)
c
c      rr=sqrt(xx**2+yy**2)
c      phi=acos(xx/rr)
c      if (yy.lt.0.0_wp_) then
c        phi=-phi
c      end if
c
      call plas_deriv(xx,yy,zz)
c
      an2=anv(1)*anv(1)+anv(2)*anv(2)+anv(3)*anv(3)
      anpl=anv(1)*bv(1)+anv(2)*bv(2)+anv(3)*bv(3)
c
      if(abs(anpl).gt.0.99_wp_) then
        if(abs(anpl).le.1.05_wp_) then
         ierr=97
        else 
         ierr=98
        end if 
      end if
c
      anpl2=anpl*anpl
      dnl=1.0_wp_-anpl2
      anpr2=an2-anpl2
      anpr=0.0_wp_
      if(anpr2.gt.0.0_wp_) anpr=sqrt(anpr2)
      yg2=yg**2
c
      an2s=1.0_wp_
      dan2sdxg=0.0_wp_
      dan2sdyg=0.0_wp_
      dan2sdnpl=0.0_wp_
      del=0.0_wp_
      fdia=0.0_wp_
      dfdiadnpl=0.0_wp_
      dfdiadxg=0.0_wp_
      dfdiadyg=0.0_wp_
c
      duh=1.0_wp_-xg-yg2
      if(xg.gt.0.0_wp_) then
        del=sqrt(dnl*dnl+4.0_wp_*anpl*anpl*(1.0_wp_-xg)/yg2)
        an2s=1.0_wp_-xg -xg*yg2*(1.0_wp_+anpl2+sox*del)/duh/2.0_wp_
c
        dan2sdxg=-yg2*(1.0_wp_-yg2)*(1.0_wp_+anpl2+sox*del)/
     .           duh**2/2.0_wp_+sox*xg*anpl2/(del*duh)-1.0_wp_
        dan2sdyg=-xg*yg*(1.0_wp_-xg)*(1.0_wp_+anpl2+sox*del)/duh**2
     .           +2.0_wp_*sox*xg*(1.0_wp_-xg)*anpl2/(yg*del*duh)
        dan2sdnpl=-xg*yg2*anpl/duh
     .            -sox*xg*anpl*(2.0_wp_*(1.0_wp_-xg)-yg2*dnl)/(del*duh)
c
        if(igrad.gt.0) then
        ddelnpl2=2.0_wp_*(2.0_wp_*(1.0_wp_-xg)
     .           *(1.0_wp_+3.0_wp_*anpl2**2)-yg2*dnl**3)/yg2/del**3
        fdia=-xg*yg2*(1.0_wp_+sox*ddelnpl2/2.0_wp_)/duh
        derdel=2.0_wp_*(1.0_wp_-xg)*anpl2*(1.0_wp_+3.0_wp_*anpl2**2)
     .         -dnl**2*(1.0_wp_+3.0_wp_*anpl2)*yg2
        derdel=4.0_wp_*derdel/(yg**5*del**5)
        ddelnpl2y=2.0_wp_*(1.0_wp_-xg)*derdel
        ddelnpl2x=yg*derdel
        dfdiadnpl=24.0_wp_*sox*xg*(1.0_wp_-xg)*anpl*(1.0_wp_-anpl**4)
     .            /(yg2*del**5)
        dfdiadxg=-yg2*(1.0_wp_-yg2)/duh**2-sox*yg2*((1.0_wp_-yg2)
     .           *ddelnpl2+xg*duh*ddelnpl2x)/(2.0_wp_*duh**2)
        dfdiadyg=-2.0_wp_*yg*xg*(1.0_wp_-xg)/duh**2
     .           -sox*xg*yg*(2.0_wp_*(1.0_wp_-xg)*ddelnpl2
     .           +yg*duh*ddelnpl2y)/(2.0_wp_*duh**2)
c
        end if
      end if
c
      bdotgr=0.0_wp_
      do iv=1,3
        bdotgr=bdotgr+bv(iv)*dgr(iv)
        dbgr(iv)=0.0_wp_
        do jv=1,3
          dbgr(iv)=dgr(jv)*derbv(jv,iv)+bv(jv)*ddgr(jv,iv)
        end do
      end do
c
      derdnm=0.0_wp_
c
      do iv=1,3
        danpldxv(iv)=anv(1)*derbv(1,iv)+anv(2)*derbv(2,iv)
     .               +anv(3)*derbv(3,iv)
        derdxv(iv)=-(derxg(iv)*dan2sdxg
     .             +deryg(iv)*dan2sdyg+danpldxv(iv)*dan2sdnpl)
        derdxv(iv)=derdxv(iv)-igrad*dgr2(iv)
c
        derdxv(iv)=derdxv(iv)+fdia*bdotgr*dbgr(iv)+0.5_wp_*bdotgr**2*
     .  (derxg(iv)*dfdiadxg+deryg(iv)*dfdiadyg+danpldxv(iv)*dfdiadnpl)
c
        derdnv(iv)=2.0_wp_*anv(iv)-bv(iv)*dan2sdnpl
     .             +0.5_wp_*bdotgr**2*bv(iv)*dfdiadnpl
c
        derdnm=derdnm+derdnv(iv)**2
      end do
c
      derdnm=sqrt(derdnm)
c
      derdom=-2.0_wp_*an2+2.0_wp_*xg*dan2sdxg+yg*dan2sdyg+anpl*dan2sdnpl
     .       +2.0_wp_*igrad*gr2
     .       -bdotgr**2*(fdia+xg*dfdiadxg+yg*dfdiadyg/2.0_wp_
     .                   +anpl*dfdiadnpl/2.0_wp_)
c
      if (idst.eq.0) then
c   integration variable: s
        denom=-derdnm
      else if (idst.eq.1) then
c   integration variable: c*t
        denom=derdom
      else
c   integration variable: Sr
        denom=-(anv(1)*derdnv(1)+anv(2)*derdnv(2)+anv(3)*derdnv(3))
      end if
c
c   coefficient for integration in s
c   ds/dst, where st is the integration variable
      dersdst=-derdnm/denom
c
      dery(1) = -derdnv(1)/denom
      dery(2) = -derdnv(2)/denom
      dery(3) = -derdnv(3)/denom
      dery(4) = derdxv(1)/denom
      dery(5) = derdxv(2)/denom
      dery(6) = derdxv(3)/denom
c
c     vgv :  ~ group velocity
c
      vgm=0
      do iv=1,3
        vgv(iv)=-derdnv(iv)/derdom
        vgm=vgm+vgv(iv)**2
      end do
      vgm=sqrt(vgm)
c
c     dd :  dispersion relation (real part)
c     ddi :  dispersion relation (imaginary part)
c
      dd=an2-an2s-igrad*(gr2-0.5_wp_*bdotgr**2*fdia)
      ddi=derdnv(1)*dgr(1)+derdnv(2)*dgr(2)+derdnv(3)*dgr(3)
c
      return
      end
c
c
c
      subroutine plas_deriv(xx,yy,zz)
      use const_and_precisions, only : wp_,pi,ccj=>mu0inv
      use graydata_flags, only : iequil
      use graydata_par, only : sgnbphi
      implicit none
c arguments
      real(wp_) :: xx,yy,zz
c local variables
      integer :: iv,jv
      real(wp_) :: b2tot,csphi,dfpolv,drrdx,drrdy,dphidx,dphidy,
     . rr,rr2,rrm,snphi,zzm
      real(wp_), dimension(3) :: dbtot,bvc
      real(wp_), dimension(3,3) :: dbvcdc,dbvdc,dbv
c common/external functions/variables
      real(wp_) bres,brr,bphi,bzz,ajphi,btot,xg,yg,dxgdpsi,dpsidr,
     . dpsidz,ddpsidrr,ddpsidzz,ddpsidrz,fpolv,ffpv,psinv
      real(wp_), dimension(3) :: bv,derxg,deryg
      real(wp_), dimension(3,3) :: derbv
c
      common/parbres/bres
      common/parpl/brr,bphi,bzz,ajphi
      common/btot/btot
      common/bb/bv
      common/dbb/derbv
      common/xgxg/xg
      common/dxgdps/dxgdpsi
      common/ygyg/yg
      common/dxgyg/derxg,deryg
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
      common/fpol/fpolv,ffpv
      common/psival/psinv

      xg=0.0_wp_
      yg=9.9e1_wp_
c
      do iv=1,3
        derxg(iv)=0.0_wp_
        deryg(iv)=0.0_wp_
        bv(iv)=0.0_wp_
        dbtot(iv)=0.0_wp_
        do jv=1,3
          dbv(iv,jv)=0.0_wp_
          derbv(iv,jv)=0.0_wp_
          dbvcdc(iv,jv)=0.0_wp_
          dbvcdc(iv,jv)=0.0_wp_
          dbvdc(iv,jv)=0.0_wp_
        end do
      end do
c
      if(iequil.eq.0) return
c
c     cylindrical coordinates
c
      rr2=xx**2+yy**2
      rr=sqrt(rr2)
      csphi=xx/rr
      snphi=yy/rr
c
      bv(1)=-snphi*sgnbphi
      bv(2)=csphi*sgnbphi
c
c     convert from cm to meters
c
      zzm=1.0e-2_wp_*zz
      rrm=1.0e-2_wp_*rr
c
      if(iequil.eq.1) then
        call equian(rrm,zzm)
      end if
c
      if(iequil.eq.2) then
        call equinum_psi(rrm,zzm)
        call equinum_derpsi(rrm,zzm,3)
        call equinum_fpol(1)
      end if

        call sub_xg_derxg
        yg=fpolv/(rrm*bres)
        bphi=fpolv/rrm
        btot=abs(bphi)
        if(psinv.lt.0.0_wp_) return
c
c     B = f(psi)/R e_phi+  grad psi x e_phi/R
c
c     bvc(i) = B_i in cylindrical coordinates
c     bv(i)  = B_i in cartesian coordinates
c
      bphi=fpolv/rrm
      brr=-dpsidz/rrm
      bzz= dpsidr/rrm
c
      dfpolv=ffpv/fpolv
c
      bvc(1)=brr
      bvc(2)=bphi
      bvc(3)=bzz
c
      bv(1)=bvc(1)*csphi-bvc(2)*snphi
      bv(2)=bvc(1)*snphi+bvc(2)*csphi
      bv(3)=bvc(3)
c
      b2tot=bv(1)**2+bv(2)**2+bv(3)**2
      btot=sqrt(b2tot)
c
c     dbvcdc(iv,jv) = d Bcil(iv) / dxvcil(jv)
c
      dbvcdc(1,1)=-ddpsidrz/rrm-brr/rrm
      dbvcdc(1,3)=-ddpsidzz/rrm
      dbvcdc(2,1)= dfpolv*dpsidr/rrm-bphi/rrm
      dbvcdc(2,3)= dfpolv*dpsidz/rrm
      dbvcdc(3,1)= ddpsidrr/rrm-bzz/rrm
      dbvcdc(3,3)= ddpsidrz/rrm
c
c     dbvdc(iv,jv) = d Bcart(iv) / dxvcil(jv)
c
      dbvdc(1,1) = dbvcdc(1,1)*csphi-dbvcdc(2,1)*snphi
      dbvdc(1,2) = -bv(2)
      dbvdc(1,3) = dbvcdc(1,3)*csphi-dbvcdc(2,3)*snphi
      dbvdc(2,1) = dbvcdc(1,1)*snphi+dbvcdc(2,1)*csphi
      dbvdc(2,2) = bv(1)
      dbvdc(2,3) = dbvcdc(1,3)*snphi+dbvcdc(2,3)*csphi
      dbvdc(3,1) = dbvcdc(3,1)
      dbvdc(3,2) = dbvcdc(3,2)
      dbvdc(3,3) = dbvcdc(3,3)
c
      drrdx=csphi
      drrdy=snphi
      dphidx=-snphi/rrm
      dphidy=csphi/rrm
c
c     dbv(iv,jv) = d Bcart(iv) / dxvcart(jv)
c
      dbv(1,1)=drrdx*dbvdc(1,1)+dphidx*dbvdc(1,2)
      dbv(1,2)=drrdy*dbvdc(1,1)+dphidy*dbvdc(1,2)
      dbv(1,3)=dbvdc(1,3)
      dbv(2,1)=drrdx*dbvdc(2,1)+dphidx*dbvdc(2,2)
      dbv(2,2)=drrdy*dbvdc(2,1)+dphidy*dbvdc(2,2)
      dbv(2,3)=dbvdc(2,3)
      dbv(3,1)=drrdx*dbvdc(3,1)+dphidx*dbvdc(3,2)
      dbv(3,2)=drrdy*dbvdc(3,1)+dphidy*dbvdc(3,2)
      dbv(3,3)=dbvdc(3,3)
c
      dbtot(1)=(bv(1)*dbv(1,1)+bv(2)*dbv(2,1)+bv(3)*dbv(3,1))/btot
      dbtot(2)=(bv(1)*dbv(1,2)+bv(2)*dbv(2,2)+bv(3)*dbv(3,2))/btot
      dbtot(3)=(bv(1)*dbv(1,3)+bv(2)*dbv(2,3)+bv(3)*dbv(3,3))/btot
c
      yg=btot/Bres
c
c     convert spatial derivatives from dummy/m -> dummy/cm
c     to be used in fwork
c
c     bv(i) = B_i / B ; derbv(i,j) = d (B_i / B) /d x,y,z
c
      do iv=1,3
        deryg(iv)=1.0e-2_wp_*dbtot(iv)/Bres
        bv(iv)=bv(iv)/btot
        do jv=1,3
          derbv(iv,jv)=1.0e-2_wp_*(dbv(iv,jv)-bv(iv)*dbtot(jv))/btot
        end do
      end do
c
      derxg(1)=1.0e-2_wp_*drrdx*dpsidr*dxgdpsi
      derxg(2)=1.0e-2_wp_*drrdy*dpsidr*dxgdpsi
      derxg(3)=1.0e-2_wp_*dpsidz*dxgdpsi
c
c     current density computation in Ampere/m^2
c     ccj==1/mu_0
c
      ajphi=ccj*(dbvcdc(1,3)-dbvcdc(3,1))
c      ajr=ccj*(dbvcdc(3,2)/rrm-dbvcdc(2,3))
c      ajz=ccj*(bvc(2)/rrm+dbvcdc(2,1)-dbvcdc(1,2))
c
      return
      end
c
c
c
      subroutine equian(rrm,zzm)
      use const_and_precisions, only : wp_
      use graydata_par, only : sgnbphi,sgniphi
      use graydata_anequil, only : b0,rr0m,zr0m,rpam,q0,qa,alq
      use interp_eqprof, only : psia
      implicit none
c arguments
      real(wp_) :: rrm,zzm
c local variables
      real(wp_) :: cst,dpsidrp,d2psidrp,dqq,qq,rn,rpm,sgn,snt,dfpolv,
     . rhop,rhot
c common/external functions/variables
      real(wp_) :: psinv,dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz,fpolv,
     . ffpv,xg,xgcn,dxgdpsi,dens,ddens
      real(wp_) :: frhopol
c
      common/psival/psinv
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
      common/fpol/fpolv,ffpv
      common/xgxg/xg
      common/dxgdps/dxgdpsi
      common/xgcn/xgcn
      common/dens/dens,ddens
c
      dpsidrp=0.0_wp_
      d2psidrp=0.0_wp_
c
c     simple model for equilibrium: large aspect ratio
c     outside plasma: analytical continuation, not solution Maxwell eqs
c
      rpm=sqrt((rrm-rr0m)**2+(zzm-zr0m)**2)
      rn=rpm/rpam
c
      snt=0.0_wp_
      cst=1.0_wp_
      if (rpm.gt.0.0_wp_) then
        snt=(zzm-zr0m)/rpm
        cst=(rrm-rr0m)/rpm
      end if
c
      rhot=rn
      if(rn.le.1) then
      rhop=frhopol(rhot)
      psinv=rhop*rhop
      else
      psinv=1.0_wp_+B0*rpam**2/2.0_wp_/abs(psia)/qa*(rn*rn-1.0_wp_)
      rhop=sqrt(psinv)
      end if
c
      sgn=sgniphi*sgnbphi
      if(rn.le.1.0_wp_) then
        qq=q0+(qa-q0)*rn**alq
        dpsidrp=B0*rpam*rn/qq*sgn
        dqq=alq*(qa-q0)*rn**(alq-1.0_wp_)
        d2psidrp=B0*(1.0_wp_-rn*dqq/qq)/qq*sgn
      else
        dpsidrp=B0*rpam/qa*rn*sgn
        d2psidrp=B0/qa*sgn
      end if
c
      fpolv=sgnbphi*b0*rr0m
      dfpolv=0.0_wp_
      ffpv=0.0_wp_
c
      dpsidr=dpsidrp*cst
      dpsidz=dpsidrp*snt
      ddpsidrr=dpsidrp*snt**2/rpm+cst**2*d2psidrp
      ddpsidrz=cst*snt*(d2psidrp-dpsidrp/rpm)
      ddpsidzz=dpsidrp*cst**2/rpm+snt**2*d2psidrp

      return
      end
c
c
c
      subroutine equinum_psi(rpsim,zpsim)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : rmnm,rmxm,zmnm,zmxm,psiant,psinop,
     . tr,tz,ccspl=>cceq,nsrt,nszt
      use dierckx, only : fpbisp
      implicit none
c local constants
      integer, parameter :: lwrk=8,liwrk=2,nrs=1,nzs=1
c arguments
      real(wp_) :: rpsim,zpsim
c local variables
      integer :: nsr,nsz
      integer, dimension(liwrk) :: iwrk
      real(wp_), dimension(1) :: rrs,zzs,ffspl
      real(wp_), dimension(lwrk) :: wrk
c common/external functions/variables
      real(wp_) :: psinv
c
      common/psival/psinv
c
      psinv=-1.0_wp_
c
c     here lengths are measured in meters
c
      if (rpsim.gt.rmxm.or.rpsim.lt.rmnm) return
      if (zpsim.gt.zmxm.or.zpsim.lt.zmnm) return
c
      rrs(1)=rpsim
      zzs(1)=zpsim
      nsr=nsrt
      nsz=nszt
      call fpbisp(tr,nsr,tz,nsz,ccspl,3,3,
     .     rrs,nrs,zzs,nzs,ffspl,wrk(1),wrk(5),iwrk(1),iwrk(2))
      psinv=(ffspl(1)-psinop)/psiant
c
      return
      end
c
c
c
      subroutine equinum_derpsi(rpsim,zpsim,iderpsi)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : nr,nz,rmnm,rmxm,zmnm,zmxm,cc01=>cceq01,
     . cc10=>cceq10,cc20=>cceq20,cc02=>cceq02,cc11=>cceq11
      implicit none
c arguments
      integer :: iderpsi
      real(wp_) :: rpsim,zpsim
c local variables
      integer :: lw10,lw01,lw20,lw02,lw11,nur,nuz
      real(wp_) :: derpsi
c common/external functions/variables
      real(wp_) :: dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz
c
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
c
      lw10=nr*3+nz*4+nr*nz
      lw01=nr*4+nz*3+nr*nz
      lw20=nr*2+nz*4+nr*nz
      lw02=nr*4+nz*2+nr*nz
      lw11=nr*3+nz*3+nr*nz
c
      dpsidr=0.0_wp_
      dpsidz=0.0_wp_
      ddpsidrr=0.0_wp_
      ddpsidzz=0.0_wp_
      ddpsidrz=0.0_wp_
 
c     here lengths are measured in meters
      if (rpsim.gt.rmxm.or.rpsim.lt.rmnm) return
      if (zpsim.gt.zmxm.or.zpsim.lt.zmnm) return
c
      select case(iderpsi)

      case(1)
      nur=1
      nuz=0
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc10,lw10)
      dpsidr=derpsi     
      nur=0
      nuz=1
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc01,lw01)
      dpsidz=derpsi

      case(2)
      nur=2
      nuz=0
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc20,lw20)
      ddpsidrr=derpsi      
      nur=0
      nuz=2
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc02,lw02)
      ddpsidzz=derpsi      
      nur=1
      nuz=1
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc11,lw11)
      ddpsidrz=derpsi

      case(3)
      nur=1
      nuz=0
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc10,lw10)
      dpsidr=derpsi      
      nur=0
      nuz=1
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc01,lw01)
      dpsidz=derpsi
      nur=2
      nuz=0
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc20,lw20)
      ddpsidrr=derpsi      
      nur=0
      nuz=2
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc02,lw02)
      ddpsidzz=derpsi      
      nur=1
      nuz=1
      call sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc11,lw11)
      ddpsidrz=derpsi

      case default
      print*,'iderpsi undefined'

      end select
c
      return
      end
c
c
c
      subroutine sub_derpsi(rpsim,zpsim,nur,nuz,derpsi,cc,lw)
      use const_and_precisions, only : wp_
      use dierckx, only : fpbisp
      use interp_eqprof, only : nr,nz,psia,tr,tz,nsrt,nszt
      implicit none
c local constants
      integer, parameter :: liwrk=2,nrs=1,nzs=1
c arguments
      integer :: nur,nuz,lw
      real(wp_) :: rpsim,zpsim,derpsi
      real(wp_), dimension(lw) :: cc
c local variables
      integer :: iwr,iwz,kkr,kkz,nsr,nsz
      integer, dimension(liwrk) :: iwrk
      real(wp_), dimension(1) :: rrs,zzs,ffspl
c
      rrs(1)=rpsim
      zzs(1)=zpsim
      nsr=nsrt
      nsz=nszt
      kkr=3-nur
      kkz=3-nuz
      iwr=1+(nr-nur-4)*(nz-nuz-4)
      iwz=iwr+4-nur
      call fpbisp(tr(nur+1),nsr-2*nur,tz(nuz+1),nsz-2*nuz,cc,kkr,kkz
     .     ,rrs,nrs,zzs,nzs,ffspl,cc(iwr),cc(iwz),iwrk(1),iwrk(2))
      derpsi=ffspl(1)*psia
c
      return
      end
c
c
c
      subroutine equinum_fpol(iderfpol)
      use const_and_precisions, only : wp_
      use dierckx, only : splev,splder
      use interp_eqprof, only : nr,psia,tfp,nsft,cfp,fpolas
      implicit none
c arguments
      integer :: iderfpol
c local variables
      integer :: ier
      real(wp_) :: dfpolv
      real(wp_), dimension(1) :: rrs,ffspl
      real(wp_), dimension(nr+4) :: wrkfd
c common/external functions/variables
      real(wp_) :: psinv,fpolv,ffpv
c
      common/psival/psinv
      common/fpol/fpolv,ffpv
c      
      fpolv=fpolas
      dfpolv=0.0_wp_
      ffpv=0.0_wp_
      if(iderfpol.lt.0.or.iderfpol.gt.1) return
c
      if(psinv.le.1.0_wp_.and.psinv.gt.0.0_wp_) then
        rrs(1)=psinv
        call splev(tfp,nsft,cfp,3,rrs,ffspl,1,ier)
        fpolv=ffspl(1)
        if(iderfpol.eq.1) then       
          call splder(tfp,nsft,cfp,3,1,rrs,ffspl,1,wrkfd,ier)
          dfpolv=ffspl(1)
          ffpv=fpolv*dfpolv/psia
        end if
      end if
c
      return
      end
c
c
c
      subroutine bfield(rpsim,zpsim,bphi,brr,bzz)
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_) :: rpsim,zpsim,bphi,brr,bzz
c
      call btor(rpsim,zpsim,bphi)
      call bpol(rpsim,zpsim,brr,bzz)
c
      return
      end
c
c
c
      subroutine btor(rpsim,zpsim,bphi)
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_) :: rpsim,zpsim,bphi
c common/external functions/variables
      real(wp_) :: psinv,fpolv,ffpv
c
      common/psival/psinv
      common/fpol/fpolv,ffpv
c
      call equinum_psi(rpsim,zpsim)
      call equinum_fpol(0)
      bphi=fpolv/rpsim
c
      return
      end
c
c
c
      subroutine bpol(rpsim,zpsim,brr,bzz)
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_) :: rpsim,zpsim,brr,bzz
c common/external functions/variables
      real(wp_) :: dpsidr,dpsidz
c
      common/derip1/dpsidr,dpsidz
c
      call equinum_derpsi(rpsim,zpsim,1)
      brr=-dpsidz/rpsim
      bzz= dpsidr/rpsim
c
      return
      end
c
c
c
      subroutine tor_curr_psi(h,ajphi)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : zmaxis
      implicit none
c arguments
      real(wp_) :: h,ajphi
c local variables
      real(wp_) :: r1,r2
c
      call psi_raxis(h,r1,r2)
      call tor_curr(r2,zmaxis,ajphi)
c
      return
      end
c
c
c
      subroutine tor_curr(rpsim,zpsim,ajphi)
      use const_and_precisions, only : wp_,ccj=>mu0inv
      implicit none
c arguments
      real(wp_) :: rpsim,zpsim,ajphi
c local variables
      real(wp_) :: bzz,dbvcdc13,dbvcdc31
c common/external functions/variables
      real(wp_) dpsidr,dpsidz,ddpsidrr,ddpsidzz,ddpsidrz
c
      common/derip1/dpsidr,dpsidz
      common/derip2/ddpsidrr,ddpsidzz,ddpsidrz
c
      call equinum_derpsi(rpsim,zpsim,3)
      bzz= dpsidr/rpsim
      dbvcdc13=-ddpsidzz/rpsim
      dbvcdc31= ddpsidrr/rpsim-bzz/rpsim
      ajphi=ccj*(dbvcdc13-dbvcdc31)
c
      return
      end
c
c
c
      subroutine psi_raxis(h,r1,r2)
      use const_and_precisions, only : wp_
      use interp_eqprof, only : psiant,psinop,zmaxis,nsr=>nsrt,
     . nsz=>nszt,cc=>cceq,tr,tz,nr
      use dierckx, only : profil,sproota
      implicit none
c local constants
      integer, parameter :: mest=4,kspl=3
c arguments
      real(wp_) :: h,r1,r2
c local variables
      integer :: iopt,ier,m
      real(wp_) :: zc,val
      real(wp_), dimension(mest) :: zeroc
      real(wp_), dimension(:), allocatable :: czc
c
      if(allocated(czc)) deallocate(czc)
      allocate(czc(nr+4))
c
      iopt=1
      zc=zmaxis
      call profil(iopt,tr,nsr,tz,nsz,cc,kspl,kspl,zc,nsr,czc,ier)
      if(ier.gt.0) print*,'    profil =',ier
      val=h*psiant+psinop
      call sproota(val,tr,nsr,czc,zeroc,mest,m,ier)
      r1=zeroc(1)
      r2=zeroc(2)
c
      deallocate(czc)
c
      return
      end
c
c
c
      subroutine sub_xg_derxg
      use const_and_precisions, only : wp_
      use interp_eqprof, only : psia
      implicit none
c common/external functions/variables
      real(wp_) psinv,xg,xgcn,dxgdpsi,dens,ddenspsin
c
      common/psival/psinv
      common/xgxg/xg
      common/dxgdps/dxgdpsi
      common/xgcn/xgcn
      common/dens/dens,ddenspsin
c
      xg=0.0_wp_
      dxgdpsi=0.0_wp_
c      if(psinv.le.psdbnd.and.psinv.ge.0) then
        call density(psinv)
        xg=xgcn*dens
        dxgdpsi=xgcn*ddenspsin/psia
c      end if
c
      return
      end
c
c
c
      subroutine density(arg)
      use const_and_precisions, only : wp_
      use graydata_flags, only : iprof
      use graydata_par, only : psdbnd
      use graydata_anequil, only : dens0,aln1,aln2
      use interp_eqprof, only : psnpp,aad=>aa,bbd=>bb,ccd=>cc,tfn,nsfd,
     . cfn,npp
      use dierckx, only : splev,splder
      implicit none
c arguments
      real(wp_) :: arg
c local variables
      integer :: ier,nn,nn1,nn2,nu
      real(wp_) :: profd,dprofd,dpsib
      real(wp_), dimension(1) :: xxs,ffs
      real(wp_), dimension(npp+4) :: wrkfd
c common/external functions/variables
      real(wp_) :: dens,ddens
c
      common/dens/dens,ddens
c
c     computation of density [10^19 m^-3] and derivative wrt psi
c
      dens=0.0_wp_
      ddens=0.0_wp_
      if(arg.gt.psdbnd.or.arg.lt.0.0_wp_) return
c
      if(iprof.eq.0) then
        if(arg.gt.1.0_wp_) return
        profd=(1.0_wp_-arg**aln1)**aln2
        dens=dens0*profd
        dprofd=-aln1*aln2*arg**(aln1-1.0_wp_)
     .         *(1.0_wp_-arg**aln1)**(aln2-1.0_wp_)
        ddens=dens0*dprofd
      else
        if(arg.le.psdbnd.and.arg.gt.psnpp) then
c
c     cubic interpolation for 1 < psi < psdbnd
c
          nn=3
          nn1=nn+1
          nn2=nn+2
          dpsib=arg-psdbnd
          dens=aad*dpsib**nn+bbd*dpsib**nn1+ccd*dpsib**nn2
          ddens=nn*aad*dpsib**(nn-1)+nn1*bbd*dpsib**nn
     .          +nn2*ccd*dpsib**nn1
        else
          xxs(1)=arg
          ier=0
          call splev(tfn,nsfd,cfn,3,xxs,ffs,1,ier)
          dens=ffs(1)
          nu=1
          ier=0
          call splder(tfn,nsfd,cfn,3,nu,xxs,ffs,1,wrkfd,ier)
          ddens=ffs(1)
          if(ier.gt.0) print*,ier
          if(abs(dens).lt.1.0e-10_wp_) dens=0.0_wp_
        end if
        if(dens.lt.0.0_wp_) print*,'    DENSITY NEGATIVE',dens
c
      end if
c
      return
      end
c
c
c
      function temp(arg)
      use const_and_precisions, only : wp_
      use graydata_flags, only : iprof
      use graydata_anequil, only : te0,dte0,alt1,alt2
      use interp_eqprof, only : psrad,ct,npp
      use utils, only : locate
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: arg,temp
c local variables
      integer :: k
      real(wp_) :: proft,dps
c
      temp=0.0_wp_
      if(arg.ge.1.0_wp_.or.arg.lt.0.0_wp_) return
      if(iprof.eq.0) then
        proft=(1.0_wp_-arg**alt1)**alt2
        temp=(te0-dte0)*proft+dte0
      else
        call locate(psrad,npp,arg,k)
        if(k.eq.0) k=1
        if(k.eq.npp) k=npp-1
        dps=arg-psrad(k)
        temp=spli(ct,npp,k,dps)
      endif
c
      return
      end
c
c
c
      function fzeff(arg)
      use const_and_precisions, only : wp_
      use graydata_flags, only : iprof
      use graydata_anequil, only : zeffan
      use interp_eqprof, only : psrad,cz,npp
      use utils, only : locate
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: arg,fzeff
c local variables
      integer :: k
      real(wp_) :: ps,dps
c
      fzeff=1
      ps=arg
      if(ps.gt.1.0_wp_.and.ps.lt.0.0_wp_) return
      if(iprof.eq.0) then
        fzeff=zeffan
      else
        call locate(psrad,npp,ps,k)
        k=max(1,min(k,npp-1))
        dps=ps-psrad(k)
        fzeff=spli(cz,npp,k,dps)
      endif
c
      return
      end
c
c
c
      subroutine ic_gb
c     beam tracing initial conditions  igrad=1
c
      use const_and_precisions, only : wp_,izero,zero,one,pi,
     . cvdr=>degree,ui=>im
      use math, only : catand
      use graydata_flags, only : ipol
      use graydata_par, only : rwmax,psipol0,chipol0
      implicit none
c local constants
      integer, parameter :: ndim=6,ndimm=3,jmx=31,kmx=36
c local variables
      integer :: j,k,iproj,nfilp
      real(wp_) :: csth,snth,csps,snps,phiwrad,phirrad,csphiw,snphiw,
     . wwcsi,wweta,sk,sw,dk,dw,rci1,ww1,rci2,ww2,wwxx,wwyy,wwxy,rcixx,
     . rciyy,rcixy,dwwxx,dwwyy,dwwxy,d2wwxx,d2wwyy,d2wwxy,drcixx,drciyy,
     . drcixy,dr,da,ddfu,u,alfak,dcsiw,detaw,dx0t,dy0t,x0t,y0t,z0t,dx0,
     . dy0,dz0,x0,y0,z0,gxt,gyt,gzt,gr2,gxxt,gyyt,gzzt,gxyt,gxzt,gyzt,
     . dgr2xt,dgr2yt,dgr2zt,dgr2x,dgr2y,dgr2z,pppx,pppy,denpp,ppx,ppy,
     . anzt,anxt,anyt,anx,any,anz,an20,an0,anx0,any0,anz0,du1tx,du1ty,
     . du1tz,vgradi,r0,x0m,y0m,r0m,z0m,ddr110,deltapol,qq,uu,vv
      real(wp_), dimension(ndim) :: ytmp,yptmp
      complex(wp_) :: sss,ddd,phic,qi1,qi2,tc,ts,qqxx,qqxy,qqyy,dqi1,
     . dqi2,dqqxx,dqqyy,dqqxy,d2qi1,d2qi2,d2qqxx,d2qqyy,d2qqxy
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_) :: ak0,akinv,fhz,wcsi,weta,rcicsi,rcieta,phiw,phir,
     . anx0c,any0c,anz0c,x00,y00,z00,dd,an2s,an2,fdia,bdotgr,ddi,
     . ddr11,psinv,dens,ddens,tekev,anpl,anpr,brr,bphi,bzz,ajphi
      real(wp_), dimension(jmx) :: dffiu,ddffiu
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: gri
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk0,ypwrk0
      real(wp_), dimension(ndimm,jmx,kmx) :: xc0,du10,dgrad2v
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
      complex(wp_), dimension(jmx,kmx,0:4) :: ext,eyt
c
      common/nray/nrayr,nrayth
      common/parwv/ak0,akinv,fhz
      common/parbeam/wcsi,weta,rcicsi,rcieta,phiw,phir
      common/anic/anx0c,any0c,anz0c
      common/wrk/ywrk0,ypwrk0
      common/grc/xc0,du10
      common/fi/dffiu,ddffiu
      common/mirr/x00,y00,z00
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
      common/ddd/dd,an2s,an2,fdia,bdotgr,ddi,ddr11
      common/evt/ext,eyt
      common/nplr/anpl,anpr
      common/psival/psinv
      common/parpl/brr,bphi,bzz,ajphi
      common/dens/dens,ddens
      common/tete/tekev
c
      csth=anz0c
      snth=sqrt(1.0_wp_-csth**2)
      csps=1.0_wp_
      snps=0.0_wp_
      if(snth.gt.0.0_wp_) then
        csps=any0c/snth
        snps=anx0c/snth
      end if
c
      phiwrad=phiw*cvdr
      phirrad=phir*cvdr
      csphiw=cos(phiwrad)
      snphiw=sin(phiwrad)
c      csphir=cos(phirrad)
c      snphir=sin(phirrad)
c
      wwcsi=2.0_wp_*akinv/wcsi**2
      wweta=2.0_wp_*akinv/weta**2
c
      if(phir.ne.phiw) then
        sk=(rcicsi+rcieta)
        sw=(wwcsi+wweta)
        dk=(rcicsi-rcieta)
        dw=(wwcsi-wweta)
        ts=-(dk*sin(2.0_wp_*phirrad)-ui*dw*sin(2.0_wp_*phiwrad))
        tc=(dk*cos(2.0_wp_*phirrad)-ui*dw*cos(2.0_wp_*phiwrad))
        phic=0.5_wp_*catand(ts/tc)
        ddd=dk*cos(2*(phirrad+phic))-ui*dw*cos(2*(phiwrad+phic))
        sss=sk-ui*sw
        qi1=0.5_wp_*(sss+ddd)
        qi2=0.5_wp_*(sss-ddd)
        rci1=dble(qi1)
        ww1=-dimag(qi1)
        rci2=dble(qi2)
        ww2=-dimag(qi2)
      else
        rci1=rcicsi
        rci2=rcieta
        ww1=wwcsi
        ww2=wweta
        phic=-phiwrad
        qi1=rci1-ui*ww1
        qi2=rci2-ui*ww2
      end if
      
c      w01=sqrt(2.0_wp_*akinv/ww1)
c      z01=-rci1/(rci1**2+ww1**2)
c      w02=sqrt(2.0_wp_*akinv/ww2)
c      z02=-rci2/(rci2**2+ww2**2)

      qqxx=qi1*cos(phic)**2+qi2*sin(phic)**2
      qqyy=qi1*sin(phic)**2+qi2*cos(phic)**2
      qqxy=-(qi1-qi2)*sin(2.0_wp_*phic)
      wwxx=-dimag(qqxx)
      wwyy=-dimag(qqyy)
      wwxy=-dimag(qqxy)/2.0_wp_
      rcixx=dble(qqxx)
      rciyy=dble(qqyy)
      rcixy=dble(qqxy)/2.0_wp_

      dqi1=-qi1**2
      dqi2=-qi2**2
      d2qi1=2*qi1**3
      d2qi2=2*qi2**3
      dqqxx=dqi1*cos(phic)**2+dqi2*sin(phic)**2
      dqqyy=dqi1*sin(phic)**2+dqi2*cos(phic)**2
      dqqxy=-(dqi1-dqi2)*sin(2.0_wp_*phic)
      d2qqxx=d2qi1*cos(phic)**2+d2qi2*sin(phic)**2
      d2qqyy=d2qi1*sin(phic)**2+d2qi2*cos(phic)**2
      d2qqxy=-(d2qi1-d2qi2)*sin(2.0_wp_*phic)
      dwwxx=-dimag(dqqxx)
      dwwyy=-dimag(dqqyy)
      dwwxy=-dimag(dqqxy)/2.0_wp_
      d2wwxx=-dimag(d2qqxx)
      d2wwyy=-dimag(d2qqyy)
      d2wwxy=-dimag(d2qqxy)/2.0_wp_
      drcixx=dble(dqqxx)
      drciyy=dble(dqqyy)
      drcixy=dble(dqqxy)/2.0_wp_

      dr=1.0_wp_
      if(nrayr.gt.1) dr=rwmax/dble(nrayr-1)
      da=2.0_wp_*pi/dble(nrayth)
c
      ddfu=2.0_wp_*dr**2*akinv
      do j=1,nrayr
        u=dble(j-1)
c        ffi=u**2*ddfu/2.0_wp_
        dffiu(j)=u*ddfu
        ddffiu(j)=ddfu
        do k=1,nrayth
          alfak=(k-1)*da
          dcsiw=dr*cos(alfak)*wcsi
          detaw=dr*sin(alfak)*weta
          dx0t=dcsiw*csphiw-detaw*snphiw
          dy0t=dcsiw*snphiw+detaw*csphiw
          x0t=u*dx0t
          y0t=u*dy0t
          z0t=-0.5_wp_*(rcixx*x0t**2+rciyy*y0t**2+2*rcixy*x0t*y0t)
c
c          csiw=u*dcsiw
c          etaw=u*detaw
c          csir=x0t*csphir+y0t*snphir
c          etar=-x0t*snphir+y0t*csphir
c
          dx0= x0t*csps+snps*(y0t*csth+z0t*snth)
          dy0=-x0t*snps+csps*(y0t*csth+z0t*snth)
          dz0= z0t*csth-y0t*snth
          x0=x00+dx0
          y0=y00+dy0
          z0=z00+dz0

            gxt=x0t*wwxx+y0t*wwxy
            gyt=x0t*wwxy+y0t*wwyy
            gzt=0.5_wp_*(x0t**2*dwwxx+y0t**2*dwwyy)+x0t*y0t*dwwxy
            gr2=gxt*gxt+gyt*gyt+gzt*gzt
            gxxt=wwxx
            gyyt=wwyy
            gzzt=0.5_wp_*(x0t**2*d2wwxx+y0t**2*d2wwyy)+x0t*y0t*d2wwxy
            gxyt=wwxy
            gxzt=x0t*dwwxx+y0t*dwwxy
            gyzt=x0t*dwwxy+y0t*dwwyy
            dgr2xt=2.0_wp_*(gxt*gxxt+gyt*gxyt+gzt*gxzt)
            dgr2yt=2.0_wp_*(gxt*gxyt+gyt*gyyt+gzt*gyzt)
            dgr2zt=2.0_wp_*(gxt*gxzt+gyt*gyzt+gzt*gzzt)
            dgr2x= dgr2xt*csps+snps*(dgr2yt*csth+dgr2zt*snth)
            dgr2y=-dgr2xt*snps+csps*(dgr2yt*csth+dgr2zt*snth)
            dgr2z= dgr2zt*csth-dgr2yt*snth
            gri(1,j,k)=gxt*csps+snps*(gyt*csth+gzt*snth)
            gri(2,j,k)=-gxt*snps+csps*(gyt*csth+gzt*snth)
            gri(3,j,k)=gzt*csth-gyt*snth
            ggri(1,1,j,k)=gxxt*csps**2+snps**2
     .                    *(gyyt*csth**2+gzzt*snth**2
     .                    +2.0_wp_*snth*csth*gyzt)
     .                    +2.0_wp_*snps*csps*(gxyt*csth+gxzt*snth)
            ggri(2,2,j,k)=gxxt*snps**2+csps**2
     .                    *(gyyt*csth**2+gzzt*snth**2
     .                    +2.0_wp_*snth*csth*gyzt)
     .                    -2.0_wp_*snps*csps*(gxyt*csth+gxzt*snth)
            ggri(3,3,j,k)=gzzt*csth**2+gyyt*snth**2
     .                    -2.0_wp_*csth*snth*gyzt
            ggri(1,2,j,k)=csps*snps*(-gxxt+csth**2*gyyt+snth**2*gzzt
     .        +2.0_wp_*csth*snth*gyzt)
     .        +(csps**2-snps**2)*(snth*gxzt+csth*gxyt)
            ggri(1,3,j,k)=csth*snth*snps*(gzzt-gyyt)+(csth**2-snth**2)
     .                    *snps*gyzt+csps*(csth*gxzt-snth*gxyt)
            ggri(2,3,j,k)=csth*snth*csps*(gzzt-gyyt)+(csth**2-snth**2)
     .                    *csps*gyzt+snps*(snth*gxyt-csth*gxzt)
            ggri(2,1,j,k)=ggri(1,2,j,k)
            ggri(3,1,j,k)=ggri(1,3,j,k)
            ggri(3,2,j,k)=ggri(2,3,j,k)
c
            du1tx=(dx0t*wwxx+dy0t*wwxy)/ddfu
            du1ty=(dx0t*wwxy+dy0t*wwyy)/ddfu
            du1tz=0.5_wp_*u*(dx0t**2*dwwxx+dy0t**2*dwwyy
     .            +2.0_wp_*dx0t*dy0t*dwwxy)/ddfu
c
          pppx=x0t*rcixx+y0t*rcixy
          pppy=x0t*rcixy+y0t*rciyy
          denpp=pppx*gxt+pppy*gyt
          if (denpp.ne.0.0_wp_) then
            ppx=-pppx*gzt/denpp
            ppy=-pppy*gzt/denpp
          else
            ppx=0.0_wp_
            ppy=0.0_wp_
          end if
c
          anzt=sqrt((1.0_wp_+gr2)/(1.0_wp_+ppx**2+ppy**2))
          anxt=ppx*anzt
          anyt=ppy*anzt
c
          anx= anxt*csps+snps*(anyt*csth+anzt*snth)
          any=-anxt*snps+csps*(anyt*csth+anzt*snth)
          anz= anzt*csth-anyt*snth
c
          an20=1.0_wp_+gr2
          an0=sqrt(an20)
          anx0=anx
          any0=any
          anz0=anz
c
          xc0(1,j,k)=x0
          xc0(2,j,k)=y0
          xc0(3,j,k)=z0
c
          ywrk0(1,j,k)=x0
          ywrk0(2,j,k)=y0
          ywrk0(3,j,k)=z0
          ywrk0(4,j,k)=anx0
          ywrk0(5,j,k)=any0
          ywrk0(6,j,k)=anz0
c
          ypwrk0(1,j,k) = anx0/an0
          ypwrk0(2,j,k) = any0/an0
          ypwrk0(3,j,k) = anz0/an0
          ypwrk0(4,j,k) = dgr2x/an0/2.0_wp_
          ypwrk0(5,j,k) = dgr2y/an0/2.0_wp_
          ypwrk0(6,j,k) = dgr2z/an0/2.0_wp_
c
          ytmp=ywrk0(:,j,k)
          yptmp=ypwrk0(:,j,k)
          call fwork(ytmp,yptmp)

          if(ipol.eq.0) then
            call pol_limit(ext(j,k,0),eyt(j,k,0))
            qq=abs(ext(j,k,0))**2-abs(eyt(j,k,0))**2
            uu=2.0_wp_*dble(ext(j,k,0)*dconjg(eyt(j,k,0)))
            vv=2.0_wp_*dimag(ext(j,k,0)*dconjg(eyt(j,k,0)))
            call polellipse(qq,uu,vv,psipol0,chipol0)
          else
            qq=cos(2.0_wp_*chipol0*cvdr)*cos(2.0_wp_*psipol0*cvdr)
            uu=cos(2.0_wp_*chipol0*cvdr)*sin(2.0_wp_*psipol0*cvdr)
            vv=sin(2.0_wp_*chipol0*cvdr)
            if(qq**2.lt.1) then
              deltapol=asin(vv/sqrt(1.0_wp_-qq**2))
              ext(j,k,0)= sqrt((1.0_wp_+qq)/2)
              eyt(j,k,0)= sqrt((1.0_wp_-qq)/2)*exp(-ui*deltapol)
            else
              ext(j,k,0)= 1.0_wp_
              eyt(j,k,0)= 0.0_wp_
            end if
          endif
c
          grad2(j,k)=gr2
          dgrad2v(1,j,k)=dgr2x
          dgrad2v(2,j,k)=dgr2y
          dgrad2v(3,j,k)=dgr2z
c
          du10(1,j,k)= du1tx*csps+snps*(du1ty*csth+du1tz*snth)
          du10(2,j,k)=-du1tx*snps+csps*(du1ty*csth+du1tz*snth)
          du10(3,j,k)= du1tz*csth-du1ty*snth
c
          dd=anx0**2+any0**2+anz0**2-an20
          vgradi=anxt*gxt+anyt*gyt+anzt*gzt
          ddi=2.0_wp_*vgradi
c
          r0=sqrt(x0**2+y0**2)
          x0m=x0/1.0e2_wp_
          y0m=y0/1.0e2_wp_
          r0m=r0/1.0e2_wp_
          z0m=z0/1.0e2_wp_
          if(j.eq.nrayr) then
             write(33,111) izero,j,k,zero,x0m,y0m,r0m,z0m,
     .          psinv,zero,anpl,zero,one
          end if
          if(j.eq.1.and.k.eq.1) then
            write(4,99) zero,r0m,z0m,atan2(y0m,x0m)*180.0_wp_/pi,
     .      psinv,one,dens,tekev,brr,bphi,bzz,
     .      ajphi*1.0e-6_wp_,sqrt(anpl**2+anpr**2),anpl,zero,
     .      zero,zero,zero,zero,zero,zero,zero,one

            ddr110=dd
          end if  
          if(j.eq.nrayr.and.k.eq.1) then
            write(17,99) zero,ddr110,dd,ddi
          end if          
        end do
      end do

      call pweigth
c
      if(nrayr.gt.1) then
        iproj=0
        nfilp=8
        call projxyzt(iproj,nfilp)
      end if
c
      return
 99   format(24(1x,e16.8e3))
111   format(3i5,20(1x,e16.8e3))
      end
c
c
c
      subroutine ic_rt
c ray tracing initial conditions  igrad=0
c
      use const_and_precisions, only : wp_,izero,zero,one,pi,
     . cvdr=>degree,ui=>im
      use graydata_flags, only : ipol
      use graydata_par, only : rwmax,psipol0,chipol0
      implicit none
c local constants
      integer, parameter :: ndim=6,ndimm=3,jmx=31,kmx=36
c local variables
      integer :: j,k,iv,jv,iproj,nfilp
      real(wp_) :: csth,snth,csps,snps,phiwrad,csphiw,snphiw,dr,da,u,
     . alfak,dcsiw,detaw,dx0t,dy0t,x0t,y0t,z0t,dx0,dy0,dz0,x0,y0,z0,
     . anzt,anxt,anyt,anx,any,anz,an20,an0,anx0,any0,anz0,vgradi,r0,
     . x0m,y0m,r0m,z0m,ancsi,aneta,ppcsi,ppeta,deltapol,qq,uu,vv
      real(wp_), dimension(ndim) :: ytmp,yptmp
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_) :: wcsi,weta,rcicsi,rcieta,phiw,phir,anx0c,any0c,anz0c,
     . x00,y00,z00,dd,an2s,an2,fdia,bdotgr,ddi,ddr11,psinv,dens,ddens,
     . tekev,anpl,anpr,brr,bphi,bzz,ajphi
      real(wp_), dimension(jmx) :: dffiu,ddffiu
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: gri
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk0,ypwrk0
      real(wp_), dimension(ndimm,jmx,kmx) :: xc0,du10,dgrad2v
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
      complex(wp_), dimension(jmx,kmx,0:4) :: ext,eyt
c
      common/nray/nrayr,nrayth
      common/parbeam/wcsi,weta,rcicsi,rcieta,phiw,phir
      common/anic/anx0c,any0c,anz0c
      common/wrk/ywrk0,ypwrk0
      common/grc/xc0,du10
      common/fi/dffiu,ddffiu
      common/mirr/x00,y00,z00
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
      common/ddd/dd,an2s,an2,fdia,bdotgr,ddi,ddr11
      common/evt/ext,eyt
      common/nplr/anpl,anpr
      common/psival/psinv
      common/parpl/brr,bphi,bzz,ajphi
      common/dens/dens,ddens
      common/tete/tekev
c
      csth=anz0c
      snth=sqrt(1.0_wp_-csth**2)
      csps=1.0_wp_
      snps=0.0_wp_
      if(snth.gt.0.0_wp_) then
        csps=any0c/snth
        snps=anx0c/snth
      end if
c
      phiwrad=phiw*cvdr
      csphiw=cos(phiwrad)
      snphiw=sin(phiwrad)
c
      dr=1.0_wp_
      if(nrayr.gt.1) dr=rwmax/dble(nrayr-1)
      da=2.0_wp_*pi/dble(nrayth)
      z0t=0.0_wp_
c
      do j=1,nrayr
        u=dble(j-1)
        dffiu(j)=0.0_wp_
        ddffiu(j)=0.0_wp_
        do k=1,nrayth
          alfak=(k-1)*da
          dcsiw=dr*cos(alfak)*wcsi
          detaw=dr*sin(alfak)*weta
          dx0t=dcsiw*csphiw-detaw*snphiw
          dy0t=dcsiw*snphiw+detaw*csphiw
          x0t=u*dx0t
          y0t=u*dy0t
c
c          csiw=u*dcsiw
c          etaw=u*detaw
c          csir=csiw
c          etar=etaw
c
          dx0= x0t*csps+snps*(y0t*csth+z0t*snth)
          dy0=-x0t*snps+csps*(y0t*csth+z0t*snth)
          dz0= z0t*csth-y0t*snth
c
          x0=x00+dx0
          y0=y00+dy0
          z0=z00+dz0
c
          ppcsi=u*dr*cos(alfak)*rcicsi
          ppeta=u*dr*sin(alfak)*rcieta
c
          anzt=1.0_wp_/sqrt(1.0_wp_+ppcsi**2+ppeta**2)
          ancsi=ppcsi*anzt
          aneta=ppeta*anzt
c
          anxt=ancsi*csphiw-aneta*snphiw
          anyt=ancsi*snphiw+aneta*csphiw
c
          anx= anxt*csps+snps*(anyt*csth+anzt*snth)
          any=-anxt*snps+csps*(anyt*csth+anzt*snth)
          anz= anzt*csth-anyt*snth
c
          an20=1.0_wp_
          an0=sqrt(an20)
          anx0=anx
          any0=any
          anz0=anz
c
          xc0(1,j,k)=x0
          xc0(2,j,k)=y0
          xc0(3,j,k)=z0
c
          ywrk0(1,j,k)=x0
          ywrk0(2,j,k)=y0
          ywrk0(3,j,k)=z0
          ywrk0(4,j,k)=anx0
          ywrk0(5,j,k)=any0
          ywrk0(6,j,k)=anz0
c
          ypwrk0(1,j,k) = anx0/an0
          ypwrk0(2,j,k) = any0/an0
          ypwrk0(3,j,k) = anz0/an0
          ypwrk0(4,j,k) = 0.0_wp_
          ypwrk0(5,j,k) = 0.0_wp_
          ypwrk0(6,j,k) = 0.0_wp_
c
          ytmp=ywrk0(:,j,k)
          yptmp=ypwrk0(:,j,k)
          call fwork(ytmp,yptmp)

          if(ipol.eq.0) then
            call pol_limit(ext(j,k,0),eyt(j,k,0))
            qq=abs(ext(j,k,0))**2-abs(eyt(j,k,0))**2
            uu=2.0_wp_*dble(ext(j,k,0)*dconjg(eyt(j,k,0)))
            vv=2.0_wp_*dimag(ext(j,k,0)*dconjg(eyt(j,k,0)))
            call polellipse(qq,uu,vv,psipol0,chipol0)
          else
            qq=cos(2.0_wp_*chipol0*cvdr)*cos(2.0_wp_*psipol0*cvdr)
            uu=cos(2.0_wp_*chipol0*cvdr)*sin(2.0_wp_*psipol0*cvdr)
            vv=sin(2.0_wp_*chipol0*cvdr)
            if(qq**2.lt.1.0_wp_) then
c             deltapol=phix-phiy, phix =0            
              deltapol=atan2(vv,uu)
              ext(j,k,0)= sqrt((1.0_wp_+qq)/2)
              eyt(j,k,0)= sqrt((1.0_wp_-qq)/2)*exp(-ui*deltapol)
            else
            if(qq.gt.0.0_wp_) then
              ext(j,k,0)= 1.0_wp_
              eyt(j,k,0)= 0.0_wp_
              else
              eyt(j,k,0)= 1.0_wp_
              ext(j,k,0)= 0.0_wp_
              end if
            end if
          endif
c
          do iv=1,3
            gri(iv,j,k)=0.0_wp_
            dgrad2v(iv,j,k)=0.0_wp_
            du10(iv,j,k)=0.0_wp_
            do jv=1,3
              ggri(iv,jv,j,k)=0.0_wp_
            end do
          end do
          grad2(j,k)=0.0_wp_
c
          dd=anx0**2+any0**2+anz0**2-an20
          vgradi=0.0_wp_
          ddi=2.0_wp_*vgradi
c
          r0=sqrt(x0**2+y0**2)
          x0m=x0/1.0e2_wp_
          y0m=y0/1.0e2_wp_
          r0m=r0/1.0e2_wp_
          z0m=z0/1.0e2_wp_
          if(j.eq.nrayr) then
             write(33,111) izero,j,k,zero,x0m,y0m,r0m,z0m,
     .          psinv,zero,anpl,zero,one
          end if
           if(j.eq.1.and.k.eq.1) then
            write(17,99) zero,zero,zero,zero
            write(4,99) zero,r0m,z0m,atan2(y0m,x0m)*180.0_wp_/pi,
     .      psinv,one,dens,tekev,brr,bphi,bzz,
     .      ajphi*1.0e-6_wp_,sqrt(anpl**2+anpr**2),anpl,zero,
     .      zero,zero,zero,zero,zero,zero,zero,one
          end if  
        end do
      end do

      call pweigth
c
      if(nrayr.gt.1) then
        iproj=0
        nfilp=8
        call projxyzt(iproj,nfilp)
      end if
c
      return
 99   format(24(1x,e16.8e3))
111   format(3i5,20(1x,e16.8e3))
      end
c
c
c
      subroutine ic_rt2
      use const_and_precisions, only : wp_,izero,zero,one,pi,
     . cvdr=>degree
      use graydata_par, only : psipol0,chipol0
      implicit none
c local constants
      integer, parameter :: ndim=6,ndimm=3,jmx=31,kmx=36
c local variables
      integer :: j,k,iv,jv,iproj,nfilp
      real(wp_) :: x0,y0,z0,an20,an0,anx0,any0,anz0,vgradi,
     . r0,x0m,y0m,r0m,z0m,strfl11,qq,uu,vv
      real(wp_), dimension(ndim) :: ytmp,yptmp
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_) :: dd,an2s,an2,fdia,bdotgr,ddi,ddr11,psinv,
     . dens,ddens,tekev,anpl,anpr,brr,bphi,bzz,ajphi
      real(wp_), dimension(jmx,kmx) :: grad2
      real(wp_), dimension(3,jmx,kmx) :: gri
      real(wp_), dimension(ndim,jmx,kmx) :: ywrk0,ypwrk0
      real(wp_), dimension(jmx,kmx,ndim) :: yyrfl
      real(wp_), dimension(ndimm,jmx,kmx) :: xc0,du10,dgrad2v
      real(wp_), dimension(3,3,jmx,kmx) :: ggri
      complex(wp_), dimension(jmx,kmx,0:4) :: ext,eyt
c
      common/nray/nrayr,nrayth
      common/wrk/ywrk0,ypwrk0
      common/grc/xc0,du10
      common/grad2jk/grad2
      common/dgrad2jk/dgrad2v
      common/gradjk/gri
      common/ggradjk/ggri
      common/ddd/dd,an2s,an2,fdia,bdotgr,ddi,ddr11
      common/yyrfl/yyrfl
      common/evt/ext,eyt
      common/nplr/anpl,anpr
      common/psival/psinv
      common/parpl/brr,bphi,bzz,ajphi
      common/dens/dens,ddens
      common/tete/tekev
c
      do j=1,nrayr
        do k=1,nrayth
          x0=yyrfl(j,k,1)
          y0=yyrfl(j,k,2)
          z0=yyrfl(j,k,3)
          anx0=yyrfl(j,k,4)
          any0=yyrfl(j,k,5)
          anz0=yyrfl(j,k,6)
          an20=anx0*anx0+any0*any0+anz0*anz0
          an0=sqrt(an20)
c
          xc0(1,j,k)=x0
          xc0(2,j,k)=y0
          xc0(3,j,k)=z0
c
          ywrk0(1,j,k)=x0
          ywrk0(2,j,k)=y0
          ywrk0(3,j,k)=z0
          ywrk0(4,j,k)=anx0/an0
          ywrk0(5,j,k)=any0/an0
          ywrk0(6,j,k)=anz0/an0
c
          ypwrk0(1,j,k) = anx0/an0
          ypwrk0(2,j,k) = any0/an0
          ypwrk0(3,j,k) = anz0/an0
          ypwrk0(4,j,k) = 0.0_wp_
          ypwrk0(5,j,k) = 0.0_wp_
          ypwrk0(6,j,k) = 0.0_wp_
c
          ytmp=ywrk0(:,j,k)
          yptmp=ypwrk0(:,j,k)
          call fwork(ytmp,yptmp)

          call pol_limit(ext(j,k,0),eyt(j,k,0))
            qq=abs(ext(j,k,0))**2-abs(eyt(j,k,0))**2
            uu=2.0_wp_*dble(ext(j,k,0)*dconjg(eyt(j,k,0)))
            vv=2.0_wp_*dimag(ext(j,k,0)*dconjg(eyt(j,k,0)))
            call polellipse(qq,uu,vv,psipol0,chipol0)
c
          do iv=1,3
            gri(iv,j,k)=0.0_wp_
            dgrad2v(iv,j,k)=0.0_wp_
            du10(iv,j,k)=0.0_wp_
            do jv=1,3
              ggri(iv,jv,j,k)=0.0_wp_
            end do
          end do
          grad2(j,k)=0.0_wp_
c
          dd=anx0**2+any0**2+anz0**2-an20
          vgradi=0.0_wp_
          ddi=2.0_wp_*vgradi
c
          r0=sqrt(x0**2+y0**2)
          x0m=x0/1.0e2_wp_
          y0m=y0/1.0e2_wp_
          r0m=r0/1.0e2_wp_
          z0m=z0/1.0e2_wp_
          if(j.eq.nrayr) then
             write(33,111) izero,j,k,zero,x0m,y0m,r0m,z0m,
     .          psinv,zero,anpl,zero,one
          end if
          if(j.eq.1.and.k.eq.1) then
            write(17,99) zero,zero,zero,zero
           write(4,99) strfl11,r0m,z0m,atan2(y0m,x0m)*180.0_wp_/pi,
     .      psinv,one,dens,tekev,brr,bphi,bzz,
     .      ajphi*1.0e-6_wp_,sqrt(anpl**2+anpr**2),anpl,zero,
     .      zero,zero,zero,zero,zero,zero,zero,one
          end if  
        end do
      end do
c
      call pweigth
c
      if(nrayr.gt.1) then
        iproj=0
        nfilp=8
        call projxyzt(iproj,nfilp)
      end if
c
      return
 99   format(24(1x,e16.8e3))
111   format(3i5,20(1x,e16.8e3))
      end
c
c
c
      subroutine pweigth
c     ray power weigth coefficient q(j)
c
      use const_and_precisions, only : wp_
      use graydata_par, only : rwmax
      implicit none
c local constants
      integer, parameter :: jmx=31
c local variables
      integer :: j,k
      real(wp_) :: dr,r1,r2,summ,sm
c common/external functions/variables
      integer :: nrayr,nrayth
      real(wp_), dimension(jmx) :: q
c
      common/qw/q
      common/nray/nrayr,nrayth
c
      dr=1.0_wp_
      if(nrayr.gt.1) dr=rwmax/dble(nrayr-1)
      r1=0.0_wp_
      summ=0.0_wp_
      q(1)=1.0_wp_
      if(nrayr.gt.1) then
        do j=1,nrayr
            r2=(dble(j)-0.5_wp_)*dr
          q(j)=(exp(-2.0_wp_*r1**2)-exp(-2.0_wp_*r2**2))
          r1=r2
          summ=summ+q(j)
        end do
c
        q(1)=q(1)/summ
        sm=q(1)
        j=1
        k=1
        do j=2,nrayr
          q(j)=q(j)/nrayth/summ
          do k=1,nrayth
            sm=sm+q(j)
          end do
        end do
      end if
c
      return
      end
c
c
c
      subroutine valpsispl(rpsi,voli,dervoli,areai,rrii,rbavi,
     .                     bmxi,bmni,fci,intp)
      use const_and_precisions, only : wp_
      use simplespline, only :spli,splid
      use magsurf_data, only : rpstab,cvol,crri,crbav,cbmx,cbmn,
     . carea,cfc,npsi
      implicit none
c arguments
      integer :: intp
      real(wp_) :: rpsi,voli,dervoli,areai,rrii,rbavi,bmxi,bmni,fci
c local variables
      integer :: ip
      real(wp_) :: dps
c
      ip=int((npsi-1)*rpsi+1)
c      if(ip.eq.0) ip=1
c      if(ip.eq.npsi) ip=npsi-1
      ip=min(max(1,ip),npsi-1)
c
      dps=rpsi-rpstab(ip)
c
      areai=spli(carea,npsi,ip,dps)
      voli=spli(cvol,npsi,ip,dps)
      dervoli=splid(cvol,npsi,ip,dps)
      rrii=spli(crri,npsi,ip,dps)
c
      if(intp.eq.0) return
c
      rbavi=spli(crbav,npsi,ip,dps)
      bmxi=spli(cbmx,npsi,ip,dps)
      bmni=spli(cbmn,npsi,ip,dps)
      fci=spli(cfc,npsi,ip,dps)
c
      return
      end
c
c
c
      subroutine ratioj(rpsi,ratjai,ratjbi,ratjpli)
      use const_and_precisions, only : wp_
      use magsurf_data, only : rpstab,cratja,cratjb,cratjpl,npsi
      use simplespline, only :spli
      implicit none
c arguments
      real(wp_) :: rpsi,ratjai,ratjbi,ratjpli
c local variables
      integer :: ip
      real(wp_) :: dps
c
      ip=int((npsi-1)*rpsi+1)
c      if(ip.eq.0) ip=1
c      if(ip.eq.npsi) ip=npsi-1
      ip=min(max(1,ip),npsi-1)
      dps=rpsi-rpstab(ip)
      ratjai=spli(cratja,npsi,ip,dps)
      ratjbi=spli(cratjb,npsi,ip,dps)
      ratjpli=spli(cratjpl,npsi,ip,dps)
c
      return
      end
c
c
c
      subroutine pabs_curr(i,j,k)
      use const_and_precisions, only : wp_,pi
      use graydata_flags, only : dst
      use graydata_par, only : sgnbphi
      use graydata_anequil, only : rr0m
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36,nmx=8000
c arguments
      integer :: i,j,k
c local variables
      integer :: intp
      real(wp_) :: dvvl,rbavi,rrii,rhop0,rhop,voli,dervoli,area,adnm,
     . dtau,effjcdav,dcidst,dpdst
c common/external functions/variables
      real(wp_) :: p0mw,dersdst,alpha,effjcd,akim,tau0,psinv,
     . bmxi,bmni,fci
      real(wp_), dimension(jmx) :: q
      real(wp_), dimension(jmx,kmx) :: tau1v
      real(wp_), dimension(jmx,kmx,nmx) :: psjki,tauv,alphav,pdjki,
     . ppabs,currj,didst,ccci
c
      common/psjki/psjki
      common/atjki/tauv,alphav
      common/dpjjki/pdjki,currj
      common/pcjki/ppabs,ccci
      common/dcjki/didst
      common/tau1v/tau1v
      common/qw/q
      common/p0/p0mw
      common/dersdst/dersdst
      common/absor/alpha,effjcd,akim,tau0
      common/psival/psinv
      common/bmxmn/bmxi,bmni
      common/fc/fci
c
      dvvl=1.0_wp_
      rbavi=1.0_wp_
      rrii=rr0m
      rhop0=sqrt(psjki(j,k,i-1))
        intp=1
        psinv=psjki(j,k,i)
        rhop=sqrt(psinv)
        call valpsispl(rhop,voli,dervoli,area,rrii,
     .              rbavi,bmxi,bmni,fci,intp)
        dvvl=dervoli*abs(rhop-rhop0)
      if(dvvl.le.0.0_wp_) dvvl=1.0_wp_
c
      adnm=2.0_wp_*pi*rrii
c
c       calcolo della corrente cohen: currtot [MA]
c       calcolo della densita' corrente cohen: currj  [MA/m^2]
c       calcolo della densita' potenza:  pdjki       [MW/m^3]
c       calcolo della efficienza <j/p>:  effjcdav      [A m/W ]
c
      tau0=tauv(j,k,i-1)
c
      call ecrh_cd
c
      alphav(j,k,i)=alpha
      dtau=(alphav(j,k,i)+alphav(j,k,i-1))*dersdst*dst/2.0_wp_
      tauv(j,k,i)=tauv(j,k,i-1)+dtau
      dpdst=p0mw*q(j)*exp(-tauv(j,k,i)-tau1v(j,k))*
     . alphav(j,k,i)*dersdst
      pdjki(j,k,i)=dpdst*dst/dvvl
      ppabs(j,k,i)=p0mw*q(j)*exp(-tau1v(j,k))*
     . (1.0_wp_-exp(-tauv(j,k,i)))
      effjcdav=rbavi*effjcd
      currj(j,k,i)=sgnbphi*effjcdav*pdjki(j,k,i)
      didst(j,k,i)=sgnbphi*effjcdav*dpdst/adnm
      dcidst=(didst(j,k,i)+didst(j,k,i-1))/2.0_wp_
      ccci(j,k,i)=ccci(j,k,i-1)+dcidst*dst
      return
      end
c
c
c
      subroutine ecrh_cd
      use const_and_precisions, only : wp_,mc2=>mc2_
      use dispersion, only : harmnumber, warmdisp
      use graydata_flags, only : iwarm,ilarm,ieccd,imx
      implicit none
c local constants
      real(wp_), parameter :: taucr=12.0_wp_,xxcr=16.0_wp_,eps=1.e-8_wp_
c local variables
      integer :: lrm,ithn
      real(wp_) :: amu,ratiovgr,rbn,rbx,zeff
c common/external functions/variables
      integer :: nhmin,nhmax,iokhawa,ierr
      real(wp_) :: xg,yg,anpl,anpr,vgm,derdnm,ak0,akinv,fhz,sox,
     . anprre,anprim,alpha,effjcd,akim,tau,psinv,tekev,dens,ddens,
     . btot,bmax,bmin,fc
      complex(wp_) :: ex,ey,ez
      real(wp_) :: fzeff,temp
c
      common/nharm/nhmin,nhmax
      common/xgxg/xg
      common/ygyg/yg
      common/nplr/anpl,anpr
      common/vgv/vgm,derdnm
      common/parwv/ak0,akinv,fhz
      common/mode/sox
      common/nprw/anprre,anprim
      common/epolar/ex,ey,ez
      common/absor/alpha,effjcd,akim,tau
      common/ierr/ierr
      common/iokh/iokhawa
      common/psival/psinv
      common/tete/tekev
      common/dens/dens,ddens
      common/btot/btot
      common/bmxmn/bmax,bmin
      common/fc/fc
c
c     absorption computation
c
      alpha=0.0_wp_
      akim=0.0_wp_
      effjcd=0.0_wp_
c
      tekev=temp(psinv)
c
      if(tekev.le.0.0_wp_.or.tau.gt.taucr) return
c
      amu=mc2/tekev
      call harmnumber(yg,amu,anpl,nhmin,nhmax,iwarm)
      if(nhmin.eq.0) return
c
      lrm=max(ilarm,nhmax)
      call warmdisp(xg,yg,amu,anpl,anpr,sox,lrm,ierr,anprre,anprim,
     *              iwarm,imx,ex,ey,ez)
c
      akim=ak0*anprim
      ratiovgr=2.0_wp_*anpr/derdnm
c      ratiovgr=2.0_wp_*anpr/derdnm*vgm
      alpha=2.0_wp_*akim*ratiovgr
      if(alpha.lt.0.0_wp_) then
        ierr=94
        print*,'    IERR = ', ierr,'  alpha negative'
      end if
c
      if(ieccd.gt.0) then
        ithn=1
        if(lrm.gt.nhmin) ithn=2
        zeff=fzeff(psinv)
        rbn=btot/bmin
        rbx=btot/bmax
        call eccd(yg,anpl,anprre,amu,zeff,rbn,rbx,fc,ex,ey,ez,
     *  dens,psinv,ieccd,nhmin,nhmax,ithn,effjcd,iokhawa,ierr)
      end if

      return
      end
c
c
c
      subroutine eccd(yg,anpl,anprre,amu,zeff,rbn,rbx,fc,ex,ey,ez,
     .  dens,psinv,ieccd,nhmn,nhmx,ithn,effjcd,iokhawa,ierr)
      use const_and_precisions, only : wp_,pi,qesi=>e_,mesi=>me_,
     . vcsi=>c_,qe=>ecgs_,me=>mecgs_,vc=>ccgs_,mc2=>mc2_
      use green_func_p, only: SpitzFuncCoeff
      use conical, only : fconic
      use magsurf_data, only : ch,tjp,tlm,njpt,nlmt
      use dierckx, only : profil
      implicit none
c local constants
      real(wp_), parameter :: mc2m2=1.0_wp_/mc2**2,
     . canucc=2.0e13_wp_*pi*qe**4/(me**2*vc**3),ceff=qesi/(mesi*vcsi)
      integer, parameter :: ksp=3
c arguments
      integer ieccd,nhmn,nhmx,ithn,iokhawa,ierr
      real(wp_) :: yg,anpl,anprre,amu,Zeff,rbn,rbx,
     . fc,dens,psinv,effjcd
      complex(wp_) :: ex,ey,ez
c local variables
      integer nhn,njp,nlm,npar
      real(wp_) :: anum,denom,resp,resj,coullog,anucc,alams,
     . fp0s,pa,cst2
      real(wp_) :: chlm(nlmt)
      real(wp_), dimension(3+2*nlmt) :: eccdpar ! dimension(max(5,3+nlmt))
c
      real(wp_), external :: fjch,fjncl,fjch0
c
      anum=0.0_wp_
      denom=0.0_wp_
      effjcd=0.0_wp_
c
      coullog=48.0_wp_-log(1.0e7_wp_*dens*mc2m2*amu**2)
      anucc=canucc*dens*coullog
c
c      nhmx=nhmn
c
      eccdpar(1)=zeff
c
      select case(ieccd) 
c
      case(1)
c     cohen model      
c     rbn=B/B_min
c     rbx=B/B_max
c     cst2=1.0_wp_-B/B_max
c     alams=sqrt(1-B_min/B_max)
c     Zeff < 31 !!!
c     fp0s= P_a (alams)   
        cst2=1.0_wp_-rbx
        if(cst2.lt.1.0e-6_wp_) cst2=0.0_wp_
        alams=sqrt(1.0_wp_-rbx/rbn)
        pa=sqrt(32.0_wp_/(Zeff+1.0_wp_)-1.0_wp_)/2.0_wp_
        fp0s=fconic(alams,pa,0)
        eccdpar(2)=rbn
        eccdpar(3)=alams
        eccdpar(4)=pa
        eccdpar(5)=fp0s
        do nhn=nhmn,nhmx
          call curr_int(yg,anpl,anprre,amu,ex,ey,ez,nhn,ithn,cst2,
     .                  fjch,eccdpar,5,resj,resp,iokhawa,ierr)
          anum=anum+resj
          denom=denom+resp
        end do
       
      case(2:9)
        cst2=0.0_wp_
        do nhn=nhmn,nhmx
          call curr_int(yg,anpl,anprre,amu,ex,ey,ez,nhn,ithn,cst2,
     .                  fjch0,eccdpar,1,resj,resp,iokhawa,ierr)
          anum=anum+resj
          denom=denom+resp
        end do
     
      case(10:11)
c     neoclassical model:
c     ft=1-fc trapped particle fraction
c     rzfc=(1+Zeff)/fc     
        cst2=1.0_wp_-rbx
        if(cst2.lt.1.0e-6_wp_) cst2=0.0_wp_
        call SpitzFuncCoeff(amu,Zeff,fc)
        eccdpar(2) = fc
        eccdpar(3) = rbx

        njp=njpt
        nlm=nlmt
        call profil(0,tjp,njp,tlm,nlm,ch,ksp,ksp,sqrt(psinv),nlm,chlm,
     .              ierr)
        if(ierr.gt.0) print*,'    Hlambda profil =',ierr
        npar=3+2*nlm
        eccdpar(4:3+nlm) = tlm
        eccdpar(4+nlm:npar) = chlm
        do nhn=nhmn,nhmx
          call curr_int(yg,anpl,anprre,amu,ex,ey,ez,nhn,ithn,cst2,
     .                  fjncl,eccdpar,npar,resj,resp,iokhawa,ierr)
          anum=anum+resj
          denom=denom+resp
        end do
       
       CASE DEFAULT
       print*,'ieccd undefined'
       
      end select
c
c     effjpl = <J_parallel>/<p_d> /(B_min/<B>)  [A m /W]
c
      if(denom.gt.0.0_wp_) effjcd=-ceff*anum/(anucc*denom)
      return
      end
c
c
c
      subroutine curr_int(yg,anpl,anprre,amu,ex,ey,ez,nhn,ithn,cst2,
     *   fcur,eccdpar,necp,resj,resp,iokhawa,ierr)
      use const_and_precisions, only : wp_
      use quadpack, only : dqagsmv
      implicit none
c local constants
      real(wp_), parameter :: epsa=0.0_wp_,epsr=1.0e-2_wp_,xxcr=16.0_wp_
      integer, parameter :: lw=5000,liw=lw/4,nfpp=15
c arguments
      integer :: i,nhn,ithn,necp,iokhawa,ierr
      real(wp_) :: yg,anpl,anprre,amu,cst2,resj,resp
      real(wp_), dimension(necp) :: eccdpar
      complex(wp_) :: ex,ey,ez
c local variables
      integer :: neval,ier,last,npar
      integer, dimension(liw) :: iw
      real(wp_) :: anpl2,dnl,ygn,ygn2,resj1,resj2,rdu2,upltp,upltm,
     . rdu,rdut,rdu2t,duu,uu1,uu2,xx1,xx2,ej,ej1,ej2,epp,uplp,uplm,
     . cst
      real(wp_), dimension(lw) :: w
      real(wp_), dimension(nfpp+necp) :: apar
c
      real(wp_), external :: fcur,fpp
c
c     EC power and current densities
c
      anpl2=anpl*anpl
      dnl=1.0_wp_-anpl2
      ygn=nhn*yg
      ygn2=ygn*ygn

      resj=0.0_wp_
      resj1=0.0_wp_
      resj2=0.0_wp_
      resp=0.0_wp_
c
      rdu2=anpl2+ygn2-1.0_wp_
      uplp=0.0_wp_
      uplm=0.0_wp_
      upltp=0.0_wp_
      upltm=0.0_wp_
c
      if (rdu2.ge.0.0_wp_) then
        rdu=sqrt(rdu2)
        uplp=(anpl*ygn+rdu)/dnl
        uplm=(anpl*ygn-rdu)/dnl
c
        uu1=uplm
        uu2=uplp
        xx1=amu*(anpl*uu1+ygn-1.0_wp_)
        xx2=amu*(anpl*uu2+ygn-1.0_wp_)
c
        if(xx2.gt.xxcr) uu2=(xxcr/amu-ygn+1.0_wp_)/anpl
        if(xx1.gt.xxcr) uu1=(xxcr/amu-ygn+1.0_wp_)/anpl
        duu=abs(uu1-uu2)
c
        apar(1) = yg
        apar(2) = anpl
        apar(3) = amu
        apar(4) = anprre
        apar(5) = dble(ex)
        apar(6) = dimag(ex)
        apar(7) = dble(ey)
        apar(8) = dimag(ey)
        apar(9) = dble(ez)
        apar(10) = dimag(ez)
        apar(11) = dble(ithn)
        apar(12) = dble(nhn)
        apar(13) = uplp
        apar(14) = uplm
        apar(15) = ygn
c
        npar=nfpp+necp
        do i=1,necp
          apar(nfpp+i) = eccdpar(i)
        end do
c
        if(duu.gt.1.0e-6_wp_) then
          call dqagsmv(fpp,uu1,uu2,apar,npar,epsa,epsr,resp,epp,neval,
     .             ier,liw,lw,last,iw,w)
          if (ier.gt.0) ierr=90
        end if
c
        rdu2t=cst2*anpl2+ygn2-1.0_wp_
c
        if (rdu2t.lt.0.0_wp_.or.cst2.eq.0.0_wp_) then
c
c       resonance curve does not cross the trapping region
c
          iokhawa=0
          if(duu.gt.1.0e-4_wp_) then
            call dqagsmv(fcur,uu1,uu2,apar,npar,epsa,epsr,
     .               resj,ej,neval,ier,liw,lw,last,iw,w)
            if (ier.gt.0) ierr=91
          end if
        else
c
c       resonance curve crosses the trapping region
c
          iokhawa=1
          rdut=sqrt(rdu2t)
          cst=sqrt(cst2)
          upltm=(cst2*anpl*ygn-cst*rdut)/(1.0_wp_-cst2*anpl2)
          upltp=(cst2*anpl*ygn+cst*rdut)/(1.0_wp_-cst2*anpl2)
c
          uu1=uplm
          uu2=upltm
          xx1=amu*(anpl*uu1+ygn-1.0_wp_)
          xx2=amu*(anpl*uu2+ygn-1.0_wp_)
          if(xx1.lt.xxcr.or.xx2.lt.xxcr) then
            if(xx2.gt.xxcr) uu2=(xxcr/amu-ygn+1.0_wp_)/anpl
            if(xx1.gt.xxcr) uu1=(xxcr/amu-ygn+1.0_wp_)/anpl
            duu=abs(uu1-uu2)
            if(duu.gt.1.0e-6_wp_) then
              call dqagsmv(fcur,uu1,uu2,apar,npar,epsa,epsr,
     .        resj1,ej1,neval,ier,liw,lw,last,iw,w)
              if (ier.gt.0)   then
              if (abs(resj1).lt.1.0e-10_wp_) then
                resj1=0.0_wp_
              else
                ierr=92
              end if
              end if
            end if
          end if
c
          uu1=upltp
          uu2=uplp
          xx1=amu*(anpl*uu1+ygn-1.0_wp_)
          xx2=amu*(anpl*uu2+ygn-1.0_wp_)
          if(xx1.lt.xxcr.or.xx2.lt.xxcr) then
            if(xx2.gt.xxcr) uu2=(xxcr/amu-ygn+1.0_wp_)/anpl
            if(xx1.gt.xxcr) uu1=(xxcr/amu-ygn+1.0_wp_)/anpl
            duu=abs(uu1-uu2)
            if(duu.gt.1.0e-6_wp_) then
              call dqagsmv(fcur,uu1,uu2,apar,npar,epsa,epsr,
     .        resj2,ej2,neval,ier,liw,lw,last,iw,w)
              if (ier.gt.0) then
                if(ier.ne.2) ierr=93
            end if
            end if
          end if
          resj=resj1+resj2
        end if
      end if

      return
      end
c
c     computation of integral for power density, integrand function fpp
c
c     ith=0 : polarization term = const
c     ith=1 : polarization term  Larmor radius expansion to lowest order
c     ith=2 : full polarization term (J Bessel)
c
      function fpp(upl,extrapar,npar)
c     integration variable upl passed explicitly. other variables passed
c     as array of extra parameters of length npar=size(extrapar)
c     
c      extrapar(1) = yg
c      extrapar(2) = anpl
c      extrapar(3) = amu
c      extrapar(4) = Re(anprw)
c      extrapar(5) = Re(ex)
c      extrapar(6) = Im(ex)
c      extrapar(7) = Re(ey)
c      extrapar(8) = Im(ey)
c      extrapar(9) = Re(ez)
c      extrapar(10) = Im(ez)
c      extrapar(11) = double(ithn)
c      extrapar(12) = double(nhn)
c      extrapar(13) = uplp
c      extrapar(14) = uplm
c      extrapar(15) = ygn
c
      use const_and_precisions, only : wp_,ui=>im
      use math, only : fact
      implicit none
c arguments
      integer :: npar
      real(wp_) :: upl,fpp
      real(wp_), dimension(npar) :: extrapar
c local variables
      integer :: ithn,nhn,nm,np
      real(wp_) :: yg,anpl,amu,anprre,uplp,uplm,ygn,upr,upr2,gam,ee,
     . thn2,thn2u,bb,cth,ajbnm,ajbnp,ajbn
      complex(wp_) :: ex,ey,ez,emxy,epxy
c
      yg=extrapar(1)
      anpl=extrapar(2)
      amu=extrapar(3)
      anprre=extrapar(4)
      ex=dcmplx(extrapar(5),extrapar(6))
      ey=dcmplx(extrapar(7),extrapar(8))
      ez=dcmplx(extrapar(9),extrapar(10))
      ithn=int(extrapar(11))
      nhn=int(extrapar(12))
      uplp=extrapar(13)
      uplm=extrapar(14)
      ygn=extrapar(15)
      
      upr2=(1.0_wp_-anpl**2)*(uplp-upl)*(upl-uplm)
      gam=anpl*upl+ygn
      ee=exp(-amu*(gam-1))

      thn2=1.0_wp_
      thn2u=upr2*thn2
      if(ithn.gt.0) then
        emxy=ex-ui*ey
        epxy=ex+ui*ey
        if(upr2.gt.0.0_wp_) then
          upr=sqrt(upr2)
          bb=anprre*upr/yg
          if(ithn.eq.1)  then
c     Larmor radius expansion polarization term at lowest order
            cth=1.0_wp_
            if(nhn.gt.1) cth=(0.5_wp_*bb)**(nhn-1)*nhn/fact(nhn)
            thn2=(0.5_wp_*cth*abs(emxy+ez*anprre*upl/ygn))**2
            thn2u=upr2*thn2
          else
c     Full polarization term
            nm=nhn-1
            np=nhn+1
            ajbnm=dbesjn(nm, bb)
            ajbnp=dbesjn(np, bb)
            ajbn=dbesjn(nhn, bb)
           thn2u=(abs(ez*ajbn*upl+upr*(ajbnp*epxy+ajbnm*emxy)
     .           /2.0_wp_))**2
          end if
        end if
      end if
      
      fpp=ee*thn2u
c
      return
      end
c
c     computation of integral for current density
c     fjch integrand for Cohen model with trapping
c     fjch0 integrand for Cohen model without trapping    
c     fjncl integrand for momentum conserv. model K(u) from Maruschenko
c     gg=F(u)/u with F(u) as in Cohen paper
c
      function fjch(upl,extrapar,npar)
c     integration variable upl passed explicitly. Other variables passed
c     as array of extra parameters of length npar=size(extrapar).
c     variables with index 1..15 must be passed to fpp
c     variable with index 16 is zeff
c     variables with index gt 16 are specific of the cd model
c     
c      extrapar(1) = yg
c      extrapar(2) = anpl
c      extrapar(3) = amu
c      extrapar(4) = Re(anprw)
c      extrapar(5) = Re(ex)
c      extrapar(6) = Im(ex)
c      extrapar(7) = Re(ey)
c      extrapar(8) = Im(ey)
c      extrapar(9) = Re(ez)
c      extrapar(10) = Im(ez)
c      extrapar(11) = double(ithn)
c      extrapar(12) = double(nhn)
c      extrapar(13) = uplp
c      extrapar(14) = uplm
c      extrapar(15) = ygn
c
c      extrapar(16) = zeff
c      extrapar(17) = rb
c      extrapar(18) = alams
c      extrapar(19) = pa
c      extrapar(20) = fp0s
c
      use const_and_precisions, only : wp_
      use conical, only : fconic
      implicit none
c arguments
      integer :: npar
      real(wp_) :: upl,fjch
      real(wp_), dimension(npar) :: extrapar
c local variables
      integer :: nhn
      real(wp_) :: anpl,anprre,uplp,uplm,ygn,zeff,rb,alams,pa,fp0s,
     . upr2,gam,u2,u,z5,xi,xib,xibi,fu2b,fu2,gu,gg,dgg,alam,fp0,
     . dfp0,fh,dfhl,eta,fpp
      complex(wp_) :: ex,ey,ez
c
      anpl=extrapar(2)
      anprre=extrapar(4)
      ex=dcmplx(extrapar(5),extrapar(6))
      ey=dcmplx(extrapar(7),extrapar(8))
      ez=dcmplx(extrapar(9),extrapar(10))
      nhn=int(extrapar(12))
      uplp=extrapar(13)
      uplm=extrapar(14)
      ygn=extrapar(15)
      zeff=extrapar(16)
      rb=extrapar(17)
      alams=extrapar(18)
      pa=extrapar(19)
      fp0s=extrapar(20)

      upr2=(1.0_wp_-anpl**2)*(uplp-upl)*(upl-uplm)
      gam=anpl*upl+ygn
      u2=gam*gam-1.0_wp_
      u=sqrt(u2)
      z5=Zeff+5.0_wp_
      xi=1.0_wp_/z5**2
      xib=1.0_wp_-xi
      xibi=1.0_wp_/xib
      fu2b=1.0_wp_+xib*u2
      fu2=1.0_wp_+xi*u2
      gu=(1.0_wp_-1.0_wp_/fu2b**xibi)/sqrt(fu2)
      gg=u*gu/z5
      dgg=(gu+u2*(2.0_wp_/fu2b**(1.0_wp_+xibi)/sqrt(fu2)-xi*gu/fu2))/z5

      alam=sqrt(1.0_wp_-upr2/u2/rb)
      fp0=fconic(alam,pa,0)
      dfp0=-(pa*pa/2.0_wp_+0.125_wp_)
      if (alam.lt.1.0_wp_) then
        dfp0=-fconic(alam,pa,1)/sqrt(1.0_wp_-alam**2)
      end if
      fh=alam*(1.0_wp_-alams*fp0/(alam*fp0s))
      dfhl=1.0_wp_-alams*dfp0/fp0s

      eta=gam*fh*(gg/u+dgg)+upl*(anpl*u2-upl*gam)*gg*dfhl/(u2*u*rb*alam)

      if(upl.lt.0.0_wp_) eta=-eta
      fjch=eta*fpp(upl,extrapar,npar)
c
      return
      end
c
c
c
      function fjch0(upl,extrapar,npar)
c     integration variable upl passed explicitly. Other variables passed
c     as array of extra parameters of length npar=size(extrapar).
c     variables with index 1..15 must be passed to fpp
c     variable with index 16 is zeff
c     variables with index gt 16 are specific of the cd model
c     
c      extrapar(1) = yg
c      extrapar(2) = anpl
c      extrapar(3) = amu
c      extrapar(4) = Re(anprw)
c      extrapar(5) = Re(ex)
c      extrapar(6) = Im(ex)
c      extrapar(7) = Re(ey)
c      extrapar(8) = Im(ey)
c      extrapar(9) = Re(ez)
c      extrapar(10) = Im(ez)
c      extrapar(11) = double(ithn)
c      extrapar(12) = double(nhn)
c      extrapar(13) = uplp
c      extrapar(14) = uplm
c      extrapar(15) = ygn
c
c      extrapar(16) = zeff
c
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_) upl,fjch0
      integer :: npar
      real(wp_), dimension(npar) :: extrapar
c local variables
      integer :: nhn
      real(wp_) :: anpl,anprre,ygn,zeff,gam,u2,u,z5,xi,xib,xibi,
     . fu2b,fu2,gu,gg,dgg,eta,fpp
      complex(wp_) :: ex,ey,ez
c
      anpl=extrapar(2)
      anprre=extrapar(4)
      ex=dcmplx(extrapar(5),extrapar(6))
      ey=dcmplx(extrapar(7),extrapar(8))
      ez=dcmplx(extrapar(9),extrapar(10))
      nhn=int(extrapar(12))
      ygn=extrapar(15)
      zeff=extrapar(16)

      gam=anpl*upl+ygn
      u2=gam*gam-1.0_wp_
      u=sqrt(u2)
      z5=Zeff+5.0_wp_
      xi=1.0_wp_/z5**2
      xib=1.0_wp_-xi
      xibi=1.0_wp_/xib
      fu2b=1.0_wp_+xib*u2
      fu2=1.0_wp_+xi*u2
      gu=(1.0_wp_-1.0_wp_/fu2b**xibi)/sqrt(fu2)
      gg=u*gu/z5
      dgg=(gu+u2*(2.0_wp_/fu2b**(1.0_wp_+xibi)/sqrt(fu2)-xi*gu/fu2))/z5
      eta=anpl*gg+gam*upl*dgg/u
      fjch0=eta*fpp(upl,extrapar,npar)
c
      return
      end
c
c
c
      function fjncl(upl,extrapar,npar)
c     integration variable upl passed explicitly. Other variables passed
c     as array of extra parameters of length npar=size(extrapar).
c     variables with index 1..15 must be passed to fpp
c     variable with index 16 is zeff
c     variables with index gt 16 are specific of the cd model
c     
c      extrapar(1) = yg
c      extrapar(2) = anpl
c      extrapar(3) = amu
c      extrapar(4) = Re(anprw)
c      extrapar(5) = Re(ex)
c      extrapar(6) = Im(ex)
c      extrapar(7) = Re(ey)
c      extrapar(8) = Im(ey)
c      extrapar(9) = Re(ez)
c      extrapar(10) = Im(ez)
c      extrapar(11) = double(ithn)
c      extrapar(12) = double(nhn)
c      extrapar(13) = uplp
c      extrapar(14) = uplm
c      extrapar(15) = ygn
c
c      extrapar(16) = zeff
c      extrapar(17) = fc
c      extrapar(18) = hb
c      extrapar(19:18+(npar-18)/2) = tlm
c      extrapar(19+(npar-18)/2:npar) = chlm
c
      use const_and_precisions, only : wp_
      use  green_func_p, only: GenSpitzFunc
      implicit none
c arguments
      integer :: npar
      real(wp_) :: upl,fjncl
      real(wp_), dimension(npar) :: extrapar
c local variables
      real(wp_) :: anpl,amu,ygn,zeff,fc,hb,gam,u2,u,upr2,
     . bth,uth,fk,dfk,alam,fu,dfu,eta,fpp
      integer :: nlm
c
      anpl=extrapar(2)
      amu=extrapar(3)
      ygn=extrapar(15)
      zeff=extrapar(16)
      fc=extrapar(17)
      hb=extrapar(18)

      gam=anpl*upl+ygn
      u2=gam*gam-1.0_wp_
      u=sqrt(u2)
      upr2=u2-upl*upl
      bth=sqrt(2.0_wp_/amu)
      uth=u/bth
      call GenSpitzFunc(Zeff,fc,uth,u,gam,fk,dfk)
      fk=fk*(4.0_wp_/amu**2)
      dfk=dfk*(2.0_wp_/amu)*bth
      
      alam=upr2/u2/hb
      nlm=(npar-18)/2
      call vlambda(alam,extrapar(19),
     .             extrapar(19+nlm),
     .             nlm,fu,dfu)
      
      eta=gam*fu*dfk/u-2.0_wp_*(anpl-gam*upl/u2)*fk*dfu*upl/u2/hb     
      if(upl.lt.0) eta=-eta
      fjncl=eta*fpp(upl,extrapar,npar)
      return
      end
c
c
c
      subroutine vlambda(alam,tlm,chlm,nlmt,fv,dfv)
      use const_and_precisions, only : wp_
      use dierckx, only : splev,splder
      implicit none
c local constants
      integer, parameter :: ksp=3
c arguments
      real(wp_) :: alam,fv,dfv
      integer :: nlmt
      real(wp_), dimension(nlmt) :: tlm,chlm
c local variables
      integer :: nlm,ier
      real(wp_), dimension(nlmt) :: wrk
      real(wp_), dimension(1) :: xxs,ffs
c
      nlm=nlmt
      xxs(1)=alam
c
      call splev(tlm,nlm,chlm,ksp,xxs(1),ffs(1),1,ier)
      fv=ffs(1)
c
      call splder(tlm,nlm,chlm,ksp,1,xxs(1),ffs(1),1,wrk,ier)
      dfv=ffs(1)
c
      return
      end  
c
c
c
      subroutine projxyzt(iproj,nfile)
      use const_and_precisions, only : wp_
      implicit none
c local constants
      integer, parameter :: jmx=31,kmx=36
c arguments
      integer iproj,nfile
c local variables
      integer :: jd,j,kkk,k
      real(wp_) :: dir,rtimn,rtimx,dx,dy,dz,dirx,diry,dirz,
     . csth1,snth1,csps1,snps1,xti,yti,zti,rti
c common/external functions/variables
      integer :: nrayr,nrayth,istep
      real(wp_) :: psinv11,st
      real(wp_), dimension(6,jmx,kmx) :: ywrk,ypwrk
c
      common/nray/nrayr,nrayth
      common/wrk/ywrk,ypwrk
      common/psinv11/psinv11
      common/istep/istep
      common/ss/st
c
      rtimn=1.0e+30_wp_
      rtimx=-1.0e-30_wp_
c
      jd=1
      if(iproj.eq.0) jd=nrayr-1
      do j=1,nrayr,jd
        kkk=nrayth
        if(j.eq.1) kkk=1
        do k=1,kkk
c
          dx=ywrk(1,j,k)-ywrk(1,1,1)
          dy=ywrk(2,j,k)-ywrk(2,1,1)
          dz=ywrk(3,j,k)-ywrk(3,1,1)
c
          dirx=ywrk(4,j,k)
          diry=ywrk(5,j,k)
          dirz=ywrk(6,j,k)
          dir=sqrt(dirx*dirx+diry*diry+dirz*dirz)
c
          if(j.eq.1.and.k.eq.1) then
            csth1=dirz/dir
            snth1=sqrt(1.0_wp_-csth1**2)
            csps1=1.0_wp_
            snps1=0.0_wp_
            if(snth1.gt.0.0_wp_) then
              csps1=diry/(dir*snth1)
              snps1=dirx/(dir*snth1)
            end if
          end if
          xti= dx*csps1-dy*snps1
          yti=(dx*snps1+dy*csps1)*csth1-dz*snth1
          zti=(dx*snps1+dy*csps1)*snth1+dz*csth1
          rti=sqrt(xti**2+yti**2)
Cc
c          dirxt= (dirx*csps1-diry*snps1)/dir
c          diryt=((dirx*snps1+diry*csps1)*csth1-dirz*snth1)/dir
c          dirzt=((dirx*snps1+diry*csps1)*snth1+dirz*csth1)/dir
Cc
c          if(k.eq.1) then
c            xti1=xti
c            yti1=yti
c            zti1=zti
c            rti1=rti
c          end if

          if(.not.(iproj.eq.0.and.j.eq.1))
     .    write(nfile,111) istep,j,k,xti,yti,zti,rti,psinv11
c
          if(rti.ge.rtimx.and.j.eq.nrayr) rtimx=rti
          if(rti.le.rtimn.and.j.eq.nrayr) rtimn=rti
c
        end do
c
c        if(.not.(iproj.eq.0.and.j.eq.1))
c     .    write(nfile,111) istep,j,k,xti,yti,zti,rti,psinv11
        if(iproj.eq.1) write(nfile,*) ' '
      end do
c
      write(nfile,*) ' '
c
      write(12,99) istep,st,psinv11,rtimn,rtimx
      return
 99   format(i5,12(1x,e16.8e3))
111   format(3i5,12(1x,e16.8e3))
      end
c
c
c
      subroutine pec(pabs,currt)
      use const_and_precisions, only : wp_,pi,one
      use numint, only : simpson
      use graydata_flags, only : ipec,ieccd,nnd,dst
      use utils, only : locatex, locate, intlin
      implicit none
c local constants
      integer, parameter :: nndmx=5001,jmx=31,kmx=36,nmx=8000,llmx=21
      real(wp_), parameter :: rtbc=one
c arguments
      real(wp_) :: pabs,currt
c local variables
      integer :: ll,it,nd,intp,kkk,i,j,k,ii,is,idecr,ise0,iis,iis1,
     . iise0,iise,ind1,ind2,iind,indi,iif1,istmx,i1,ind,itb1
      integer, dimension(llmx) :: isev
      real(wp_) :: stf,voli0,areai0,drt,rt,rt1,psit,psit1,voli1,
     . dervoli,areai1,rrii,rbavi,bmxi,bmni,fci,ppa1,cci1,ppa2,cci2,
     . dppa,didst,rhotpav,rhot2pav,rhotjav,rhotjava,rhot2java,spds,
     . sccs,sccsa,drhot2pav,drhotpav,drhot2java,drhotjava,facpds,
     . facjs,rhpp,rhpj,dpdvp,ajphip,ratjamx,ratjbmx,ratjplmx,rhopp,
     . drhopp,rhotjfi,rhopfi,ajmxfi,drhotjfi,drhopfi,xps,dpdvmx,
     . rhotp,drhotp,stmx,pins_02,pins_05,pins_085,xrhot,currtka,rhop,
     . pinsr,ratjpli,ratjai,ratjbi,frhotor,frhopol,fdadrhot,
     . fdvdrhot,h,psin
      real(wp_), dimension(nmx) :: xxi,ypt,yamp
      real(wp_), dimension(nndmx) :: ajphiv,dpdv,dvolt,darea,ratjav,
     . ratjbv,ratjplv,ajplv,ajcdav,ajcdbv,pins,currins,fi,rtab,rhotv
      real(wp_), dimension(0:nndmx) :: rtab1
c common/external functions/variables
      integer :: nrayr,nrayth,istep,index_rt
      integer, dimension(jmx,kmx) :: iiv
      real(wp_) :: alpha0,beta0,taumn,taumx,pabstot,currtot,
     . psipol,chipol
      real(wp_), dimension(jmx,kmx,nmx) :: psjki,ppabs,ccci,pdjki,currj
c
      common/nray/nrayr,nrayth
      common/istep/istep
      common/index_rt/index_rt
      common/iiv/iiv
      common/psjki/psjki
      common/pcjki/ppabs,ccci
      common/dpjjki/pdjki,currj
      common/angles/alpha0,beta0
      common/taumnx/taumn,taumx,pabstot,currtot
      common/polcof/psipol,chipol
c
      stf=istep*dst
      nd=nnd

      if(pabstot.gt.0.0_wp_) then
      do ll=1,llmx
        isev(ll)=0
      end do

      intp=0
      voli0=0.0_wp_
      areai0=0.0_wp_
      rtab1(0)=0.0_wp_
      do it=1,nd
        drt=1.0_wp_/dble(nd-1)
        rt=dble(it-1)*drt
        if(it.lt.nd) then
          rt1=rt+drt/2.0_wp_
        else
          rt1=rt
        end if
        rtab(it)=rt
        rtab1(it)=rt1
        dpdv(it)=0.0_wp_
        ajphiv(it)=0.0_wp_
        if (ipec.eq.0) then
          psit=rt
          psit1=rt1
        else
          psit=rt**2
          psit1=rt1**2
        end if
          rhotv(it)=frhotor(psit)
          call valpsispl(sqrt(psit1),voli1,dervoli,areai1,rrii,
     .                  rbavi,bmxi,bmni,fci,intp)
          dvolt(it)=abs(voli1-voli0)
          darea(it)=abs(areai1-areai0)
          voli0=voli1
          areai0=areai1
          call ratioj(sqrt(psit),ratjai,ratjbi,ratjpli)
          ratjav(it)=ratjai
          ratjbv(it)=ratjbi
          ratjplv(it)=ratjpli
      end do

      kkk=1
      do j=1,nrayr
        if(j.gt.1) kkk=nrayth
        do k=1,kkk
          ise0=0
          ii=iiv(j,k)
          if (ii.lt.nmx) then
            if(psjki(j,k,ii+1).ne.0.0_wp_) ii=ii+1
          end if
          idecr=-1
          is=1
          do i=1,ii
            if(ipec.eq.0) xxi(i)=abs(psjki(j,k,i))
            if(ipec.eq.1) xxi(i)=sqrt(abs(psjki(j,k,i)))
            if(psjki(j,k,i).ge.0.and.psjki(j,k,i).le.rtbc) then
              ypt(i)=ppabs(j,k,i)
              yamp(i)=ccci(j,k,i)
            else
              ypt(i)=0.0_wp_
              yamp(i)=0.0_wp_
            end if
            if(ise0.eq.0) then
              if(xxi(i).lt.rtbc) then
              ise0=i
              isev(is)=i-1
              is=is+1
              end if
            else
              if (idecr.eq.-1) then
                if(xxi(i).gt.xxi(i-1)) then
                  isev(is)=i-1
                  is=is+1
                  idecr=1
                 end if
              else
                if(xxi(i).gt.rtbc) exit
                if(xxi(i).lt.xxi(i-1)) then
c                  isev(is)=i    !!!!!!!!!! it should be isev(is)=i-1
                  isev(is)=i-1
                  is=is+1
                  idecr=-1
                end if
              end if
            end if
          end do
c
          isev(is)=i-1
          ppa1=0.0_wp_
          cci1=0.0_wp_
          do iis=1,is-1
            iis1=iis+1
            iise0=isev(iis)
            iise=isev(iis1)
            if (mod(iis,2).ne.0) then
              idecr=-1
              ind1=nd
              ind2=2
              iind=-1
            else
              idecr=1
              ind1=1
              ind2=nd
              iind=1
            end if
            do ind=ind1,ind2,iind     !!!!!!!!!! is it safe?
c              iind=ind                !!!!!!!!!! iind reused in the loop!
              indi=ind                !!!!!!!!!! iind reused in the loop!
              if (idecr.eq.-1) indi=ind-1
              rt1=rtab1(indi)
              call locatex(xxi,iise,iise0,iise,rt1,itb1)
              if(itb1.ge.iise0.and.itb1.lt.iise) then
                call intlin(xxi(itb1),ypt(itb1),xxi(itb1+1),
     .                      ypt(itb1+1),rt1,ppa2)
                call intlin(xxi(itb1),yamp(itb1),xxi(itb1+1),
     .                      yamp(itb1+1),rt1,cci2)
                dppa=ppa2-ppa1
                dpdv(ind)=dpdv(ind)+dppa
                didst=(cci2-cci1)
                ajphiv(ind)=ajphiv(ind)+didst
                ppa1=ppa2
                cci1=cci2
              end if
            end do
          end do
        end do
      end do
 
      h=1.0_wp_/dble(nd-1)
      rhotpav=0.0_wp_
      drhotpav=0.0_wp_
      rhotjav=0.0_wp_
      rhotjava=0.0_wp_
      rhot2java=0.0_wp_
     
      fi=dpdv/h
      call simpson (nd,h,fi,spds)
      fi=rhotv*dpdv/h
      call simpson (nd,h,fi,rhotpav)
      fi=rhotv*rhotv*dpdv/h
      call simpson (nd,h,fi,rhot2pav)
      rhotpav=rhotpav/spds
      rhot2pav=rhot2pav/spds

      if (ieccd.ne.0) then      
      fi=ajphiv/h
      call simpson (nd,h,fi,sccs)
      fi=rhotv*ajphiv/h
      call simpson (nd,h,fi,rhotjav)    
      fi=abs(ajphiv)/h
      call simpson (nd,h,fi,sccsa)
      fi=rhotv*abs(ajphiv)/h
      call simpson (nd,h,fi,rhotjava)
      fi=rhotv*rhotv*abs(ajphiv)/h
      call simpson (nd,h,fi,rhot2java)     
      rhotjav=rhotjav/sccs
      rhotjava=rhotjava/sccsa
      rhot2java=rhot2java/sccsa
      end if

c     factor sqrt(8)=2 sqrt(2) to match with full width
c     of gaussian profile
      drhot2pav=rhot2pav-rhotpav**2
      drhotpav=sqrt(8.0_wp_*drhot2pav)
      drhot2java=rhot2java-rhotjava**2
      drhotjava=sqrt(8.0_wp_*drhot2java)
      
      spds=0.0_wp_
      sccs=0.0_wp_
      do i=1,nd
        spds=spds+dpdv(i)
        sccs=sccs+ajphiv(i)
        pins(i)=spds
        currins(i)=sccs
        dpdv(i)=dpdv(i)/dvolt(i)
        ajphiv(i)=ajphiv(i)/darea(i)
      end do
      
      facpds=1.0_wp_
      facjs=1.0_wp_
      if(spds.gt.0.0_wp_) facpds=pabs/spds
      if(sccs.ne.0.0_wp_) facjs=currt/sccs

      do i=1,nd
        dpdv(i)=facpds*dpdv(i)
        ajphiv(i)=facjs*ajphiv(i)
        ajcdav(i)=ajphiv(i)*ratjav(i)
        ajcdbv(i)=ajphiv(i)*ratjbv(i)
        ajplv(i)=ajphiv(i)*ratjplv(i)
      end do
     
      rhpp=frhopol(rhotpav)
      rhpj=frhopol(rhotjava)
      dpdvp=pabs*2.0_wp_/(sqrt(pi)*drhotpav*fdvdrhot(rhpp))
      ajphip=currt*2.0_wp_/(sqrt(pi)*drhotjava*fdadrhot(rhpj))
      call ratioj(rhpj,ratjamx,ratjbmx,ratjplmx)
      
      call profwidth(nd,rtab,dpdv,rhotp,rhopp,dpdvmx,
     .                 drhotp,drhopp)
      if(ieccd.ne.0) then
        call profwidth(nd,rtab,ajphiv,rhotjfi,rhopfi,ajmxfi,
     .                 drhotjfi,drhopfi)
        xps=rhopfi
      else
        rhotjfi=0.0_wp_
        rhopfi=0.0_wp_
        ajmxfi=0.0_wp_
        ajphip=0.0_wp_
        drhotjfi=0.0_wp_
        drhopfi=0.0_wp_
        xps=rhopp
        ratjamx=1.0_wp_
        ratjbmx=1.0_wp_
        ratjplmx=1.0_wp_
      end if

       iif1=iiv(1,1)
       istmx=1
       do i=2,iif1
        if(psjki(1,1,i).ge.0.0_wp_) then
         if(pdjki(1,1,i).gt.pdjki(1,1,i-1)) istmx=i
        end if
      end do
      stmx=istmx*dst
      
      pins_02=0.0_wp_
      pins_05=0.0_wp_
      pins_085=0.0_wp_

      xrhot=0.2_wp_
      call locate(rhotv,nd,xrhot,i1)
      call intlin(rhotv(i1),pins(i1),rhotv(i1+1),pins(i1+1),
     .  xrhot,pins_02)
      xrhot=0.5_wp_
      call locate(rhotv,nd,xrhot,i1)
      call intlin(rhotv(i1),pins(i1),rhotv(i1+1),pins(i1+1),
     .  xrhot,pins_05)
      xrhot=0.85_wp_
      call locate(rhotv,nd,xrhot,i1)
      call intlin(rhotv(i1),pins(i1),rhotv(i1+1),pins(i1+1),
     .  xrhot,pins_085)
     
      else
      ajmxfi=0.0_wp_
      ajphip=0.0_wp_
      dpdvp=0.0_wp_
      dpdvmx=0.0_wp_
      rhotjfi=1.0_wp_
      rhotjav=1.0_wp_
      rhotjava=1.0_wp_
      rhotp=1.0_wp_
      rhotpav=1.0_wp_
      drhotjfi=0.0_wp_
      drhotjava=0.0_wp_
      drhotp=0.0_wp_
      drhotpav=0.0_wp_
      ratjamx=1.0_wp_
      ratjbmx=1.0_wp_
      taumn=0
      taumx=0
      stmx=stf
      pins_02=0.0_wp_
      pins_05=0.0_wp_
      pins_085=0.0_wp_     
c  end of pabstot > 0   
      end if     

c     dPdV [MW/m^3], Jcd [MA/m^2]

      if(ieccd.eq.0) currt=0.0_wp_
      currtka=currt*1.0e3_wp_

      write(6,*)' '
      write(6,*)'#beta0 alpha0 Icd Pa Jphip dPdVp '//
     .'rhotj rhotjava rhotp rhotpav drhotjava drhotpav ratjamx '//
     .'ratjbmx stmx psipol chipol index_rt Jphimx dPdVmx drhotj drhotp'
      write(6,99) beta0,alpha0,currtka,pabstot,ajphip,dpdvp,
     .  rhotjfi,rhotjava,rhotp,rhotpav,
     .  drhotjava,drhotpav,ratjamx,ratjbmx,stmx,psipol,chipol,
     .  real(index_rt),ajmxfi,dpdvmx,drhotjfi,drhotp
     
      write(7,99) currtka,pabstot,ajphip,dpdvp,
     .  rhotjfi,rhotjava,rhotp,rhotpav,
     .  drhotjava,drhotpav,ratjamx,ratjbmx,stmx,psipol,chipol,
     .  real(index_rt),ajmxfi,dpdvmx,drhotjfi,drhotp

      do i=1,nd
        if (ipec.eq.0) then
          psin=rtab(i)
          rhop=sqrt(rtab(i))
        else
          psin=rtab(i)**2
          rhop=rtab(i)
        end if
        pinsr=0.0_wp_
        if(pabstot.gt.0) pinsr=pins(i)/pabstot
        write(48,99) psin,rhotv(i),ajphiv(i),ajcdbv(i),dpdv(i),
     .   currins(i),pins(i),pinsr,real(index_rt)
      end do

      return
 99   format(30(1x,e12.5))
      end
c
c
c
      subroutine profwidth(nd,xx,yy,rhotmx,rhopmx,ypk,drhot,drhop)
      use const_and_precisions, only : wp_,emn1
      use graydata_flags, only : ipec
      use utils, only : locatex, locate, intlin, vmaxmini
      implicit none
c arguments
      integer :: nd
      real(wp_) :: rhotmx,rhopmx,ypk,drhot,drhop
      real(wp_), dimension(nd) :: xx,yy
c local variables
      integer :: imn,imx,ipk,ie1,ie2
      real(wp_) :: xmn,xmx,ymn,ymx,xpkp,xpkm,xpk,yye,rte1,rte2,
     . psie1,psie2,rhopmn,rhote2,rhote1,rhotmn
c common/external functions/variables
      real(wp_) :: rhotp,rhotm,ypkp,ypkm
      real(wp_) :: frhotor
c
      common/jmxmn/rhotp,rhotm,ypkp,ypkm
c
      call vmaxmini(yy,nd,ymn,ymx,imn,imx)
      ypk=0.0_wp_
      xmx=xx(imx)
      xmn=xx(imn)
      if (abs(ymx).gt.abs(ymn)) then
        ipk=imx
        ypkp=ymx
        xpkp=xmx
        if(abs(ymn/ymx).lt.1.0e-2_wp_) ymn=0.0_wp_
        ypkm=ymn
        xpkm=xmn
      else
        ipk=imn
        ypkp=ymn
        xpkp=xmn
        if(abs(ymx/ymn).lt.1.0e-2_wp_) ymx=0.0_wp_
        ypkm=ymx
        xpkm=xmx
      end if
      if(xpkp.gt.0.0_wp_) then
        xpk=xpkp
        ypk=ypkp
        yye=ypk*emn1
        call locatex(yy,nd,1,ipk,yye,ie1)
        if(ie1.gt.0.and.ie1.lt.nd) then
          call intlin(yy(ie1),xx(ie1),yy(ie1+1),xx(ie1+1),yye,rte1)
        else
          rte1=0.0_wp_
        end if
        call locatex(yy,nd,ipk,nd,yye,ie2)
        if(ie2.gt.0.and.ie2.lt.nd) then
          call intlin(yy(ie2),xx(ie2),yy(ie2+1),xx(ie2+1),yye,rte2)
        else
          rte2=0.0_wp_
        end if
      else
        ipk=2
        xpk=xx(2)
        ypk=yy(2)
        rte1=0.0_wp_
        yye=ypk*emn1
        call locate(yy,nd,yye,ie2)
        call intlin(yy(ie2),xx(ie2),yy(ie1+2),xx(ie1+2),yye,rte2)
      end if
c
      ipk=1
      if(ymx.ne.0.and.ymn.ne.0) ipk=2
c
      drhop=0.0_wp_
      drhot=0.0_wp_
      psie1=0.0_wp_
      psie2=1.0_wp_
      rhopmx=1.0_wp_
      rhopmn=0.0_wp_
      if (ie1.gt.0.or.ie2.gt.0) then
        if(ipec.eq.0) then
          rhopmx=sqrt(xpkp)
          rhopmn=sqrt(xpkm)
          psie2=rte2
          psie1=rte1
          drhop=sqrt(rte2)-sqrt(rte1)
        else
          rhopmx=xpkp
          rhopmn=xpkm
          drhop=rte2-rte1
          psie2=rte2**2
          psie1=rte1**2
        end if
      end if
c
        rhotmx=frhotor(rhopmx**2)
        rhotmn=frhotor(rhopmn**2)
        rhote2=frhotor(psie2)
        rhote1=frhotor(psie1)
        drhot=rhote2-rhote1
c
      if(ipk.eq.2) then
        drhop=-drhop
        drhot=-drhot
      end if

      ypk=ypkp
      rhotp=rhotmx
      rhotm=rhotmn
c
      return
      end
c
c
c
      subroutine polarcold(exf,eyif,ezf,elf,etf)
      use const_and_precisions, only : wp_
      implicit none
c arguments
      real(wp_) :: exf,eyif,ezf,elf,etf
c local variables
      real(wp_) :: anpl2,anpr2,an2,yg2,dy2,aa,e3,qq,p
c common/external functions/variables
      real(wp_) anpl,anpr,xg,yg,sox
c
      common/nplr/anpl,anpr
      common/xgxg/xg
      common/ygyg/yg
      common/mode/sox
c
c     dcold dispersion
c     dielectric tensor (transposed)
c
c      exf=0.0_wp_
c      eyif=0.0_wp_
c      ezf=0.0_wp_
c      if(xg.le.0) return
c
      anpl2=anpl*anpl
      anpr2=anpr*anpr
      an2=anpl2+anpr2
      yg2=yg**2
      dy2=1.0_wp_-yg2
      aa=1.0_wp_-xg-yg2
      e3=1.0_wp_-xg
c
      if(xg.gt.0.0_wp_) then
        if (anpl.ne.0.0_wp_) then
          qq=xg*yg/(an2*dy2-aa)
          p=(anpr2-e3)/(anpl*anpr)
          ezf=1.0_wp_/sqrt(1.0_wp_+p*p*(1.0_wp_+qq*qq))
          exf=p*ezf
          eyif=qq*exf
        else
          if(sox.lt.0.0_wp_) then
            ezf=1
            exf=0
            eyif=0
          else
            ezf=0
            qq=-aa/(xg*yg)
            exf=1.0_wp_/sqrt(1.0_wp_+qq*qq)
            eyif=qq*exf
          end if
        end if
        elf=(anpl*ezf+anpr*exf)/sqrt(an2)
        etf=sqrt(1.0_wp_-elf*elf)
      else
        if(sox.lt.0.0_wp_) then
          ezf=1
          exf=0
          eyif=0
        else
          ezf=0
          exf=0.0_wp_
          eyif=1.0_wp_
        end if
        elf=0
        etf=1
      end if
c
      return
      end
c
c
c
      subroutine pol_limit(ext,eyt)
      use const_and_precisions, only : wp_,ui=>im,pi
      implicit none
c arguments
      complex(wp_) :: ext,eyt
c local variables
      real(wp_) :: anx,any,anz,xe2om,ye2om,xe2xm,ye2xm,an2,an,anxy,
     . sngam,csgam,csg2,sng2,ffo,ffx,ffo2,ffx2,deno,denx,anpl2,dnl,
     . del0,gam
      complex(wp_) :: exom,eyom,exxm,eyxm
c common/external functions/variables
      real(wp_) :: anpl,anpr,yg,sox
      real(wp_), dimension(3) :: bv,anv
c
      common/anv/anv
      common/nplr/anpl,anpr
      common/ygyg/yg
      common/bb/bv
      common/mode/sox
c
      anx=anv(1)
      any=anv(2)
      anz=anv(3)
      anpl2=anpl*anpl
      dnl=1.0_wp_-anpl2
      del0=sqrt(dnl**2+4.0_wp_*anpl2/yg**2)
      ffo=0.5_wp_*yg*(dnl+del0)
      ffx=0.5_wp_*yg*(dnl-del0)
      an2=anx*anx+any*any+anz*anz
      an=sqrt(an2)
      anxy=sqrt(anx*anx+any*any)
      sngam=(anz*anpl-an2*bv(3))/(an*anxy*anpr)
      csgam=-(any*bv(1)-anx*bv(2))/anxy/anpr
      csg2=csgam**2
      sng2=sngam**2
      ffo2=ffo*ffo
      ffx2=ffx*ffx
      deno=ffo2+anpl2
      denx=ffx2+anpl2
      xe2om=(ffo2*csg2+anpl2*sng2)/deno
      ye2om=(ffo2*sng2+anpl2*csg2)/deno
      xe2xm=(ffx2*csg2+anpl2*sng2)/denx
      ye2xm=(ffx2*sng2+anpl2*csg2)/denx
c
      exom=(ffo*csgam-ui*anpl*sngam)/sqrt(deno)
      eyom=(-ffo*sngam-ui*anpl*csgam)/sqrt(deno)
c
      exxm=(ffx*csgam-ui*anpl*sngam)/sqrt(denx)
      eyxm=(-ffx*sngam-ui*anpl*csgam)/sqrt(denx)
c
      if (sox.lt.0.0_wp_) then
        ext=exom
        eyt=eyom
      else
        ext=exxm
        eyt=eyxm
      endif

      gam=atan(sngam/csgam)*180.0_wp_/pi

      return
      end
c
c
c
      subroutine stokes(ext,eyt,qq,uu,vv)
      use const_and_precisions, only : wp_
      implicit none
c arguments
      complex(wp_) :: ext,eyt
      real(wp_) :: qq,uu,vv
c
      qq=abs(ext)**2-abs(eyt)**2
      uu=2.0_wp_*dble(ext*dconjg(eyt))
      vv=2.0_wp_*dimag(ext*dconjg(eyt))
c
      end subroutine stokes
c
c
c      
      subroutine polellipse(qq,uu,vv,psipol,chipol)
      use const_and_precisions, only : wp_,pi
      implicit none
c arguments
      real(wp_) :: qq,uu,vv,psipol,chipol
c
c      real*8 llm,aa,bb,ell
c      llm=sqrt(qq**2+uu**2)
c      aa=sqrt((1+llm)/2.0_wp_)
c      bb=sqrt((1-llm)/2.0_wp_)
c      ell=bb/aa
      psipol=0.5_wp_*atan2(uu,qq)*180.0_wp_/pi
      chipol=0.5_wp_*asin(vv)*180.0_wp_/pi
c
      end subroutine polellipse
c
c
c
      logical function inside_plasma(rrm,zzm)
      use const_and_precisions, only : wp_
      use graydata_flags, only : iequil
      use graydata_par, only : psdbnd
      use interp_eqprof, only : zbmin,zbmax
      implicit none
c arguments
      real(wp_) :: rrm,zzm
c common/external functions/variables
      real(wp_) :: psinv
c
      common/psival/psinv
c
      if(iequil.eq.1) then
        call equian(rrm,zzm)
      else
        call equinum_psi(rrm,zzm)
      end if
c
      if (psinv.ge.0.0_wp_.and.psinv.lt.psdbnd) then
        if (psinv.lt.1.0_wp_.and.zzm.lt.zbmin.or.zzm.gt.zbmax) then
          inside_plasma=.false.
        else
          inside_plasma=.true.
        end if
      else
        inside_plasma=.false.
      end if
c
      end function inside_plasma
c
c
c
      subroutine wall_refl(xv,anv,ext,eyt,xvrfl,anvrfl,extr,eytr,walln,
     .                     irfl)
      use const_and_precisions, only : wp_,ui=>im,pi
      use reflections, only : inters_linewall,inside
      use interp_eqprof, only : rlim,zlim,nlim
      implicit none
c arguments
      integer :: irfl
      real(wp_), dimension(3) :: xv,anv,xvrfl,anvrfl,walln
      complex(wp_) :: ext,eyt,extr,eytr
c local variables
      real(wp_) :: smax,rrm,zzm
      real(wp_), dimension(3) :: anv0,vv1,vv2,vv3
      complex(wp_) :: eztr
      complex(wp_), dimension(3) :: evin,evrfl 
c
      anv0=anv/sqrt(anv(1)**2+anv(2)**2+anv(3)**2)
      rrm=1.0e-2_wp_*sqrt(xv(1)**2+xv(2)**2)
      zzm=1.0e-2_wp_*xv(3)
c
c     computation of reflection coordinates and normal to the wall
      call inters_linewall(xv/1.0e2_wp_,anv0,rlim(1:nlim),zlim(1:nlim),
     .                     nlim,smax,walln)
      smax=smax*1.0e2_wp_
      xvrfl=xv+smax*anv0
      irfl=1
      if (.not.inside(rlim,zlim,nlim,rrm,zzm)) then
        ! first wall interface is outside-inside
        if (dot_product(walln,walln)<tiny(walln)) then
          ! wall never hit
          xvrfl=xv
          anvrfl=anv0
          extr=ext
          eytr=eyt
          irfl=0
          return
        end if
        ! search second wall interface (inside-outside)
        call inters_linewall(xvrfl/1.0e2_wp_,anv0,rlim(1:nlim),
     .                       zlim(1:nlim),nlim,smax,walln)
        smax=smax*1.0e2_wp_
        xvrfl=xvrfl+smax*anv0
        irfl=2
      end if
c
c     rotation matrix from local to lab frame
      vv1(1)=anv0(2)
      vv1(2)=-anv0(1)
      vv1(3)=0.0_wp_
      vv2(1)=anv0(1)*anv0(3)
      vv2(2)=anv0(2)*anv0(3)
      vv2(3)=-anv0(1)*anv0(1)-anv0(2)*anv0(2)
      vv1=vv1/sqrt(vv1(1)**2+vv1(2)**2+vv1(3)**2)
      vv2=vv2/sqrt(vv2(1)**2+vv2(2)**2+vv2(3)**2)
      vv3=anv0
c
      evin=ext*vv1+eyt*vv2
c     wave vector and electric field after reflection in lab frame
      anvrfl=anv0-2.0_wp_*
     .       (anv0(1)*walln(1)+anv0(2)*walln(2)+anv0(3)*walln(3))*walln
      evrfl=-evin+2.0_wp_*
     .       (evin(1)*walln(1)+evin(2)*walln(2)+evin(3)*walln(3))*walln
c
      vv1(1)=anvrfl(2)
      vv1(2)=-anvrfl(1)
      vv1(3)=0.0_wp_
      vv2(1)=anvrfl(1)*anvrfl(3)
      vv2(2)=anvrfl(2)*anvrfl(3)
      vv2(3)=-anvrfl(1)*anvrfl(1)-anvrfl(2)*anvrfl(2)
      vv1=vv1/sqrt(vv1(1)**2+vv1(2)**2+vv1(3)**2)
      vv2=vv2/sqrt(vv2(1)**2+vv2(2)**2+vv2(3)**2)
      vv3=anvrfl/sqrt(anvrfl(1)**2+anvrfl(2)**2+anvrfl(3)**2)
c
      extr=dot_product(vv1,evrfl)
      eytr=dot_product(vv2,evrfl)
      eztr=dot_product(vv3,evrfl)
c
      return
      end
c
c
c
      subroutine vacuum_rt(xvstart,anv,xvend,ivac)
      use const_and_precisions, only : wp_
      use reflections, only : inters_linewall,inside
      use graydata_flags, only : dst
      use interp_eqprof, only : rlim,zlim,nlim
      implicit none
c arguments
      integer :: ivac
      real(wp_), dimension(3) :: xvstart,anv,xvend
c local variables
      integer :: i
      real(wp_) :: st,rrm,zzm,smax
      real(wp_), dimension(3) :: walln,xv0,anv0
      logical :: plfound
c common/external functions/variables
      real(wp_) :: dstvac
      logical :: inside_plasma
c
      external inside_plasma
c
      common/dstvac/dstvac
c
c     ivac=1 plasma hit before wall reflection
c     ivac=2 wall hit before plasma
c     ivac=-1 vessel (and thus plasma) never crossed
      
c     the real argument associated to xvstart is in a common block
c     used by fwork!!!
      xv0=xvstart
      anv0=anv
      call inters_linewall(xv0/1.0e2_wp_,anv,rlim(1:nlim),zlim(1:nlim),
     .                     nlim,smax,walln)
      smax=smax*1.0e2_wp_
      rrm=1.0e-2_wp_*sqrt(xv0(1)**2+xv0(2)**2)
      zzm=1.0e-2_wp_*xv0(3)
      if (.not.inside(rlim,zlim,nlim,rrm,zzm)) then
        ! first wall interface is outside-inside
        if (dot_product(walln,walln)<tiny(walln)) then
          ! wall never hit
          dstvac=0.0_wp_
          xvend=xv0
          ivac=-1
          return
        end if
        ! search second wall interface (inside-outside)
        st=smax
        xvend=xv0+st*anv
        call inters_linewall(xvend/1.0e2_wp_,anv,rlim(1:nlim),
     .                       zlim(1:nlim),nlim,smax,walln)
        smax=smax*1.0e2_wp_+st
      end if
      i=0
      do
        st=i*dst
        xvend=xv0+st*anv0
        rrm=1.0e-2_wp_*sqrt(xvend(1)**2+xvend(2)**2)
        zzm=1.0e-2_wp_*xvend(3)
        plfound=inside_plasma(rrm,zzm)
        if (st.ge.smax.or.plfound) exit
        i=i+1
      end do
      
      if (plfound) then
        ivac=1
        dstvac=st
      else
        ivac=2
        dstvac=smax
        xvend=xv0+smax*anv0
      end if

c     the real argument associated to xvstart is in a common block
c     used by fwork!!!
      xvstart=xv0
      anv=anv0
      
      return
      end

